@@ -22,6 +22,11 @@
 #include "EbSvtAv1ErrorCodes.h"
 #include "EbAppInputy4m.h"
 #include "EbTime.h"
+
+#ifndef _WIN32
+#include <sys/mman.h>
+#endif
+
 /***************************************
  * Macros
  ***************************************/
@@ -50,8 +55,6 @@ void log_error_output(FILE *error_log_file, uint32_t error_code) {
         fprintf(error_log_file, "Error: No more than 6 SAO types\n");
         break;
 
-
-
         // EB_ENC_ERRORS:
     case EB_ENC_ROB_OF_ERROR:
         fprintf(error_log_file, "Error: Recon Output Buffer Overflow!\n");
@@ -111,11 +114,73 @@ void log_error_output(FILE *error_log_file, uint32_t error_code) {
     return;
 }
 
+//retunrs the offset from the file start
+int64_t get_mmap_offset(EbConfig *config, uint64_t frame_size) {
+    int64_t offset;
+    if (config->y4m_input)
+        offset = config->mmap.y4m_seq_hdr +
+            (config->mmap.file_frame_it + 1) * config->mmap.y4m_frm_hdr +
+            config->mmap.file_frame_it * frame_size;
+    else
+        offset = config->mmap.file_frame_it * frame_size;
+
+    return offset;
+}
+/* returns a RAM address from a memory mapped file  */
+void *svt_mmap(MemMapFile *h, int64_t offset, int64_t size) {
+    if (offset + size > h->file_size)
+        return NULL;
+
+    int align = offset & h->align_mask;
+    offset -= align;
+    size += align;
+#ifndef _WIN32
+    uint8_t *base = mmap(NULL, size, PROT_READ, MAP_PRIVATE, h->fd, offset);
+    //printf(" base=%p \n", base);
+    if (base != MAP_FAILED)
+        return base + align;
+
+        //else
+        //    printf(" error base=%p \n",base);
+#endif
+
+    return NULL;
+}
+/* release  memory mapped file  */
+void svt_munmap(MemMapFile *h, void *addr, int64_t size) {
+    void *base = (void *)((intptr_t)addr & ~h->align_mask);
+#ifndef _WIN32
+    munmap(base, size + (intptr_t)addr - (intptr_t)base);
+#endif
+}
+/* release  memory mapped file  */
+static void release_memory_mapped_file(EbConfig *config, uint8_t is_16bit,
+                                       EbBufferHeaderType *header_ptr) {
+    const uint32_t input_padded_width  = config->input_padded_width;
+    const uint32_t input_padded_height = config->input_padded_height;
+    uint64_t luma_read_size = (uint64_t)input_padded_width * input_padded_height
+        << (config->config.compressed_ten_bit_format ? 0 : is_16bit);
+    const uint8_t  color_format = config->config.encoder_color_format;
+    EbSvtIOFormat *input_ptr    = (EbSvtIOFormat *)header_ptr->p_buffer;
+    svt_munmap(&config->mmap, input_ptr->luma, luma_read_size);
+    svt_munmap(&config->mmap, input_ptr->cb, luma_read_size >> (3 - color_format));
+    svt_munmap(&config->mmap, input_ptr->cr, luma_read_size >> (3 - color_format));
+
+    if (config->config.compressed_ten_bit_format) {
+        uint32_t nbit_luma_read_size   = (input_padded_width / 4) * input_padded_height;
+        uint32_t nbit_chroma_read_size = nbit_luma_read_size >> (3 - color_format);
+
+        svt_munmap(&config->mmap, input_ptr->luma_ext, nbit_luma_read_size);
+        svt_munmap(&config->mmap, input_ptr->cb_ext, nbit_chroma_read_size >> (3 - color_format));
+        svt_munmap(&config->mmap, input_ptr->cr_ext, nbit_chroma_read_size >> (3 - color_format));
+    }
+}
+
 void read_input_frames(EbConfig *config, uint8_t is_16bit, EbBufferHeaderType *header_ptr) {
     const uint32_t input_padded_width  = config->input_padded_width;
     const uint32_t input_padded_height = config->input_padded_height;
-    FILE *         input_file          = config->input_file;
-    EbSvtIOFormat *input_ptr           = (EbSvtIOFormat *)header_ptr->p_buffer;
+    FILE *         input_file = config->input_file;
+    EbSvtIOFormat *input_ptr  = (EbSvtIOFormat *)header_ptr->p_buffer;
 
     const uint8_t color_format  = config->config.encoder_color_format;
     const uint8_t subsampling_x = (color_format == EB_YUV444 ? 1 : 2) - 1;
@@ -131,11 +196,20 @@ void read_input_frames(EbConfig *config, uint8_t is_16bit, EbBufferHeaderType *h
                 input_padded_width, input_padded_height, color_format, is_16bit);
 
             header_ptr->n_filled_len = 0;
-            /* if input is a y4m file, read next line which contains "FRAME" */
-            if (config->y4m_input == EB_TRUE)
-                read_y4m_frame_delimiter(config->input_file, config->error_log_file);
+
+            if (config->mmap.enable) {
+                if (config->y4m_input == EB_TRUE && config->processed_frame_count == 0) {
+                    read_and_compute_y4m_frame_delimiter(
+                        config->input_file, config->error_log_file, &config->mmap.y4m_frm_hdr);
+                }
+            } else {
+                /* if input is a y4m file, read next line which contains "FRAME" */
+                if (config->y4m_input == EB_TRUE)
+                    read_y4m_frame_delimiter(config->input_file, config->error_log_file);
+            }
             uint64_t luma_read_size = (uint64_t)input_padded_width * input_padded_height
                 << is_16bit;
+            uint32_t chroma_read_size = ((uint32_t)luma_read_size >> (3 - color_format));
             uint8_t *eb_input_ptr = input_ptr->luma;
             if (!config->y4m_input && config->processed_frame_count == 0 &&
                 (config->input_file == stdin || config->input_file_is_fifo)) {
@@ -146,27 +220,61 @@ void read_input_frames(EbConfig *config, uint8_t is_16bit, EbBufferHeaderType *h
                 header_ptr->n_filled_len += (uint32_t)fread(
                     eb_input_ptr, 1, luma_read_size - YUV4MPEG2_IND_SIZE, input_file);
             } else {
-                header_ptr->n_filled_len += (uint32_t)fread(
-                    input_ptr->luma, 1, luma_read_size, input_file);
+                if (config->mmap.enable) {
+                    int64_t offset  = get_mmap_offset(config, read_size);
+                    input_ptr->luma = svt_mmap(&config->mmap, offset, luma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->luma ? (uint32_t)luma_read_size : 0);
+                } else
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->luma, 1, luma_read_size, input_file);
             }
-            header_ptr->n_filled_len += (uint32_t)fread(
-                input_ptr->cb, 1, luma_read_size >> (3 - color_format), input_file);
-            header_ptr->n_filled_len += (uint32_t)fread(
-                input_ptr->cr, 1, luma_read_size >> (3 - color_format), input_file);
 
-            if (read_size != header_ptr->n_filled_len) {
-                fseek(input_file, 0, SEEK_SET);
-                if (config->y4m_input == EB_TRUE) {
-                    read_and_skip_y4m_header(config->input_file);
-                    read_y4m_frame_delimiter(config->input_file, config->error_log_file);
-                }
-                header_ptr->n_filled_len = (uint32_t)fread(
-                    input_ptr->luma, 1, luma_read_size, input_file);
+            if (config->mmap.enable) {
+                int64_t offset = get_mmap_offset(config, read_size);
+
+                offset += luma_read_size;
+                input_ptr->cb = svt_mmap(&config->mmap, offset, chroma_read_size);
+                header_ptr->n_filled_len += (input_ptr->cb ? chroma_read_size : 0);
+
+                offset += chroma_read_size;
+                input_ptr->cr = svt_mmap(&config->mmap, offset, chroma_read_size);
+                header_ptr->n_filled_len += (input_ptr->cr ? chroma_read_size : 0);
+            } else {
                 header_ptr->n_filled_len += (uint32_t)fread(
                     input_ptr->cb, 1, luma_read_size >> (3 - color_format), input_file);
                 header_ptr->n_filled_len += (uint32_t)fread(
                     input_ptr->cr, 1, luma_read_size >> (3 - color_format), input_file);
             }
+
+            if (read_size != header_ptr->n_filled_len) {
+                if (config->mmap.enable) {
+                    config->mmap.file_frame_it = 0;
+
+                    int64_t offset  = get_mmap_offset(config, read_size);
+                    input_ptr->luma = svt_mmap(&config->mmap, offset, luma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->luma ? (uint32_t)luma_read_size : 0);
+
+                    offset += luma_read_size;
+                    input_ptr->cb = svt_mmap(&config->mmap, offset, chroma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->cb ? chroma_read_size : 0);
+
+                    offset += chroma_read_size;
+                    input_ptr->cr = svt_mmap(&config->mmap, offset, chroma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->cr ? chroma_read_size : 0);
+                } else if (!config->input_file_is_fifo) {
+                    fseek(input_file, 0, SEEK_SET);
+                    if (config->y4m_input == EB_TRUE) {
+                        read_and_skip_y4m_header(config->input_file);
+                        read_y4m_frame_delimiter(config->input_file, config->error_log_file);
+                    }
+                    header_ptr->n_filled_len = (uint32_t)fread(
+                        input_ptr->luma, 1, luma_read_size, input_file);
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->cb, 1, luma_read_size >> (3 - color_format), input_file);
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->cr, 1, luma_read_size >> (3 - color_format), input_file);
+                }
+            }
         } else {
             assert(is_16bit == 1 && config->config.compressed_ten_bit_format == 1);
             // 10-bit Compressed Unpacked Mode
@@ -174,35 +282,44 @@ void read_input_frames(EbConfig *config, uint8_t is_16bit, EbBufferHeaderType *h
             const uint32_t chroma_read_size      = luma_read_size >> (3 - color_format);
             const uint32_t nbit_luma_read_size   = (input_padded_width / 4) * input_padded_height;
             const uint32_t nbit_chroma_read_size = nbit_luma_read_size >> (3 - color_format);
-
+            read_size = luma_read_size + nbit_luma_read_size +
+                2 * (chroma_read_size + nbit_chroma_read_size);
             // Fill the buffer with a complete frame
             header_ptr->n_filled_len = 0;
 
-            header_ptr->n_filled_len += (uint32_t)fread(
-                input_ptr->luma, 1, luma_read_size, input_file);
-            header_ptr->n_filled_len += (uint32_t)fread(
-                input_ptr->cb, 1, chroma_read_size, input_file);
-            header_ptr->n_filled_len += (uint32_t)fread(
-                input_ptr->cr, 1, chroma_read_size, input_file);
+            if (config->mmap.enable) {
+                int64_t offset  = get_mmap_offset(config, read_size);
+                input_ptr->luma = svt_mmap(&config->mmap, offset, luma_read_size);
+                header_ptr->n_filled_len += (input_ptr->luma ? (uint32_t)luma_read_size : 0);
 
-            header_ptr->n_filled_len += (uint32_t)fread(
-                input_ptr->luma_ext, 1, nbit_luma_read_size, input_file);
-            header_ptr->n_filled_len += (uint32_t)fread(
-                input_ptr->cb_ext, 1, nbit_chroma_read_size, input_file);
-            header_ptr->n_filled_len += (uint32_t)fread(
-                input_ptr->cr_ext, 1, nbit_chroma_read_size, input_file);
+                offset += luma_read_size;
+                input_ptr->cb = svt_mmap(&config->mmap, offset, chroma_read_size);
+                header_ptr->n_filled_len += (input_ptr->cb ? chroma_read_size : 0);
 
-            read_size = luma_read_size + nbit_luma_read_size +
-                2 * (chroma_read_size + nbit_chroma_read_size);
+                offset += chroma_read_size;
+                input_ptr->cr = svt_mmap(&config->mmap, offset, chroma_read_size);
+                header_ptr->n_filled_len += (input_ptr->cr ? chroma_read_size : 0);
 
-            if (read_size != header_ptr->n_filled_len) {
-                fseek(input_file, 0, SEEK_SET);
+                offset += chroma_read_size;
+                input_ptr->luma_ext = svt_mmap(&config->mmap, offset, nbit_luma_read_size);
+                header_ptr->n_filled_len += (input_ptr->luma_ext ? (uint32_t)nbit_luma_read_size
+                                                                 : 0);
+
+                offset += nbit_luma_read_size;
+                input_ptr->cb_ext = svt_mmap(&config->mmap, offset, nbit_chroma_read_size);
+                header_ptr->n_filled_len += (input_ptr->cb_ext ? nbit_chroma_read_size : 0);
+
+                offset += nbit_chroma_read_size;
+                input_ptr->cr_ext = svt_mmap(&config->mmap, offset, nbit_chroma_read_size);
+                header_ptr->n_filled_len += (input_ptr->cr_ext ? nbit_chroma_read_size : 0);
+            } else {
                 header_ptr->n_filled_len += (uint32_t)fread(
                     input_ptr->luma, 1, luma_read_size, input_file);
                 header_ptr->n_filled_len += (uint32_t)fread(
                     input_ptr->cb, 1, chroma_read_size, input_file);
                 header_ptr->n_filled_len += (uint32_t)fread(
                     input_ptr->cr, 1, chroma_read_size, input_file);
+
                 header_ptr->n_filled_len += (uint32_t)fread(
                     input_ptr->luma_ext, 1, nbit_luma_read_size, input_file);
                 header_ptr->n_filled_len += (uint32_t)fread(
@@ -210,6 +327,50 @@ void read_input_frames(EbConfig *config, uint8_t is_16bit, EbBufferHeaderType *h
                 header_ptr->n_filled_len += (uint32_t)fread(
                     input_ptr->cr_ext, 1, nbit_chroma_read_size, input_file);
             }
+            if (read_size != header_ptr->n_filled_len) {
+                if (config->mmap.enable) {
+                    config->mmap.file_frame_it = 0;
+
+                    int64_t offset  = get_mmap_offset(config, read_size);
+                    input_ptr->luma = svt_mmap(&config->mmap, offset, luma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->luma ? (uint32_t)luma_read_size : 0);
+
+                    offset += luma_read_size;
+                    input_ptr->cb = svt_mmap(&config->mmap, offset, chroma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->cb ? chroma_read_size : 0);
+
+                    offset += chroma_read_size;
+                    input_ptr->cr = svt_mmap(&config->mmap, offset, chroma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->cr ? chroma_read_size : 0);
+
+                    offset += chroma_read_size;
+                    input_ptr->luma_ext = svt_mmap(&config->mmap, offset, nbit_luma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->luma_ext ? (uint32_t)nbit_luma_read_size
+                                                                     : 0);
+
+                    offset += nbit_luma_read_size;
+                    input_ptr->cb_ext = svt_mmap(&config->mmap, offset, nbit_chroma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->cb_ext ? nbit_chroma_read_size : 0);
+
+                    offset += nbit_chroma_read_size;
+                    input_ptr->cr_ext = svt_mmap(&config->mmap, offset, nbit_chroma_read_size);
+                    header_ptr->n_filled_len += (input_ptr->cr_ext ? nbit_chroma_read_size : 0);
+                } else {
+                    fseek(input_file, 0, SEEK_SET);
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->luma, 1, luma_read_size, input_file);
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->cb, 1, chroma_read_size, input_file);
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->cr, 1, chroma_read_size, input_file);
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->luma_ext, 1, nbit_luma_read_size, input_file);
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->cb_ext, 1, nbit_chroma_read_size, input_file);
+                    header_ptr->n_filled_len += (uint32_t)fread(
+                        input_ptr->cr_ext, 1, nbit_chroma_read_size, input_file);
+                }
+            }
         }
 
         if (feof(input_file) != 0) {
@@ -318,7 +479,7 @@ static long get_next_qp_from_qp_file(FILE *const qp_file, int *const qp_read_fro
     return qp;
 }
 
-static unsigned char send_qp_on_the_fly(FILE *const qp_file, uint8_t *use_qp_file) {
+static unsigned char send_qp_on_the_fly(FILE *const qp_file, EbBool *use_qp_file) {
     long tmp_qp            = 0;
     int  qp_read_from_file = 0;
 
@@ -374,9 +535,9 @@ void process_input_buffer(EncChannel *channel) {
 
     AppExitConditionType return_value = APP_ExitConditionNone;
 
-    const uint8_t color_format         = config->config.encoder_color_format;
-    const int64_t input_padded_width   = config->input_padded_width;
-    const int64_t input_padded_height  = config->input_padded_height;
+    const uint8_t color_format = config->config.encoder_color_format;
+    const int64_t input_padded_width = config->input_padded_width;
+    const int64_t input_padded_height = config->input_padded_height;
     const int64_t frames_to_be_encoded = config->frames_to_be_encoded;
     int64_t       total_bytes_to_process_count;
     int64_t       remaining_byte_count;
@@ -407,7 +568,9 @@ void process_input_buffer(EncChannel *channel) {
             // Update the context parameters
             config->processed_byte_count += header_ptr->n_filled_len;
             header_ptr->p_app_private = (EbPtr)NULL;
-            config->frames_encoded    = (int32_t)(++config->processed_frame_count);
+
+            config->mmap.file_frame_it++;
+            config->frames_encoded = (int32_t)(++config->processed_frame_count);
 
             // Configuration parameters changed on the fly
             if (config->config.use_qp_file && config->qp_file)
@@ -420,9 +583,11 @@ void process_input_buffer(EncChannel *channel) {
             header_ptr->pic_type = EB_AV1_INVALID_PICTURE;
             header_ptr->flags    = 0;
             header_ptr->metadata = NULL;
-
             // Send the picture
             svt_av1_enc_send_picture(component_handle, header_ptr);
+
+            if (config->mmap.enable)
+                release_memory_mapped_file(config, is_16bit, header_ptr);
         }
 
         if ((config->processed_frame_count == (uint64_t)config->frames_to_be_encoded) ||
@@ -435,7 +600,6 @@ void process_input_buffer(EncChannel *channel) {
             header_ptr->p_buffer      = NULL;
             header_ptr->pic_type      = EB_AV1_INVALID_PICTURE;
             header_ptr->metadata      = NULL;
-
             svt_av1_enc_send_picture(component_handle, header_ptr);
         }
 
@@ -591,8 +755,8 @@ void process_output_statistics_buffer(EbBufferHeaderType *header_ptr, EbConfig *
                 (double)cb_sse / (source_width / 2 * source_height / 2),
                 (double)cr_sse / (source_width / 2 * source_height / 2),
                 luma_ssim,
-                cr_ssim,
                 cb_ssim,
+                cr_ssim,
                 (int)picture_stream_size);
     }
 
@@ -683,7 +847,7 @@ void process_output_stream_buffer(EncChannel *channel, EncApp *enc_app, int32_t
             svt_av1_enc_release_out_buffer(&header_ptr);
 
             if (flags & EB_BUFFERFLAG_EOS) {
-                if (config->config.rc_firstpass_stats_out) {
+                if (config->config.pass == ENC_FIRST_PASS) {
                     SvtAv1FixedBuf first_pass_stat;
                     EbErrorType    ret = svt_av1_enc_get_stream_info(
                         component_handle,
@@ -706,8 +870,16 @@ void process_output_stream_buffer(EncChannel *channel, EncApp *enc_app, int32_t
                         }
                     }
                 }
-            }
+                if (config->config.pass == ENC_SECOND_PASS) {
 
+                    if (config->output_stat_file && config->config.rc_stats_buffer.buf) {
+                        fwrite(config->config.rc_stats_buffer.buf,
+                               1,
+                               config->config.rc_stats_buffer.sz,
+                               config->output_stat_file);
+                    }
+                }
+            }
             ++*frame_count;
             const double fps = (double)*frame_count / config->performance_context.total_encode_time;
             const double frame_rate = config->config.frame_rate_numerator &&
