@@ -17,7 +17,6 @@
 #include "EbPictureBufferDesc.h"
 #include "EbAv1Structs.h"
 
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -188,6 +187,10 @@ typedef struct RestorationUnitInfo {
     SgrprojInfo     sgrproj_info;
 } RestorationUnitInfo;
 
+typedef struct WienerUnitInfo {
+    RestorationType restoration_type;
+    WienerInfo      wiener_info;
+} WienerUnitInfo;
 typedef struct Av1PixelRect {
     int32_t left, top, right, bottom;
 } Av1PixelRect;
@@ -330,47 +333,6 @@ typedef void (*RestTileStartVisitor)(int32_t tile_row, int32_t tile_col, void *p
 //                                                  struct AV1Common *cm,
 //                                                  int32_t after_cdef);
 
-static const double tiny_near_zero = 1.0E-16;
-
-// Solves Ax = b, where x and b are column vectors of size nx1 and A is nxn
-static INLINE int32_t linsolve(int32_t n, double *A, int32_t stride, double *b, double *x) {
-    int32_t i, j, k;
-    double  c;
-    // Forward elimination
-    for (k = 0; k < n - 1; k++) {
-        // Bring the largest magitude to the diagonal position
-        for (i = n - 1; i > k; i--) {
-            if (fabs(A[(i - 1) * stride + k]) < fabs(A[i * stride + k])) {
-                for (j = 0; j < n; j++) {
-                    c                       = A[i * stride + j];
-                    A[i * stride + j]       = A[(i - 1) * stride + j];
-                    A[(i - 1) * stride + j] = c;
-                }
-                c        = b[i];
-                b[i]     = b[i - 1];
-                b[i - 1] = c;
-            }
-        }
-        for (i = k; i < n - 1; i++) {
-            if (fabs(A[k * stride + k]) < tiny_near_zero)
-                return 0;
-            c = A[(i + 1) * stride + k] / A[k * stride + k];
-            for (j = 0; j < n; j++) A[(i + 1) * stride + j] -= c * A[k * stride + j];
-            b[i + 1] -= c * b[k];
-        }
-    }
-    // Backward substitution
-    for (i = n - 1; i >= 0; i--) {
-        if (fabs(A[i * stride + i]) < tiny_near_zero)
-            return 0;
-        c = 0;
-        for (j = i + 1; j <= n - 1; j++) c += A[i * stride + j] * x[j];
-        x[i] = (b[i] - c) / A[i * stride + i];
-    }
-
-    return 1;
-}
-
 // Returns 1 if a superres upscaled frame is unscaled and 0 otherwise.
 static INLINE int32_t av1_superres_unscaled(const FrameSize *frm_size) {
     return (frm_size->frame_width == frm_size->superres_upscaled_width);
