@@ -60,15 +60,6 @@ uint64_t *        svt_dec_total_lib_memory;
 
 uint32_t svt_dec_lib_malloc_count = 0;
 
-//TODO: Should be removed! Check
-EbMemoryMapEntry *memory_map;
-uint32_t *        memory_map_index;
-uint64_t *        total_lib_memory;
-
-uint32_t lib_malloc_count    = 0;
-uint32_t lib_semaphore_count = 0;
-uint32_t lib_mutex_count     = 0;
-
 /* Track address for memory during change in resoultion*/
 EbMemoryMapEntry *memory_map_start_address;
 EbMemoryMapEntry *memory_map_end_address;
@@ -84,12 +75,9 @@ EbErrorType decode_multiple_obu(EbDecHandle *dec_handle_ptr, uint8_t **data, siz
                                 uint32_t is_annexb);
 
 static void dec_switch_to_real_time() {
-#ifndef _WIN32
-
-    struct sched_param schedParam = {.sched_priority = 99};
-
-    int32_t retValue = pthread_setschedparam(pthread_self(), SCHED_FIFO, &schedParam);
-    UNUSED(retValue);
+#if !defined(_WIN32) && !DISABLE_REALTIME
+    (void)pthread_setschedparam(
+        pthread_self(), SCHED_FIFO, &(struct sched_param){.sched_priority = 99});
 #endif
 }
 
@@ -194,7 +182,7 @@ int svt_dec_out_buf(EbDecHandle *dec_handle_ptr, EbBufferHeaderType *p_buffer) {
             out_img->cr_stride = wd;
             chroma_size        = size * ht * wd;
             break;
-        default: SVT_LOG("Unsupported colour format. \n"); return 0;
+        default: SVT_ERROR("Unsupported colour format.\n"); return 0;
         }
 
         /* FilmGrain module req. even dim. for internal operation */
@@ -202,10 +190,8 @@ int svt_dec_out_buf(EbDecHandle *dec_handle_ptr, EbBufferHeaderType *p_buffer) {
         out_img->width    = wd;
         out_img->height   = ht;
         if (out_img->bit_depth != (EbBitDepth)recon_picture_buf->bit_depth) {
-            SVT_LOG(
-                "Warning : Output bit depth conversion not supported."
-                " Output depth set to %d. ",
-                recon_picture_buf->bit_depth);
+            SVT_WARN("Output bit depth conversion not supported. Output depth set to %d.\n",
+                     recon_picture_buf->bit_depth);
             out_img->bit_depth = (EbBitDepth)recon_picture_buf->bit_depth;
         }
 
@@ -465,7 +451,9 @@ static EbErrorType init_svt_av1_decoder_handle(EbComponentType *hComponent) {
     SVT_LOG("SVT [build]  :\tunknown compiler");
 #endif
     SVT_LOG(" %zu bit\n", sizeof(void *) * 8);
+#if !REPRODUCIBLE_BUILDS
     SVT_LOG("LIB Build date: %s %s\n", __DATE__, __TIME__);
+#endif
     SVT_LOG("-------------------------------------------\n");
 
     dec_switch_to_real_time();
