@@ -29,19 +29,17 @@ void dec_init_intra_predictors_12b_internal(void) {
 /*TODO: Remove replication and harmonize with encoder after data str. harmonization */
 int32_t dec_get_filt_type(const PartitionInfo *part_info, int32_t plane) {
     int ab_sm, le_sm;
-
     if (plane == 0) {
         const BlockModeInfo *ab = part_info->above_mbmi;
         const BlockModeInfo *le = part_info->left_mbmi;
-        ab_sm                   = ab ? is_smooth(ab, plane) : 0;
-        le_sm                   = le ? is_smooth(le, plane) : 0;
+        ab_sm                   = ab ? is_smooth_dec(ab, plane) : 0;
+        le_sm                   = le ? is_smooth_dec(le, plane) : 0;
     } else {
         const BlockModeInfo *ab = part_info->chroma_above_mbmi;
         const BlockModeInfo *le = part_info->chroma_left_mbmi;
-        ab_sm                   = ab ? is_smooth(ab, plane) : 0;
-        le_sm                   = le ? is_smooth(le, plane) : 0;
+        ab_sm                   = ab ? is_smooth_dec(ab, plane) : 0;
+        le_sm                   = le ? is_smooth_dec(le, plane) : 0;
     }
-
     return (ab_sm || le_sm) ? 1 : 0;
 }
 
