@@ -29,7 +29,7 @@
 /* Search for the best luma+chroma strength to add as an option, knowing we
 already selected nb_strengths options. */
 uint64_t svt_search_one_dual_avx2(int *lev0, int *lev1, int nb_strengths,
-                                  uint64_t (**mse)[TOTAL_STRENGTHS], int sb_count, int start_gi,
+                                  uint64_t **mse[2], int sb_count, int start_gi,
                                   int end_gi) {
     DECLARE_ALIGNED(32, uint64_t, tot_mse[TOTAL_STRENGTHS][TOTAL_STRENGTHS]);
     uint64_t  best_tot_mse    = (uint64_t)1 << 62;
@@ -79,26 +79,32 @@ uint64_t svt_search_one_dual_avx2(int *lev0, int *lev1, int nb_strengths,
     return best_tot_mse;
 }
 
-static INLINE void mse_4x4_16bit_avx2(const uint16_t **src, const uint16_t *dst,
-                                      const int32_t dstride, __m256i *sum) {
-    const __m256i s    = _mm256_loadu_si256((const __m256i *)*src);
-    const __m256i d    = _mm256_setr_epi64x(*(uint64_t *)(dst + 0 * dstride),
-                                         *(uint64_t *)(dst + 1 * dstride),
+static INLINE void mse_4x4_16bit_2x_subsampled_avx2(const uint16_t **src, const uint16_t *dst,
+                                                    const int32_t dstride, __m256i *sum) {
+    const __m256i s = _mm256_loadu_si256((const __m256i *)*src);
+
+    // set every line to src so distortion will be 0
+    const __m256i d = _mm256_setr_epi64x(*(uint64_t *)(dst + 0 * dstride),
+                                         *(uint64_t *)(*src + 1 * 4),
                                          *(uint64_t *)(dst + 2 * dstride),
-                                         *(uint64_t *)(dst + 3 * dstride));
+                                         *(uint64_t *)(*src + 3 * 4));
+
     const __m256i diff = _mm256_sub_epi16(d, s);
     const __m256i mse  = _mm256_madd_epi16(diff, diff);
     *sum               = _mm256_add_epi32(*sum, mse);
+
     *src += 16;
 }
 
-static INLINE void mse_4x4_8bit_avx2(const uint8_t **src, const uint8_t *dst, const int32_t dstride,
-                                     __m256i *sum) {
+static INLINE void mse_4x4_8bit_2x_subsampled_avx2(const uint8_t **src, const uint8_t *dst,
+                                                   const int32_t dstride, __m256i *sum) {
     const __m128i s = _mm_loadu_si128((const __m128i *)*src);
+
+    // set every line to src so distortion will be 0
     const __m128i d = _mm_setr_epi32(*(uint32_t *)(dst + 0 * dstride),
-                                     *(uint32_t *)(dst + 1 * dstride),
+                                     *(uint32_t *)(*src + 1 * 4),
                                      *(uint32_t *)(dst + 2 * dstride),
-                                     *(uint32_t *)(dst + 3 * dstride));
+                                     *(uint32_t *)(*src + 3 * 4));
 
     const __m256i s_16 = _mm256_cvtepu8_epi16(s);
     const __m256i d_16 = _mm256_cvtepu8_epi16(d);
@@ -106,46 +112,103 @@ static INLINE void mse_4x4_8bit_avx2(const uint8_t **src, const uint8_t *dst, co
     const __m256i diff = _mm256_sub_epi16(d_16, s_16);
     const __m256i mse  = _mm256_madd_epi16(diff, diff);
     *sum               = _mm256_add_epi32(*sum, mse);
+
     *src += 16;
 }
 
-static INLINE void mse_8x2_16bit_avx2(const uint16_t **src, const uint16_t *dst,
-                                      const int32_t dstride, __m256i *sum) {
-    const __m256i s    = _mm256_loadu_si256((const __m256i *)*src);
-    const __m128i d0   = _mm_loadu_si128((const __m128i *)(dst + 0 * dstride));
-    const __m128i d1   = _mm_loadu_si128((const __m128i *)(dst + 1 * dstride));
-    const __m256i d    = _mm256_setr_m128i(d0, d1);
-    const __m256i diff = _mm256_sub_epi16(d, s);
-    const __m256i mse  = _mm256_madd_epi16(diff, diff);
-    *sum               = _mm256_add_epi32(*sum, mse);
-    *src += 16;
+static INLINE void mse_4xn_16bit_avx2(const uint16_t **src, const uint16_t *dst,
+                                      const int32_t dstride, __m256i *sum, uint8_t height,
+                                      uint8_t subsampling_factor) {
+    for (int32_t r = 0; r < height; r += 4 * subsampling_factor) {
+        const __m256i s = _mm256_setr_epi64x(
+            *(uint64_t *)(*src + 0 * 4),
+            *(uint64_t *)(*src + (1 * subsampling_factor) * 4),
+            *(uint64_t *)(*src + (2 * subsampling_factor) * 4),
+            *(uint64_t *)(*src +
+                          (3 * subsampling_factor) *
+                              4)); // don't add r * dstride b/c add it at end of loop iterations
+        const __m256i d = _mm256_setr_epi64x(
+            *(uint64_t *)(dst + r * dstride),
+            *(uint64_t *)(dst + (r + (1 * subsampling_factor)) * dstride),
+            *(uint64_t *)(dst + (r + (2 * subsampling_factor)) * dstride),
+            *(uint64_t *)(dst + (r + (3 * subsampling_factor)) * dstride));
+
+        const __m256i diff = _mm256_sub_epi16(d, s);
+        const __m256i mse  = _mm256_madd_epi16(diff, diff);
+        *sum               = _mm256_add_epi32(*sum, mse);
+
+        *src += 4 * 4 * subsampling_factor; // with * 4 rows per iter * subsampling
+    }
 }
 
-static INLINE void mse_8x2_8bit_avx2(const uint8_t **src, const uint8_t *dst, const int32_t dstride,
-                                     __m256i *sum) {
-    const __m128i s = _mm_loadu_si128((const __m128i *)*src);
-    const __m128i d = _mm_set_epi64x(*(uint64_t *)(dst + 1 * dstride),
-                                     *(uint64_t *)(dst + 0 * dstride));
+static INLINE void mse_4xn_8bit_avx2(const uint8_t **src, const uint8_t *dst, const int32_t dstride,
+                                     __m256i *sum, uint8_t height, uint8_t subsampling_factor) {
+    for (int32_t r = 0; r < height; r += 4 * subsampling_factor) {
+        const __m128i s = _mm_setr_epi32(
+            *(uint32_t *)(*src + 0 * 4),
+            *(uint32_t *)(*src + (1 * subsampling_factor) * 4),
+            *(uint32_t *)(*src + (2 * subsampling_factor) * 4),
+            *(uint32_t *)(*src +
+                          (3 * subsampling_factor) *
+                              4)); // don't add r * dstride b/c add it at end of loop iterations
+        const __m128i d = _mm_setr_epi32(
+            *(uint32_t *)(dst + r * dstride),
+            *(uint32_t *)(dst + (r + (1 * subsampling_factor)) * dstride),
+            *(uint32_t *)(dst + (r + (2 * subsampling_factor)) * dstride),
+            *(uint32_t *)(dst + (r + (3 * subsampling_factor)) * dstride));
 
-    const __m256i s_16 = _mm256_cvtepu8_epi16(s);
-    const __m256i d_16 = _mm256_cvtepu8_epi16(d);
+        const __m256i s_16 = _mm256_cvtepu8_epi16(s);
+        const __m256i d_16 = _mm256_cvtepu8_epi16(d);
 
-    const __m256i diff = _mm256_sub_epi16(d_16, s_16);
-    const __m256i mse  = _mm256_madd_epi16(diff, diff);
-    *sum               = _mm256_add_epi32(*sum, mse);
-    *src += 16;
+        const __m256i diff = _mm256_sub_epi16(d_16, s_16);
+        const __m256i mse  = _mm256_madd_epi16(diff, diff);
+        *sum               = _mm256_add_epi32(*sum, mse);
+
+        *src += 4 * 4 * subsampling_factor; // with * 4 rows per iter * subsampling
+    }
 }
 
-static INLINE void mse_8x4_16bit_avx2(const uint16_t **src, const uint16_t *dst,
-                                      const int32_t dstride, __m256i *sum) {
-    mse_8x2_16bit_avx2(src, dst + 0 * dstride, dstride, sum);
-    mse_8x2_16bit_avx2(src, dst + 2 * dstride, dstride, sum);
+static INLINE void mse_8xn_16bit_avx2(const uint16_t **src, const uint16_t *dst,
+                                      const int32_t dstride, __m256i *sum, uint8_t height,
+                                      uint8_t subsampling_factor) {
+    for (int32_t r = 0; r < height; r += 2 * subsampling_factor) {
+        const __m128i s0 = _mm_loadu_si128(
+            (const __m128i *)(*src +
+                              0 * 8)); // don't add r * dstride b/c add it at end of loop iterations
+        const __m128i s1 = _mm_loadu_si128((const __m128i *)(*src + subsampling_factor * 8));
+        const __m256i s  = _mm256_setr_m128i(s0, s1);
+        const __m128i d0 = _mm_loadu_si128((const __m128i *)(dst + r * dstride));
+        const __m128i d1 = _mm_loadu_si128(
+            (const __m128i *)(dst + (r + subsampling_factor) * dstride));
+        const __m256i d = _mm256_setr_m128i(d0, d1);
+
+        const __m256i diff = _mm256_sub_epi16(d, s);
+        const __m256i mse  = _mm256_madd_epi16(diff, diff);
+        *sum               = _mm256_add_epi32(*sum, mse);
+
+        *src += 8 * 2 * subsampling_factor;
+    }
 }
 
-static INLINE void mse_8x4_8bit_avx2(const uint8_t **src, const uint8_t *dst, const int32_t dstride,
-                                     __m256i *sum) {
-    mse_8x2_8bit_avx2(src, dst + 0 * dstride, dstride, sum);
-    mse_8x2_8bit_avx2(src, dst + 2 * dstride, dstride, sum);
+static INLINE void mse_8xn_8bit_avx2(const uint8_t **src, const uint8_t *dst, const int32_t dstride,
+                                     __m256i *sum, uint8_t height, uint8_t subsampling_factor) {
+    for (int32_t r = 0; r < height; r += 2 * subsampling_factor) {
+        const __m128i s = _mm_set_epi64x(
+            *(uint64_t *)(*src + subsampling_factor * 8),
+            *(uint64_t *)(*src +
+                          0 * 8)); // don't add r * dstride b/c add it at end of loop iterations
+        const __m128i d = _mm_set_epi64x(*(uint64_t *)(dst + (r + subsampling_factor) * dstride),
+                                         *(uint64_t *)(dst + r * dstride));
+
+        const __m256i s_16 = _mm256_cvtepu8_epi16(s);
+        const __m256i d_16 = _mm256_cvtepu8_epi16(d);
+
+        const __m256i diff = _mm256_sub_epi16(d_16, s_16);
+        const __m256i mse  = _mm256_madd_epi16(diff, diff);
+        *sum               = _mm256_add_epi32(*sum, mse);
+
+        *src += 8 * 2 * subsampling_factor;
+    }
 }
 
 static INLINE uint32_t sum32(const __m256i src) {
@@ -160,8 +223,9 @@ static INLINE uint32_t sum32(const __m256i src) {
     return (uint32_t)_mm_cvtsi128_si32(dst);
 }
 
-static INLINE uint64_t dist_8x8_16bit_avx2(const uint16_t **src, const uint16_t *dst,
-                                           const int32_t dstride, const int32_t coeff_shift) {
+static INLINE uint64_t dist_8xn_16bit_avx2(const uint16_t **src, const uint16_t *dst,
+                                           const int32_t dstride, const int32_t coeff_shift,
+                                           uint8_t height, uint8_t subsampling_factor) {
     __m256i ss = _mm256_setzero_si256();
     __m256i dd = _mm256_setzero_si256();
     __m256i s2 = _mm256_setzero_si256();
@@ -170,17 +234,27 @@ static INLINE uint64_t dist_8x8_16bit_avx2(const uint16_t **src, const uint16_t
     __m256i ssdd;
     __m128i sum;
 
-    for (int32_t r = 0; r < 4; r++) {
-        const __m256i s  = _mm256_loadu_si256((const __m256i *)*src);
-        const __m128i d0 = _mm_loadu_si128((const __m128i *)(dst + 2 * r * dstride + 0 * dstride));
-        const __m128i d1 = _mm_loadu_si128((const __m128i *)(dst + 2 * r * dstride + 1 * dstride));
-        const __m256i d  = _mm256_setr_m128i(d0, d1);
-        ss               = _mm256_add_epi16(ss, s);
-        dd               = _mm256_add_epi16(dd, d);
-        s2               = _mm256_add_epi32(s2, _mm256_madd_epi16(s, s));
-        sd               = _mm256_add_epi32(sd, _mm256_madd_epi16(s, d));
-        d2               = _mm256_add_epi32(d2, _mm256_madd_epi16(d, d));
-        *src += 16;
+    for (int32_t r = 0; r < height; r += 2 * subsampling_factor) {
+        const __m128i s0 = _mm_loadu_si128(
+            (const __m128i *)(*src +
+                              0 * 8)); // don't add r * dstride b/c add it at end of loop iterations
+        const __m128i s1 = _mm_loadu_si128(
+            (const __m128i *)(*src +
+                              subsampling_factor *
+                                  8)); // don't add r * dstride b/c add it at end of loop iterations
+        const __m256i s = _mm256_setr_m128i(s0, s1);
+
+        const __m128i d0 = _mm_loadu_si128((const __m128i *)(dst + r * dstride));
+        const __m128i d1 = _mm_loadu_si128(
+            (const __m128i *)(dst + (r + subsampling_factor) * dstride));
+        const __m256i d = _mm256_setr_m128i(d0, d1);
+        ss              = _mm256_add_epi16(ss, s);
+        dd              = _mm256_add_epi16(dd, d);
+        s2              = _mm256_add_epi32(s2, _mm256_madd_epi16(s, s));
+        sd              = _mm256_add_epi32(sd, _mm256_madd_epi16(s, d));
+        d2              = _mm256_add_epi32(d2, _mm256_madd_epi16(d, d));
+
+        *src += 8 * 2 * subsampling_factor;
     }
 
     ssdd                 = _mm256_hadd_epi16(ss, dd);
@@ -207,8 +281,9 @@ static INLINE uint64_t dist_8x8_16bit_avx2(const uint16_t **src, const uint16_t
                                (sqrt((20000 << 4 * coeff_shift) + svar * (double)dvar)));
 }
 
-static INLINE uint64_t dist_8x8_8bit_avx2(const uint8_t **src, const uint8_t *dst,
-                                          const int32_t dstride, const int32_t coeff_shift) {
+static INLINE uint64_t dist_8xn_8bit_avx2(const uint8_t **src, const uint8_t *dst,
+                                          const int32_t dstride, const int32_t coeff_shift,
+                                          uint8_t height, uint8_t subsampling_factor) {
     __m256i ss = _mm256_setzero_si256();
     __m256i dd = _mm256_setzero_si256();
     __m256i s2 = _mm256_setzero_si256();
@@ -217,10 +292,13 @@ static INLINE uint64_t dist_8x8_8bit_avx2(const uint8_t **src, const uint8_t *ds
     __m256i ssdd;
     __m128i sum;
 
-    for (int32_t r = 0; r < 4; r++) {
-        const __m128i s = _mm_loadu_si128((const __m128i *)*src);
-        const __m128i d = _mm_set_epi64x(*(uint64_t *)(dst + 2 * r * dstride + 1 * dstride),
-                                         *(uint64_t *)(dst + 2 * r * dstride + 0 * dstride));
+    for (int32_t r = 0; r < height; r += 2 * subsampling_factor) {
+        const __m128i s = _mm_set_epi64x(
+            *(uint64_t *)(*src + subsampling_factor * 8),
+            *(uint64_t *)(*src +
+                          0 * 8)); // don't add r * dstride b/c add it at end of loop iterations
+        const __m128i d = _mm_set_epi64x(*(uint64_t *)(dst + (r + subsampling_factor) * dstride),
+                                         *(uint64_t *)(dst + r * dstride));
 
         const __m256i s_16 = _mm256_cvtepu8_epi16(s);
         const __m256i d_16 = _mm256_cvtepu8_epi16(d);
@@ -230,7 +308,8 @@ static INLINE uint64_t dist_8x8_8bit_avx2(const uint8_t **src, const uint8_t *ds
         s2 = _mm256_add_epi32(s2, _mm256_madd_epi16(s_16, s_16));
         sd = _mm256_add_epi32(sd, _mm256_madd_epi16(s_16, d_16));
         d2 = _mm256_add_epi32(d2, _mm256_madd_epi16(d_16, d_16));
-        *src += 16;
+
+        *src += 8 * 2 * subsampling_factor; // width * 2 lines per iter. * subsampling
     }
 
     ssdd                 = _mm256_hadd_epi16(ss, dd);
@@ -276,7 +355,8 @@ static INLINE uint64_t sum64(const __m256i src) {
 /* Compute MSE only on the blocks we filtered. */
 uint64_t compute_cdef_dist_16bit_avx2(const uint16_t *dst, int32_t dstride, const uint16_t *src,
                                       const CdefList *dlist, int32_t cdef_count, BlockSize bsize,
-                                      int32_t coeff_shift, int32_t pli) {
+                                      int32_t coeff_shift, int32_t pli,
+                                      uint8_t subsampling_factor) {
     uint64_t sum;
     int32_t  bi, bx, by;
 
@@ -285,7 +365,8 @@ uint64_t compute_cdef_dist_16bit_avx2(const uint16_t *dst, int32_t dstride, cons
         for (bi = 0; bi < cdef_count; bi++) {
             by = dlist[bi].by;
             bx = dlist[bi].bx;
-            sum += dist_8x8_16bit_avx2(&src, dst + 8 * by * dstride + 8 * bx, dstride, coeff_shift);
+            sum += dist_8xn_16bit_avx2(
+                &src, dst + 8 * by * dstride + 8 * bx, dstride, coeff_shift, 8, subsampling_factor);
         }
     } else {
         __m256i mse64 = _mm256_setzero_si256();
@@ -295,8 +376,12 @@ uint64_t compute_cdef_dist_16bit_avx2(const uint16_t *dst, int32_t dstride, cons
                 __m256i mse32 = _mm256_setzero_si256();
                 by            = dlist[bi].by;
                 bx            = dlist[bi].bx;
-                mse_8x4_16bit_avx2(&src, dst + (8 * by + 0) * dstride + 8 * bx, dstride, &mse32);
-                mse_8x4_16bit_avx2(&src, dst + (8 * by + 4) * dstride + 8 * bx, dstride, &mse32);
+                mse_8xn_16bit_avx2(&src,
+                                   dst + (8 * by + 0) * dstride + 8 * bx,
+                                   dstride,
+                                   &mse32,
+                                   8,
+                                   subsampling_factor);
                 sum_32_to_64(mse32, &mse64);
             }
         } else if (bsize == BLOCK_4X8) {
@@ -304,8 +389,12 @@ uint64_t compute_cdef_dist_16bit_avx2(const uint16_t *dst, int32_t dstride, cons
                 __m256i mse32 = _mm256_setzero_si256();
                 by            = dlist[bi].by;
                 bx            = dlist[bi].bx;
-                mse_4x4_16bit_avx2(&src, dst + (8 * by + 0) * dstride + 4 * bx, dstride, &mse32);
-                mse_4x4_16bit_avx2(&src, dst + (8 * by + 4) * dstride + 4 * bx, dstride, &mse32);
+                mse_4xn_16bit_avx2(&src,
+                                   dst + (8 * by + 0) * dstride + 4 * bx,
+                                   dstride,
+                                   &mse32,
+                                   8,
+                                   subsampling_factor);
                 sum_32_to_64(mse32, &mse64);
             }
         } else if (bsize == BLOCK_8X4) {
@@ -313,7 +402,8 @@ uint64_t compute_cdef_dist_16bit_avx2(const uint16_t *dst, int32_t dstride, cons
                 __m256i mse32 = _mm256_setzero_si256();
                 by            = dlist[bi].by;
                 bx            = dlist[bi].bx;
-                mse_8x4_16bit_avx2(&src, dst + 4 * by * dstride + 8 * bx, dstride, &mse32);
+                mse_8xn_16bit_avx2(
+                    &src, dst + 4 * by * dstride + 8 * bx, dstride, &mse32, 4, subsampling_factor);
                 sum_32_to_64(mse32, &mse64);
             }
         } else {
@@ -322,7 +412,19 @@ uint64_t compute_cdef_dist_16bit_avx2(const uint16_t *dst, int32_t dstride, cons
                 __m256i mse32 = _mm256_setzero_si256();
                 by            = dlist[bi].by;
                 bx            = dlist[bi].bx;
-                mse_4x4_16bit_avx2(&src, dst + 4 * by * dstride + 4 * bx, dstride, &mse32);
+                // For 4x4 blocks, all points can be computed at once.  Subsampling is done in a special function
+                // to avoid accessing memory that doesn't belong to the current picture (since subsampling is implemented
+                // as a multiplier to the step size).
+                if (subsampling_factor == 2)
+                    mse_4x4_16bit_2x_subsampled_avx2(
+                        &src, dst + 4 * by * dstride + 4 * bx, dstride, &mse32);
+                else
+                    mse_4xn_16bit_avx2(&src,
+                                       dst + 4 * by * dstride + 4 * bx,
+                                       dstride,
+                                       &mse32,
+                                       4,
+                                       1); // no subsampling
                 sum_32_to_64(mse32, &mse64);
             }
         }
@@ -335,7 +437,7 @@ uint64_t compute_cdef_dist_16bit_avx2(const uint16_t *dst, int32_t dstride, cons
 
 uint64_t compute_cdef_dist_8bit_avx2(const uint8_t *dst8, int32_t dstride, const uint8_t *src8,
                                      const CdefList *dlist, int32_t cdef_count, BlockSize bsize,
-                                     int32_t coeff_shift, int32_t pli) {
+                                     int32_t coeff_shift, int32_t pli, uint8_t subsampling_factor) {
     uint64_t sum;
     int32_t  bi, bx, by;
 
@@ -344,8 +446,12 @@ uint64_t compute_cdef_dist_8bit_avx2(const uint8_t *dst8, int32_t dstride, const
         for (bi = 0; bi < cdef_count; bi++) {
             by = dlist[bi].by;
             bx = dlist[bi].bx;
-            sum += dist_8x8_8bit_avx2(
-                &src8, dst8 + 8 * by * dstride + 8 * bx, dstride, coeff_shift);
+            sum += dist_8xn_8bit_avx2(&src8,
+                                      dst8 + 8 * by * dstride + 8 * bx,
+                                      dstride,
+                                      coeff_shift,
+                                      8,
+                                      subsampling_factor);
         }
     } else {
         __m256i mse64 = _mm256_setzero_si256();
@@ -355,8 +461,12 @@ uint64_t compute_cdef_dist_8bit_avx2(const uint8_t *dst8, int32_t dstride, const
                 __m256i mse32 = _mm256_setzero_si256();
                 by            = dlist[bi].by;
                 bx            = dlist[bi].bx;
-                mse_8x4_8bit_avx2(&src8, dst8 + (8 * by + 0) * dstride + 8 * bx, dstride, &mse32);
-                mse_8x4_8bit_avx2(&src8, dst8 + (8 * by + 4) * dstride + 8 * bx, dstride, &mse32);
+                mse_8xn_8bit_avx2(&src8,
+                                  dst8 + (8 * by + 0) * dstride + 8 * bx,
+                                  dstride,
+                                  &mse32,
+                                  8,
+                                  subsampling_factor);
                 sum_32_to_64(mse32, &mse64);
             }
         } else if (bsize == BLOCK_4X8) {
@@ -364,8 +474,12 @@ uint64_t compute_cdef_dist_8bit_avx2(const uint8_t *dst8, int32_t dstride, const
                 __m256i mse32 = _mm256_setzero_si256();
                 by            = dlist[bi].by;
                 bx            = dlist[bi].bx;
-                mse_4x4_8bit_avx2(&src8, dst8 + (8 * by + 0) * dstride + 4 * bx, dstride, &mse32);
-                mse_4x4_8bit_avx2(&src8, dst8 + (8 * by + 4) * dstride + 4 * bx, dstride, &mse32);
+                mse_4xn_8bit_avx2(&src8,
+                                  dst8 + (8 * by + 0) * dstride + 4 * bx,
+                                  dstride,
+                                  &mse32,
+                                  8,
+                                  subsampling_factor);
                 sum_32_to_64(mse32, &mse64);
             }
         } else if (bsize == BLOCK_8X4) {
@@ -373,7 +487,12 @@ uint64_t compute_cdef_dist_8bit_avx2(const uint8_t *dst8, int32_t dstride, const
                 __m256i mse32 = _mm256_setzero_si256();
                 by            = dlist[bi].by;
                 bx            = dlist[bi].bx;
-                mse_8x4_8bit_avx2(&src8, dst8 + 4 * by * dstride + 8 * bx, dstride, &mse32);
+                mse_8xn_8bit_avx2(&src8,
+                                  dst8 + 4 * by * dstride + 8 * bx,
+                                  dstride,
+                                  &mse32,
+                                  4,
+                                  subsampling_factor);
                 sum_32_to_64(mse32, &mse64);
             }
         } else {
@@ -382,7 +501,19 @@ uint64_t compute_cdef_dist_8bit_avx2(const uint8_t *dst8, int32_t dstride, const
                 __m256i mse32 = _mm256_setzero_si256();
                 by            = dlist[bi].by;
                 bx            = dlist[bi].bx;
-                mse_4x4_8bit_avx2(&src8, dst8 + 4 * by * dstride + 4 * bx, dstride, &mse32);
+                // For 4x4 blocks, all points can be computed at once.  Subsampling is done in a special function
+                // to avoid accessing memory that doesn't belong to the current picture (since subsampling is implemented
+                // as a multiplier to the step size).
+                if (subsampling_factor == 2)
+                    mse_4x4_8bit_2x_subsampled_avx2(
+                        &src8, dst8 + 4 * by * dstride + 4 * bx, dstride, &mse32);
+                else
+                    mse_4xn_8bit_avx2(&src8,
+                                      dst8 + 4 * by * dstride + 4 * bx,
+                                      dstride,
+                                      &mse32,
+                                      4,
+                                      1); // no subsampling
                 sum_32_to_64(mse32, &mse64);
             }
         }
