@@ -22,6 +22,11 @@
 #include "EbCommonUtils.h"
 //#include "EbLog.h"
 
+void get_recon_pic(PictureControlSet *pcs_ptr, EbPictureBufferDesc **recon_ptr, EbBool is_highbd);
+/*************************************************************************************************
+ * svt_av1_loop_filter_init
+ * Initialize the loop filter limits and thresholds
+ *************************************************************************************************/
 void svt_av1_loop_filter_init(PictureControlSet *pcs_ptr) {
     //assert(MB_MODE_COUNT == n_elements(mode_lf_lut));
     LoopFilterInfoN *  lfi = &pcs_ptr->parent_pcs_ptr->lf_info;
@@ -67,7 +72,8 @@ static INLINE void setup_pred_plane(struct Buf2D *dst, BlockSize bsize, uint8_t
     dst->height     = height;
     dst->stride     = stride;
 }
-void svt_av1_setup_dst_planes(struct MacroblockdPlane *planes, BlockSize bsize,
+void svt_av1_setup_dst_planes(PictureControlSet *pcs_ptr, struct MacroblockdPlane *planes,
+                              BlockSize bsize,
                               //const Yv12BufferConfig *src,
                               const EbPictureBufferDesc *src, int32_t mi_row, int32_t mi_col,
                               const int32_t plane_start, const int32_t plane_end) {
@@ -80,6 +86,8 @@ void svt_av1_setup_dst_planes(struct MacroblockdPlane *planes, BlockSize bsize,
     //        src->crop_heights[is_uv], src->strides[is_uv], mi_row,
     //        mi_col, NULL, pd->subsampling_x, pd->subsampling_y);
     //}
+    SequenceControlSet *scs_ptr = (SequenceControlSet *)
+                                      pcs_ptr->parent_pcs_ptr->scs_wrapper_ptr->object_ptr;
     for (int32_t i = plane_start; i < AOMMIN(plane_end, 3); ++i) {
         if (i == 0) {
             struct MacroblockdPlane *const pd = &planes[0];
@@ -87,8 +95,10 @@ void svt_av1_setup_dst_planes(struct MacroblockdPlane *planes, BlockSize bsize,
                 &pd->dst,
                 bsize,
                 &src->buffer_y[(src->origin_x + src->origin_y * src->stride_y) << pd->is_16bit],
-                src->width,
-                src->height,
+                (scs_ptr->max_input_luma_width -
+                 scs_ptr
+                     ->max_input_pad_right), // The width/height should be the unpadded width/height (see AV1 spec 7.14.2 Edge Loop Filter Process)
+                (scs_ptr->max_input_luma_height - scs_ptr->max_input_pad_bottom),
                 src->stride_y,
                 mi_row,
                 mi_col,
@@ -102,8 +112,9 @@ void svt_av1_setup_dst_planes(struct MacroblockdPlane *planes, BlockSize bsize,
                 bsize,
                 &src->buffer_cb[((src->origin_x + src->origin_y * src->stride_cb) << pd->is_16bit) /
                                 2],
-                src->width / 2,
-                src->height / 2,
+                (scs_ptr->max_input_luma_width - scs_ptr->max_input_pad_right) >>
+                    1, // The width/height should be the unpadded width/height (see AV1 spec 7.14.2 Edge Loop Filter Process)
+                (scs_ptr->max_input_luma_height - scs_ptr->max_input_pad_bottom) >> 1,
                 src->stride_cb,
                 mi_row,
                 mi_col,
@@ -117,8 +128,9 @@ void svt_av1_setup_dst_planes(struct MacroblockdPlane *planes, BlockSize bsize,
                 bsize,
                 &src->buffer_cr[((src->origin_x + src->origin_y * src->stride_cr) << pd->is_16bit) /
                                 2],
-                src->width / 2,
-                src->height / 2,
+                (scs_ptr->max_input_luma_width - scs_ptr->max_input_pad_right) >>
+                    1, // The width/height should be the unpadded width/height (see AV1 spec 7.14.2 Edge Loop Filter Process)
+                (scs_ptr->max_input_luma_height - scs_ptr->max_input_pad_bottom) >> 1,
                 src->stride_cr,
                 mi_row,
                 mi_col,
@@ -131,14 +143,17 @@ void svt_av1_setup_dst_planes(struct MacroblockdPlane *planes, BlockSize bsize,
 
 //***************************************************************************************************//
 static INLINE TxSize get_transform_size(const MbModeInfo *const mbmi, const EdgeDir edge_dir,
-    const int32_t plane, const struct MacroblockdPlane *plane_ptr, const EbBool is_skip) {
+                                        const int32_t                  plane,
+                                        const struct MacroblockdPlane *plane_ptr,
+                                        const EbBool                   is_skip) {
     assert(mbmi != NULL);
 
     TxSize tx_size = (plane == COMPONENT_LUMA)
-        ? (is_skip
-            ? tx_depth_to_tx_size[0][mbmi->block_mi.sb_type]
-            : tx_depth_to_tx_size[mbmi->tx_depth][mbmi->block_mi.sb_type]) // use max_tx_size
-        : av1_get_max_uv_txsize(mbmi->block_mi.sb_type, plane_ptr->subsampling_x, plane_ptr->subsampling_y);
+        ? (is_skip ? tx_depth_to_tx_size[0][mbmi->block_mi.sb_type]
+                   : tx_depth_to_tx_size[mbmi->block_mi.tx_depth]
+                                        [mbmi->block_mi.sb_type]) // use max_tx_size
+        : av1_get_max_uv_txsize(
+              mbmi->block_mi.sb_type, plane_ptr->subsampling_x, plane_ptr->subsampling_y);
     assert(tx_size < TX_SIZES_ALL);
 
     // since in case of chrominance or non-square transorm need to convert
@@ -151,9 +166,8 @@ static INLINE TxSize get_transform_size(const MbModeInfo *const mbmi, const Edge
 // Return TxSize from get_transform_size(), so it is plane and direction
 // awared
 static TxSize set_lpf_parameters(Av1DeblockingParameters *const params, const uint64_t mode_step,
-                                 const PictureControlSet *const pcs_ptr,
-                                 const EdgeDir edge_dir, const uint32_t x,
-                                 const uint32_t y, const int32_t plane,
+                                 const PictureControlSet *const pcs_ptr, const EdgeDir edge_dir,
+                                 const uint32_t x, const uint32_t y, const int32_t plane,
                                  const struct MacroblockdPlane *const plane_ptr) {
     FrameHeader *          frm_hdr = &pcs_ptr->parent_pcs_ptr->frm_hdr;
     const LoopFilterInfoN *lfi_n   = &pcs_ptr->parent_pcs_ptr->lf_info;
@@ -232,7 +246,8 @@ static TxSize set_lpf_parameters(Av1DeblockingParameters *const params, const ui
                     return TX_INVALID;
                 const int32_t pv_skip = mi_prev->block_mi.skip &&
                     is_inter_block_no_intrabc(mi_prev->block_mi.ref_frame[0]);
-                const TxSize  pv_ts = get_transform_size(mi_prev, edge_dir, plane, plane_ptr, pv_skip);
+                const TxSize pv_ts = get_transform_size(
+                    mi_prev, edge_dir, plane, plane_ptr, pv_skip);
                 uint32_t pv_lvl;
                 mode = (mi_prev->block_mi.mode == INTRA_MODE_4x4) ? DC_PRED
                                                                   : mi_prev->block_mi.mode;
@@ -283,19 +298,19 @@ static TxSize set_lpf_parameters(Av1DeblockingParameters *const params, const ui
 
     return ts;
 }
-
-void svt_av1_filter_block_plane_vert(const PictureControlSet *const pcs_ptr,
-                                     const int32_t plane,
+/*************************************************************************************************
+* svt_av1_filter_block_plane_vert
+* Filter all the vertical edges in the same superblock
+*************************************************************************************************/
+void svt_av1_filter_block_plane_vert(const PictureControlSet *const pcs_ptr, const int32_t plane,
                                      const MacroblockdPlane *const plane_ptr, const uint32_t mi_row,
                                      const uint32_t mi_col) {
     SequenceControlSet *scs_ptr = (SequenceControlSet *)
                                       pcs_ptr->parent_pcs_ptr->scs_wrapper_ptr->object_ptr;
-    EbBool is_16bit = scs_ptr->static_config.encoder_bit_depth > 8;
     // TODO
     // when loop_filter_mode = 1, dblk is processed in encdec
     // 16 bit dblk for loop_filter_mode = 1 needs to enabled after 16bit encdec is done
-    if (scs_ptr->static_config.is_16bit_pipeline)
-        is_16bit = EB_TRUE;
+    EbBool         is_16bit   = scs_ptr->is_16bit_pipeline;
     const int32_t  row_step   = MI_SIZE >> MI_SIZE_LOG2;
     const uint32_t scale_horz = plane_ptr->subsampling_x;
     const uint32_t scale_vert = plane_ptr->subsampling_y;
@@ -305,33 +320,20 @@ void svt_av1_filter_block_plane_vert(const PictureControlSet *const pcs_ptr,
                                                                    : (SB64_MIB_SIZE >> scale_vert);
     int32_t x_range = scs_ptr->seq_header.sb_size == BLOCK_128X128 ? (MAX_MIB_SIZE >> scale_horz)
                                                                    : (SB64_MIB_SIZE >> scale_horz);
-    const uint32_t sb_size = (scs_ptr->seq_header.sb_size == BLOCK_128X128) ? 128 : 64;
-    if (mi_row == (scs_ptr->max_input_luma_height / sb_size * sb_size) >> MI_SIZE_LOG2) {
-        y_range = (((scs_ptr->max_input_luma_height - scs_ptr->max_input_pad_bottom) % sb_size) +
-                   MI_SIZE - 1) >>
-            MI_SIZE_LOG2;
-        if (plane) {
-            y_range = ((((scs_ptr->max_input_luma_height - scs_ptr->max_input_pad_bottom) %
-                         sb_size) >>
-                        scale_vert) +
-                       MI_SIZE - 1) >>
-                MI_SIZE_LOG2;
+
+    if (pcs_ptr->parent_pcs_ptr->frame_superres_enabled) {
+        // the boundary of last column should use the actual width for frame might be downscaled in super resolution
+        const uint32_t       sb_size = (scs_ptr->seq_header.sb_size == BLOCK_128X128) ? 128 : 64;
+        EbPictureBufferDesc *pic_ptr = pcs_ptr->parent_pcs_ptr->enhanced_picture_ptr;
+        if (mi_col == (pic_ptr->width / sb_size * sb_size) >> MI_SIZE_LOG2) {
+            x_range = (((pic_ptr->width) % sb_size) + MI_SIZE - 1) >> MI_SIZE_LOG2;
+            if (plane) {
+                x_range = ((((pic_ptr->width) % sb_size + scale_horz) >> scale_horz) + MI_SIZE -
+                           1) >>
+                    MI_SIZE_LOG2;
+            }
         }
     }
-
-    if (mi_col == (scs_ptr->max_input_luma_width / sb_size * sb_size) >> MI_SIZE_LOG2) {
-        x_range = (((scs_ptr->max_input_luma_width - scs_ptr->max_input_pad_right) % sb_size) +
-                   MI_SIZE - 1) >>
-            MI_SIZE_LOG2;
-        if (plane) {
-            x_range = ((((scs_ptr->max_input_luma_width - scs_ptr->max_input_pad_right) %
-                         sb_size) >>
-                        scale_horz) +
-                       MI_SIZE - 1) >>
-                MI_SIZE_LOG2;
-        }
-    }
-
     for (int32_t y = 0; y < y_range; y += row_step) {
         uint8_t *p = dst_ptr + ((y * MI_SIZE * dst_stride) << plane_ptr->is_16bit);
         for (int32_t x = 0; x < x_range;) {
@@ -424,18 +426,18 @@ void svt_av1_filter_block_plane_vert(const PictureControlSet *const pcs_ptr,
         }
     }
 }
-
-void svt_av1_filter_block_plane_horz(const PictureControlSet *const pcs_ptr,
-                                     const int32_t plane,
+/*************************************************************************************************
+* svt_av1_filter_block_plane_horz
+* Filter all the horizontal edges in the same superblock
+*************************************************************************************************/
+void svt_av1_filter_block_plane_horz(const PictureControlSet *const pcs_ptr, const int32_t plane,
                                      const MacroblockdPlane *const plane_ptr, const uint32_t mi_row,
                                      const uint32_t mi_col) {
     SequenceControlSet *scs_ptr = (SequenceControlSet *)
                                       pcs_ptr->parent_pcs_ptr->scs_wrapper_ptr->object_ptr;
-    EbBool is_16bit = scs_ptr->static_config.encoder_bit_depth > 8;
     // when loop_filter_mode = 1, dblk is processed in encdec
     // 16 bit dblk for loop_filter_mode = 1 needs to enabled after 16bit encdec is done
-    if (scs_ptr->static_config.is_16bit_pipeline)
-        is_16bit = EB_TRUE;
+    EbBool         is_16bit   = scs_ptr->is_16bit_pipeline;
     const int32_t  col_step   = MI_SIZE >> MI_SIZE_LOG2;
     const uint32_t scale_horz = plane_ptr->subsampling_x;
     const uint32_t scale_vert = plane_ptr->subsampling_y;
@@ -447,33 +449,19 @@ void svt_av1_filter_block_plane_horz(const PictureControlSet *const pcs_ptr,
                                                                       : (SB64_MIB_SIZE >> scale_horz);
     uint32_t mi_stride = pcs_ptr->mi_stride;
 
-    const uint32_t sb_size = (scs_ptr->seq_header.sb_size == BLOCK_128X128) ? 128 : 64;
-    if (mi_row == (scs_ptr->max_input_luma_height / sb_size * sb_size) >> MI_SIZE_LOG2) {
-        y_range = (((scs_ptr->max_input_luma_height - scs_ptr->max_input_pad_bottom) % sb_size) +
-                   MI_SIZE - 1) >>
-            MI_SIZE_LOG2;
-        if (plane) {
-            y_range = ((((scs_ptr->max_input_luma_height - scs_ptr->max_input_pad_bottom) %
-                         sb_size) >>
-                        scale_vert) +
-                       MI_SIZE - 1) >>
-                MI_SIZE_LOG2;
+    if (pcs_ptr->parent_pcs_ptr->frame_superres_enabled) {
+        // the boundary of last column should use the actual width for frames might be downscaled in super resolution
+        const uint32_t       sb_size = (scs_ptr->seq_header.sb_size == BLOCK_128X128) ? 128 : 64;
+        EbPictureBufferDesc *pic_ptr = pcs_ptr->parent_pcs_ptr->enhanced_picture_ptr;
+        if (mi_col == (pic_ptr->width / sb_size * sb_size) >> MI_SIZE_LOG2) {
+            x_range = (((pic_ptr->width) % sb_size) + MI_SIZE - 1) >> MI_SIZE_LOG2;
+            if (plane) {
+                x_range = ((((pic_ptr->width) % sb_size + scale_horz) >> scale_horz) + MI_SIZE -
+                           1) >>
+                    MI_SIZE_LOG2;
+            }
         }
     }
-
-    if (mi_col == (scs_ptr->max_input_luma_width / sb_size * sb_size) >> MI_SIZE_LOG2) {
-        x_range = (((scs_ptr->max_input_luma_width - scs_ptr->max_input_pad_right) % sb_size) +
-                   MI_SIZE - 1) >>
-            MI_SIZE_LOG2;
-        if (plane) {
-            x_range = ((((scs_ptr->max_input_luma_width - scs_ptr->max_input_pad_right) %
-                         sb_size) >>
-                        scale_horz) +
-                       MI_SIZE - 1) >>
-                MI_SIZE_LOG2;
-        }
-    }
-
     for (int32_t x = 0; x < x_range; x += col_step) {
         uint8_t *p = dst_ptr + ((x * MI_SIZE) << plane_ptr->is_16bit);
         for (int32_t y = 0; y < y_range;) {
@@ -573,13 +561,14 @@ void svt_av1_filter_block_plane_horz(const PictureControlSet *const pcs_ptr,
         }
     }
 }
-
-// New function to filter each sb (64x64)
+/*************************************************************************************************
+* loop_filter_sb
+* Loop over all superblocks in the picture and filter each superblock
+*************************************************************************************************/
 void loop_filter_sb(EbPictureBufferDesc *frame_buffer, //reconpicture,
                     //Yv12BufferConfig *frame_buffer,
-                    PictureControlSet *pcs_ptr,
-                    int32_t mi_row, int32_t mi_col,
-                    int32_t plane_start, int32_t plane_end, uint8_t last_col) {
+                    PictureControlSet *pcs_ptr, int32_t mi_row, int32_t mi_col, int32_t plane_start,
+                    int32_t plane_end, uint8_t last_col) {
     FrameHeader *           frm_hdr = &pcs_ptr->parent_pcs_ptr->frm_hdr;
     struct MacroblockdPlane pd[3];
     int32_t                 plane;
@@ -597,7 +586,7 @@ void loop_filter_sb(EbPictureBufferDesc *frame_buffer, //reconpicture,
     pd[2].plane_type    = PLANE_TYPE_UV;
     pd[2].is_16bit      = frame_buffer->bit_depth > 8;
 
-    if (pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.is_16bit_pipeline)
+    if (pcs_ptr->parent_pcs_ptr->scs_ptr->is_16bit_pipeline)
         pd[0].is_16bit = pd[1].is_16bit = pd[2].is_16bit = EB_TRUE;
 
     for (plane = plane_start; plane < plane_end; plane++) {
@@ -610,9 +599,10 @@ void loop_filter_sb(EbPictureBufferDesc *frame_buffer, //reconpicture,
             continue;
 
         if (frm_hdr->loop_filter_params.combine_vert_horz_lf) {
-            // filter all vertical and horizontal edges in every 64x64 super block
+            // filter all vertical and horizontal edges in every super block
             // filter vertical edges
-            svt_av1_setup_dst_planes(pd,
+            svt_av1_setup_dst_planes(pcs_ptr,
+                                     pd,
                                      pcs_ptr->parent_pcs_ptr->scs_ptr->seq_header.sb_size,
                                      frame_buffer,
                                      mi_row,
@@ -627,7 +617,8 @@ void loop_filter_sb(EbPictureBufferDesc *frame_buffer, //reconpicture,
                 : SB64_MIB_SIZE;
 
             if (mi_col - max_mib_size >= 0) {
-                svt_av1_setup_dst_planes(pd,
+                svt_av1_setup_dst_planes(pcs_ptr,
+                                         pd,
                                          pcs_ptr->parent_pcs_ptr->scs_ptr->seq_header.sb_size,
                                          frame_buffer,
                                          mi_row,
@@ -639,7 +630,8 @@ void loop_filter_sb(EbPictureBufferDesc *frame_buffer, //reconpicture,
             }
             // Filter the horizontal edges of the last sb in each row
             if (last_col) {
-                svt_av1_setup_dst_planes(pd,
+                svt_av1_setup_dst_planes(pcs_ptr,
+                                         pd,
                                          pcs_ptr->parent_pcs_ptr->scs_ptr->seq_header.sb_size,
                                          frame_buffer,
                                          mi_row,
@@ -650,7 +642,8 @@ void loop_filter_sb(EbPictureBufferDesc *frame_buffer, //reconpicture,
             }
         } else {
             // filter all vertical edges in every 64x64 super block
-            svt_av1_setup_dst_planes(pd,
+            svt_av1_setup_dst_planes(pcs_ptr,
+                                     pd,
                                      pcs_ptr->parent_pcs_ptr->scs_ptr->seq_header.sb_size,
                                      frame_buffer,
                                      mi_row,
@@ -661,7 +654,8 @@ void loop_filter_sb(EbPictureBufferDesc *frame_buffer, //reconpicture,
             svt_av1_filter_block_plane_vert(pcs_ptr, plane, &pd[plane], mi_row, mi_col);
 
             // filter all horizontal edges in every 64x64 super block
-            svt_av1_setup_dst_planes(pd,
+            svt_av1_setup_dst_planes(pcs_ptr,
+                                     pd,
                                      pcs_ptr->parent_pcs_ptr->scs_ptr->seq_header.sb_size,
                                      frame_buffer,
                                      mi_row,
@@ -672,7 +666,10 @@ void loop_filter_sb(EbPictureBufferDesc *frame_buffer, //reconpicture,
         }
     }
 }
-
+/*************************************************************************************************
+* svt_av1_loop_filter_frame
+* Apply loop filtering to the frame based on the selected loop filter parameters
+*************************************************************************************************/
 void svt_av1_loop_filter_frame(EbPictureBufferDesc *frame_buffer, PictureControlSet *pcs_ptr,
                                int32_t plane_start, int32_t plane_end) {
     SequenceControlSet *scs_ptr = (SequenceControlSet *)
@@ -719,10 +716,7 @@ extern int16_t svt_av1_ac_quant_q3(int32_t qindex, int32_t delta, AomBitDepth bi
 
 void svt_copy_buffer(EbPictureBufferDesc *srcBuffer, EbPictureBufferDesc *dstBuffer,
                      PictureControlSet *pcs_ptr, uint8_t plane) {
-    EbBool is_16bit = (EbBool)(pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.encoder_bit_depth >
-                               EB_8BIT);
-    if (pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.is_16bit_pipeline)
-        is_16bit = EB_TRUE;
+    EbBool is_16bit        = pcs_ptr->parent_pcs_ptr->scs_ptr->is_16bit_pipeline;
     dstBuffer->origin_x    = srcBuffer->origin_x;
     dstBuffer->origin_y    = srcBuffer->origin_y;
     dstBuffer->width       = srcBuffer->width;
@@ -796,8 +790,7 @@ uint64_t picture_sse_calculations(PictureControlSet *pcs_ptr, EbPictureBufferDes
 
 {
     SequenceControlSet *scs_ptr  = pcs_ptr->parent_pcs_ptr->scs_ptr;
-    EbBool              is_16bit = scs_ptr->static_config.is_16bit_pipeline ||
-        (scs_ptr->static_config.encoder_bit_depth > EB_8BIT);
+    EbBool              is_16bit = scs_ptr->is_16bit_pipeline;
 
     const uint32_t ss_x = scs_ptr->subsampling_x;
     const uint32_t ss_y = scs_ptr->subsampling_y;
@@ -926,7 +919,11 @@ uint64_t picture_sse_calculations(PictureControlSet *pcs_ptr, EbPictureBufferDes
         return 0;
     }
 }
-
+/*************************************************************************************************
+* try_filter_frame
+* Sett the filter levels, compute the filtering sse, and resett the recon buffer.
+* Returns the filtering SSE
+*************************************************************************************************/
 static int64_t try_filter_frame(
     //const Yv12BufferConfig *sd,
     //Av1Comp *const cpi,
@@ -945,26 +942,9 @@ static int64_t try_filter_frame(
     if (plane == 0 && dir == 1)
         filter_level[0] = frm_hdr->loop_filter_params.filter_level[0];
 
-    EbBool is_16bit = (EbBool)(pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.encoder_bit_depth >
-                               EB_8BIT);
+    EbBool is_16bit = pcs_ptr->parent_pcs_ptr->scs_ptr->is_16bit_pipeline;
     EbPictureBufferDesc *recon_buffer;
-    if (pcs_ptr->parent_pcs_ptr->is_used_as_reference_flag == EB_TRUE) {
-        //get the 16bit form of the input SB
-        if (pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.is_16bit_pipeline || is_16bit) {
-            recon_buffer = ((EbReferenceObject *)
-                                pcs_ptr->parent_pcs_ptr->reference_picture_wrapper_ptr->object_ptr)
-                               ->reference_picture16bit;
-        } else {
-            recon_buffer = ((EbReferenceObject *)
-                                pcs_ptr->parent_pcs_ptr->reference_picture_wrapper_ptr->object_ptr)
-                               ->reference_picture;
-        }
-    } else { // non ref pictures
-        recon_buffer = (pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.is_16bit_pipeline ||
-                        is_16bit)
-            ? pcs_ptr->parent_pcs_ptr->enc_dec_ptr->recon_picture16bit_ptr
-            : pcs_ptr->parent_pcs_ptr->enc_dec_ptr->recon_picture_ptr;
-    }
+    get_recon_pic(pcs_ptr, &recon_buffer, is_16bit);
 
     // set base filters for use of get_filter_level when in DELTA_Q_LF mode
     switch (plane) {
@@ -988,6 +968,10 @@ static int64_t try_filter_frame(
 
     return filt_err;
 }
+/*************************************************************************************************
+* search_filter_level
+* Perform a search for the best filter level for the picture data plane
+*************************************************************************************************/
 static int32_t search_filter_level(
     //const Yv12BufferConfig *sd, Av1Comp *cpi,
     EbPictureBufferDesc *sd, // source
@@ -1013,26 +997,9 @@ static int32_t search_filter_level(
     int32_t filt_mid    = clamp(lvl, min_filter_level, max_filter_level);
     int32_t filter_step = filt_mid < 16 ? 4 : filt_mid / 4;
 
-    EbBool is_16bit = (EbBool)(pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.encoder_bit_depth >
-                               EB_8BIT);
+    EbBool is_16bit = pcs_ptr->parent_pcs_ptr->scs_ptr->is_16bit_pipeline;
     EbPictureBufferDesc *recon_buffer;
-
-    if (pcs_ptr->parent_pcs_ptr->is_used_as_reference_flag == EB_TRUE) {
-        //get the 16bit form of the input SB
-        if (pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.is_16bit_pipeline || is_16bit)
-            recon_buffer = ((EbReferenceObject *)
-                                pcs_ptr->parent_pcs_ptr->reference_picture_wrapper_ptr->object_ptr)
-                               ->reference_picture16bit;
-        else
-            recon_buffer = ((EbReferenceObject *)
-                                pcs_ptr->parent_pcs_ptr->reference_picture_wrapper_ptr->object_ptr)
-                               ->reference_picture;
-    } else { // non ref pictures
-        recon_buffer = (pcs_ptr->parent_pcs_ptr->scs_ptr->static_config.is_16bit_pipeline ||
-                        is_16bit)
-            ? pcs_ptr->parent_pcs_ptr->enc_dec_ptr->recon_picture16bit_ptr
-            : pcs_ptr->parent_pcs_ptr->enc_dec_ptr->recon_picture_ptr;
-    }
+    get_recon_pic(pcs_ptr, &recon_buffer, is_16bit);
     // Sum squared error at each filter level
     int64_t ss_err[MAX_LOOP_FILTER + 1];
 
@@ -1048,50 +1015,6 @@ static int32_t search_filter_level(
         sd, temp_lf_recon_buffer, pcs_ptr, filt_mid, partial_frame, plane, dir);
     filt_best        = filt_mid;
     ss_err[filt_mid] = best_err;
-
-    if (pcs_ptr->parent_pcs_ptr->loop_filter_mode <= 2) {
-        filter_step             = 2;
-        const int32_t filt_high = AOMMIN(filt_mid + filter_step, max_filter_level);
-        const int32_t filt_low  = AOMMAX(filt_mid - filter_step, min_filter_level);
-
-        // Bias against raising loop filter in favor of lowering it.
-        int64_t bias = (best_err >> (15 - (filt_mid / 8))) * filter_step;
-
-        //if ((cpi->oxcf.pass == 2) && (cpi->twopass.section_intra_rating < 20))
-        //    bias = (bias * cpi->twopass.section_intra_rating) / 20;
-
-        // yx, bias less for large block size
-        if (frm_hdr->tx_mode != ONLY_4X4)
-            bias >>= 1;
-
-        if (filt_direction <= 0 && filt_low != filt_mid) {
-            // Get Low filter error score
-            if (ss_err[filt_low] < 0) {
-                ss_err[filt_low] = try_filter_frame(
-                    sd, temp_lf_recon_buffer, pcs_ptr, filt_low, partial_frame, plane, dir);
-            }
-            // If value is close to the best so far then bias towards a lower loop
-            // filter value.
-            if (ss_err[filt_low] < (best_err + bias)) {
-                // Was it actually better than the previous best?
-                if (ss_err[filt_low] < best_err)
-                    best_err = ss_err[filt_low];
-                filt_best = filt_low;
-            }
-        }
-
-        // Now look at filt_high
-        if (filt_direction >= 0 && filt_high != filt_mid) {
-            if (ss_err[filt_high] < 0) {
-                ss_err[filt_high] = try_filter_frame(
-                    sd, temp_lf_recon_buffer, pcs_ptr, filt_high, partial_frame, plane, dir);
-            }
-            // If value is significantly better than previous best, bias added against
-            // raising filter value
-            if (ss_err[filt_high] < (best_err - bias))
-                filt_best = filt_high;
-        }
-    } else {
         while (filter_step > 0) {
             const int32_t filt_high = AOMMIN(filt_mid + filter_step, max_filter_level);
             const int32_t filt_low  = AOMMAX(filt_mid - filter_step, min_filter_level);
@@ -1145,7 +1068,6 @@ static int32_t search_filter_level(
                 filt_mid       = filt_best;
             }
         }
-    }
     // Update best error
     best_err = ss_err[filt_best];
 
@@ -1153,9 +1075,13 @@ static int32_t search_filter_level(
         *best_cost_ret = (double)best_err; //RDCOST_DBL(x->rdmult, 0, best_err);
     return filt_best;
 }
-void svt_av1_pick_filter_level(
-                               EbPictureBufferDesc *srcBuffer, // source input
-                               PictureControlSet *pcs_ptr, LpfPickMethod method) {
+/*************************************************************************************************
+* svt_av1_pick_filter_level
+* Choose the optimal loop filter levels
+*************************************************************************************************/
+EbErrorType svt_av1_pick_filter_level(
+    EbPictureBufferDesc *srcBuffer, // source input
+    PictureControlSet *pcs_ptr, LpfPickMethod method) {
     SequenceControlSet *scs_ptr = (SequenceControlSet *)
                                       pcs_ptr->parent_pcs_ptr->scs_wrapper_ptr->object_ptr;
     FrameHeader *frm_hdr = &pcs_ptr->parent_pcs_ptr->frm_hdr;
@@ -1193,7 +1119,7 @@ void svt_av1_pick_filter_level(
             assert(0 &&
                    "bit_depth should be AOM_BITS_8, AOM_BITS_10 "
                    "or AOM_BITS_12");
-            return;
+            return EB_ErrorNone;
         }
         if (scs_ptr->static_config.encoder_bit_depth != EB_8BIT && frm_hdr->frame_type == KEY_FRAME)
             filt_guess -= 4;
@@ -1206,11 +1132,35 @@ void svt_av1_pick_filter_level(
         lf->filter_level_u  = clamp(filt_guess_chroma, min_filter_level, max_filter_level);
         lf->filter_level_v  = clamp(filt_guess_chroma, min_filter_level, max_filter_level);
     } else {
+        uint16_t                    padding = scs_ptr->super_block_size + 32;
+        EbPictureBufferDescInitData temp_lf_recon_desc_init_data;
+        temp_lf_recon_desc_init_data.max_width          = (uint16_t)scs_ptr->max_input_luma_width;
+        temp_lf_recon_desc_init_data.max_height         = (uint16_t)scs_ptr->max_input_luma_height;
+        temp_lf_recon_desc_init_data.buffer_enable_mask = PICTURE_BUFFER_DESC_FULL_MASK;
+
+        temp_lf_recon_desc_init_data.left_padding  = padding;
+        temp_lf_recon_desc_init_data.right_padding = padding;
+        temp_lf_recon_desc_init_data.top_padding   = padding;
+        temp_lf_recon_desc_init_data.bot_padding   = padding;
+        temp_lf_recon_desc_init_data.split_mode    = EB_FALSE;
+        temp_lf_recon_desc_init_data.color_format  = scs_ptr->static_config.encoder_color_format;
+        EbBool is_16bit = scs_ptr->static_config.encoder_bit_depth > 8 ? EB_TRUE : EB_FALSE;
+        if (scs_ptr->is_16bit_pipeline || is_16bit) {
+            temp_lf_recon_desc_init_data.bit_depth = EB_16BIT;
+            EB_NEW(pcs_ptr->temp_lf_recon_picture16bit_ptr,
+                   svt_recon_picture_buffer_desc_ctor,
+                   (EbPtr)&temp_lf_recon_desc_init_data);
+            if (!is_16bit)
+                pcs_ptr->temp_lf_recon_picture16bit_ptr->bit_depth = EB_8BIT;
+        } else {
+            temp_lf_recon_desc_init_data.bit_depth = EB_8BIT;
+            EB_NEW(pcs_ptr->temp_lf_recon_picture_ptr,
+                   svt_recon_picture_buffer_desc_ctor,
+                   (EbPtr)&temp_lf_recon_desc_init_data);
+        }
         const int32_t last_frame_filter_level[4] = {
             lf->filter_level[0], lf->filter_level[1], lf->filter_level_u, lf->filter_level_v};
-        EbPictureBufferDesc *temp_lf_recon_buffer = (scs_ptr->static_config.is_16bit_pipeline ||
-                                                     scs_ptr->static_config.encoder_bit_depth !=
-                                                         EB_8BIT)
+        EbPictureBufferDesc *temp_lf_recon_buffer = scs_ptr->is_16bit_pipeline
             ? pcs_ptr->temp_lf_recon_picture16bit_ptr
             : pcs_ptr->temp_lf_recon_picture_ptr;
 
@@ -1240,5 +1190,9 @@ void svt_av1_pick_filter_level(
                                                  NULL,
                                                  2,
                                                  0);
+        EB_DELETE(pcs_ptr->temp_lf_recon_picture_ptr);
+        EB_DELETE(pcs_ptr->temp_lf_recon_picture16bit_ptr);
     }
+
+    return EB_ErrorNone;
 }
