@@ -30,6 +30,24 @@ static PartitionType from_shape_to_part[] = {
         PARTITION_SPLIT
 };
 
+IntraSize intra_unit[] =
+{
+    /*Note: e.g for V: there are case where we need the first
+            pixel from left to pad the ref array */
+    {1,1},//DC_PRED
+    {1,1},//V_PRED
+    {1,1},//H_PRED
+    {2,1},//D45_PRED
+    {1,1},//D135_PRED
+    {1,1},//D113_PRED
+    {1,1},//D157_PRED
+    {1,2},//D203_PRED
+    {2,1},//D67_PRED
+    {1,1},//SMOOTH_PRED
+    {1,1},//SMOOTH_V_PRED
+    {1,1},//SMOOTH_H_PRED
+    {2,2} //PAETH_PRED
+};
 
 static int get_filt_type(const MacroBlockD *xd, int plane) {
     int ab_sm, le_sm;
@@ -72,6 +90,8 @@ static void build_intra_predictors(
     const uint8_t *left_ref = left_neigh_array;//CHKN ref - 1;
     DECLARE_ALIGNED(32, uint8_t, left_data[MAX_TX_SIZE * 2 + 48]);
     DECLARE_ALIGNED(32, uint8_t, above_data[MAX_TX_SIZE * 2 + 48]);
+    memset(left_data, 0x80, sizeof(left_data));
+    memset(above_data, 0x80, sizeof(above_data));
     uint8_t *const above_row = above_data + 32;
     uint8_t *const left_col = left_data + 32;
 
@@ -248,6 +268,8 @@ static void build_intra_predictors_high(
 
     DECLARE_ALIGNED(16, uint16_t, left_data[MAX_TX_SIZE * 2 + 32]);
     DECLARE_ALIGNED(16, uint16_t, above_data[MAX_TX_SIZE * 2 + 32]);
+    memset(left_data, 0x80, sizeof(left_data));
+    memset(above_data, 0x80, sizeof(above_data));
     uint16_t *const above_row = above_data + 16;
     uint16_t *const left_col = left_data + 16;
     const int32_t txwpx = tx_size_wide[tx_size];
@@ -722,27 +744,20 @@ EbErrorType svt_av1_intra_prediction_cl(
     (void) hbd_mode_decision;
     EbErrorType return_error = EB_ErrorNone;
 
-    uint32_t mode_type_left_neighbor_index = get_neighbor_array_unit_left_index(
-            md_context_ptr->mode_type_neighbor_array,
-            md_context_ptr->blk_origin_y);
-    uint32_t mode_type_top_neighbor_index = get_neighbor_array_unit_top_index(
-            md_context_ptr->mode_type_neighbor_array,
-            md_context_ptr->blk_origin_x);
-    uint32_t intra_luma_mode_left_neighbor_index = get_neighbor_array_unit_left_index(
-            md_context_ptr->intra_luma_mode_neighbor_array,
-            md_context_ptr->blk_origin_y);
-    uint32_t intra_luma_mode_top_neighbor_index = get_neighbor_array_unit_top_index(
-            md_context_ptr->intra_luma_mode_neighbor_array,
-            md_context_ptr->blk_origin_x);
-    md_context_ptr->intra_luma_left_mode = (uint32_t)(
-            (md_context_ptr->mode_type_neighbor_array->left_array[mode_type_left_neighbor_index] != INTRA_MODE) ? DC_PRED/*EB_INTRA_DC*/ :
-            (uint32_t)md_context_ptr->intra_luma_mode_neighbor_array->left_array[intra_luma_mode_left_neighbor_index]);
+    if (!md_context_ptr->shut_fast_rate) {
 
-    md_context_ptr->intra_luma_top_mode = (uint32_t)(
-            (md_context_ptr->mode_type_neighbor_array->top_array[mode_type_top_neighbor_index] != INTRA_MODE) ? DC_PRED/*EB_INTRA_DC*/ :
-            (uint32_t)md_context_ptr->intra_luma_mode_neighbor_array->top_array[intra_luma_mode_top_neighbor_index]);       //   use DC. This seems like we could use a SB-width
+         MacroBlockD *xd = md_context_ptr->blk_ptr->av1xd;
+        md_context_ptr->intra_luma_left_mode = DC_PRED;
+        md_context_ptr->intra_luma_top_mode  = DC_PRED;
+        if (xd->left_available)
+            md_context_ptr->intra_luma_left_mode =  xd->mi[-1]->mbmi.block_mi.mode >= NEARESTMV ? DC_PRED : xd->mi[-1]->mbmi.block_mi.mode;
+        if (xd->up_available)
+            md_context_ptr->intra_luma_top_mode = xd->mi[-xd->mi_stride]->mbmi.block_mi.mode >= NEARESTMV ? DC_PRED : xd->mi[-xd->mi_stride]->mbmi.block_mi.mode;
+    }
     TxSize  tx_size = md_context_ptr->blk_geom->txsize[candidate_buffer_ptr->candidate_ptr->tx_depth][0]; // Nader - Intra 128x128 not supported
     TxSize  tx_size_chroma = md_context_ptr->blk_geom->txsize_uv[candidate_buffer_ptr->candidate_ptr->tx_depth][0]; //Nader - Intra 128x128 not supported
+    uint32_t sb_size_luma   = pcs_ptr->parent_pcs_ptr->scs_ptr->sb_size_pix;
+    uint32_t sb_size_chroma   = pcs_ptr->parent_pcs_ptr->scs_ptr->sb_size_pix/2;
 
     if(!md_context_ptr->hbd_mode_decision) {
         uint8_t    top_neigh_array[64 * 2 + 1];
@@ -750,42 +765,103 @@ EbErrorType svt_av1_intra_prediction_cl(
         PredictionMode mode;
         // Hsan: plane should be derived @ an earlier stage (e.g. @ the call of perform_fast_loop())
         int32_t start_plane = (md_context_ptr->uv_intra_comp_only) ? 1 : 0;
-        int32_t end_plane = (md_context_ptr->blk_geom->has_uv && md_context_ptr->chroma_level <= CHROMA_MODE_1 && !md_context_ptr->md_staging_skip_chroma_pred) ? (int)MAX_MB_PLANE : 1;
+        int32_t end_plane = md_context_ptr->end_plane;
         for (int32_t plane = start_plane; plane < end_plane; ++plane) {
-            if (plane == 0) {
-                if (md_context_ptr->blk_origin_y != 0)
-                    svt_memcpy(top_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->top_array + md_context_ptr->blk_origin_x, md_context_ptr->blk_geom->bwidth * 2);
-                if (md_context_ptr->blk_origin_x != 0)
-                    svt_memcpy(left_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->left_array + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * 2);
-                if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
-                    top_neigh_array[0] = left_neigh_array[0] = md_context_ptr->luma_recon_neighbor_array->top_left_array[MAX_PICTURE_HEIGHT_SIZE + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y];
-            }
-
-            else if (plane == 1) {
-                if (md_context_ptr->round_origin_y != 0)
-                    svt_memcpy(top_neigh_array + 1, md_context_ptr->cb_recon_neighbor_array->top_array + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * 2);
-
-                if (md_context_ptr->round_origin_x != 0)
-                    svt_memcpy(left_neigh_array + 1, md_context_ptr->cb_recon_neighbor_array->left_array + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * 2);
-
-                if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
-                    top_neigh_array[0] = left_neigh_array[0] = md_context_ptr->cb_recon_neighbor_array->top_left_array[MAX_PICTURE_HEIGHT_SIZE / 2 + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2];
-            }
-            else {
-                if (md_context_ptr->round_origin_y != 0)
-                    svt_memcpy(top_neigh_array + 1, md_context_ptr->cr_recon_neighbor_array->top_array + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * 2);
-
-                if (md_context_ptr->round_origin_x != 0)
-                    svt_memcpy(left_neigh_array + 1, md_context_ptr->cr_recon_neighbor_array->left_array + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * 2);
-
-                if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
-                    top_neigh_array[0] = left_neigh_array[0] = md_context_ptr->cr_recon_neighbor_array->top_left_array[MAX_PICTURE_HEIGHT_SIZE / 2 + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2];
-            }
-
             if (plane)
-                mode = (candidate_buffer_ptr->candidate_ptr->intra_chroma_mode == UV_CFL_PRED) ? (PredictionMode) UV_DC_PRED : (PredictionMode) candidate_buffer_ptr->candidate_ptr->intra_chroma_mode;
+                mode = (candidate_buffer_ptr->candidate_ptr->intra_chroma_mode == UV_CFL_PRED) ? (PredictionMode)UV_DC_PRED : (PredictionMode)candidate_buffer_ptr->candidate_ptr->intra_chroma_mode;
             else
                 mode = candidate_buffer_ptr->candidate_ptr->pred_mode;
+            assert(mode < INTRA_MODES);
+             int ang = plane ? candidate_buffer_ptr->candidate_ptr->angle_delta[PLANE_TYPE_UV] : candidate_buffer_ptr->candidate_ptr->angle_delta[PLANE_TYPE_Y];
+             if (ang==0 ){
+                    IntraSize intra_size = intra_unit[mode];
+                    if (plane == 0) {
+                        if (md_context_ptr->blk_origin_y != 0 && intra_size.top)
+                            svt_memcpy(top_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->top_array + md_context_ptr->blk_origin_x, md_context_ptr->blk_geom->bwidth * intra_size.top);
+                        if (md_context_ptr->blk_origin_x != 0 && intra_size.left){
+                            uint16_t multipler = (md_context_ptr->blk_origin_y % sb_size_luma + md_context_ptr->blk_geom->bheight * intra_size.left) > sb_size_luma ? 1 : intra_size.left;
+                            svt_memcpy(left_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->left_array + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * multipler);
+                        }
+
+
+                        if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
+                            top_neigh_array[0] = left_neigh_array[0] =
+                            md_context_ptr->luma_recon_neighbor_array->top_left_array[md_context_ptr->luma_recon_neighbor_array->max_pic_h + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y];
+                    }
+                    else if (plane == 1) {
+                        if (md_context_ptr->round_origin_y != 0 && intra_size.top)
+                            svt_memcpy(top_neigh_array + 1, md_context_ptr->cb_recon_neighbor_array->top_array + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * intra_size.top);
+
+                        if (md_context_ptr->round_origin_x != 0 && intra_size.left){
+                            uint16_t multipler = ((md_context_ptr->round_origin_y / 2) % sb_size_chroma + md_context_ptr->blk_geom->bheight_uv * intra_size.left) > sb_size_chroma ? 1 : intra_size.left;
+                            svt_memcpy(left_neigh_array + 1, md_context_ptr->cb_recon_neighbor_array->left_array + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * multipler);
+                        }
+
+                        if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
+                            top_neigh_array[0] = left_neigh_array[0] =
+                            md_context_ptr->cb_recon_neighbor_array->top_left_array[md_context_ptr->cb_recon_neighbor_array->max_pic_h + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2];
+                    }
+                    else {
+                        if (md_context_ptr->round_origin_y != 0 && intra_size.top)
+                            svt_memcpy(top_neigh_array + 1, md_context_ptr->cr_recon_neighbor_array->top_array + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * intra_size.top);
+
+                        if (md_context_ptr->round_origin_x != 0 && intra_size.left){
+                            uint16_t multipler = ((md_context_ptr->round_origin_y / 2) % sb_size_chroma + md_context_ptr->blk_geom->bheight_uv * intra_size.left) > sb_size_chroma ? 1 : intra_size.left;
+                            svt_memcpy(left_neigh_array + 1, md_context_ptr->cr_recon_neighbor_array->left_array + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * multipler);
+                        }
+
+                        if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
+                            top_neigh_array[0] = left_neigh_array[0] =
+                            md_context_ptr->cr_recon_neighbor_array->top_left_array[md_context_ptr->cr_recon_neighbor_array->max_pic_h + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2];
+                    }
+             }
+             else {
+
+                 if (plane == 0) {
+                     if (md_context_ptr->blk_origin_y != 0)
+                         svt_memcpy(top_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->top_array + md_context_ptr->blk_origin_x, md_context_ptr->blk_geom->bwidth * 2);
+                     if (md_context_ptr->blk_origin_x != 0){
+                         uint16_t multipler = (md_context_ptr->blk_origin_y % sb_size_luma + md_context_ptr->blk_geom->bheight * 2) > sb_size_luma ? 1 : 2;
+                         svt_memcpy(left_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->left_array + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * multipler);
+                     }
+
+                     if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
+                         top_neigh_array[0] = left_neigh_array[0] =
+                         md_context_ptr->luma_recon_neighbor_array->top_left_array[md_context_ptr->luma_recon_neighbor_array->max_pic_h + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y];
+                 }
+
+                 else if (plane == 1) {
+                     if (md_context_ptr->round_origin_y != 0)
+                         svt_memcpy(top_neigh_array + 1, md_context_ptr->cb_recon_neighbor_array->top_array + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * 2);
+
+                     if (md_context_ptr->round_origin_x != 0){
+                         uint16_t multipler = ((md_context_ptr->round_origin_y / 2) % sb_size_chroma + md_context_ptr->blk_geom->bheight_uv * 2) > sb_size_chroma ? 1 : 2;
+                         svt_memcpy(left_neigh_array + 1, md_context_ptr->cb_recon_neighbor_array->left_array + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * multipler);
+                     }
+
+                     if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
+                         top_neigh_array[0] = left_neigh_array[0] =
+                         md_context_ptr->cb_recon_neighbor_array->top_left_array[md_context_ptr->cb_recon_neighbor_array->max_pic_h + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2];
+
+                 }
+                 else {
+                     if (md_context_ptr->round_origin_y != 0)
+                         svt_memcpy(top_neigh_array + 1, md_context_ptr->cr_recon_neighbor_array->top_array + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * 2);
+
+                     if (md_context_ptr->round_origin_x != 0){
+                         uint16_t multipler = ((md_context_ptr->round_origin_y / 2) % sb_size_chroma + md_context_ptr->blk_geom->bheight_uv * 2) > sb_size_chroma ? 1 : 2;
+                         svt_memcpy(left_neigh_array + 1, md_context_ptr->cr_recon_neighbor_array->left_array + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * multipler);
+                     }
+
+
+                     if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
+                         top_neigh_array[0] = left_neigh_array[0] =
+                         md_context_ptr->cr_recon_neighbor_array->top_left_array[md_context_ptr->cr_recon_neighbor_array->max_pic_h + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2];
+
+                 }
+
+             }
+
 
             svt_av1_predict_intra_block(
                     !ED_STAGE,
@@ -797,7 +873,7 @@ EbErrorType svt_av1_intra_prediction_cl(
                     mode,                                                                           //PredictionMode mode,
                     plane ? candidate_buffer_ptr->candidate_ptr->angle_delta[PLANE_TYPE_UV] : candidate_buffer_ptr->candidate_ptr->angle_delta[PLANE_TYPE_Y],
                     plane==0 ? (candidate_buffer_ptr->candidate_ptr->palette_info ?
-                                    candidate_buffer_ptr->candidate_ptr->palette_info->pmi.palette_size[0]>0 : 0) : 0,
+                                    candidate_buffer_ptr->candidate_ptr->palette_size[0]>0 : 0) : 0,
                     plane==0 ? candidate_buffer_ptr->candidate_ptr->palette_info : NULL,    //MD
                     plane ? FILTER_INTRA_MODES : candidate_buffer_ptr->candidate_ptr->filter_intra_mode,
                     top_neigh_array + 1,
@@ -823,44 +899,106 @@ EbErrorType svt_av1_intra_prediction_cl(
         PredictionMode mode;
         // Hsan: plane should be derived @ an earlier stage (e.g. @ the call of perform_fast_loop())
         int32_t start_plane = (md_context_ptr->uv_intra_comp_only) ? 1 : 0;
-        int32_t end_plane = (md_context_ptr->blk_geom->has_uv && md_context_ptr->chroma_level <= CHROMA_MODE_1 && !md_context_ptr->md_staging_skip_chroma_pred) ? (int)MAX_MB_PLANE : 1;
+        int32_t end_plane =  md_context_ptr->end_plane;
         for (int32_t plane = start_plane; plane < end_plane; ++plane) {
-            if (plane == 0) {
-                if (md_context_ptr->blk_origin_y != 0)
-                    svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_array) + md_context_ptr->blk_origin_x, md_context_ptr->blk_geom->bwidth * 2 * sizeof(uint16_t));
-
-                if (md_context_ptr->blk_origin_x != 0)
-                    svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->left_array) + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * 2 * sizeof(uint16_t));
-
-                if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
-                    top_neigh_array[0] = left_neigh_array[0] = ((uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_left_array) + MAX_PICTURE_HEIGHT_SIZE + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y)[0];
-            }
-            else if (plane == 1) {
-                if (md_context_ptr->round_origin_y != 0)
-                    svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->cb_recon_neighbor_array16bit->top_array) + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * 2 * sizeof(uint16_t));
-
-                if (md_context_ptr->round_origin_x != 0)
-                    svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->cb_recon_neighbor_array16bit->left_array) + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * 2 * sizeof(uint16_t));
-
-                if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
-                    top_neigh_array[0] = left_neigh_array[0] = ((uint16_t*) (md_context_ptr->cb_recon_neighbor_array16bit->top_left_array) + MAX_PICTURE_HEIGHT_SIZE / 2 + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2)[0];
-            }
-            else {
-                if (md_context_ptr->round_origin_y != 0)
-                    svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->cr_recon_neighbor_array16bit->top_array) + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * 2 * sizeof(uint16_t));
-
-                if (md_context_ptr->round_origin_x != 0)
-                    svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->cr_recon_neighbor_array16bit->left_array) + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * 2 * sizeof(uint16_t));
-
-                if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
-                    top_neigh_array[0] = left_neigh_array[0] = ((uint16_t*) (md_context_ptr->cr_recon_neighbor_array16bit->top_left_array) + MAX_PICTURE_HEIGHT_SIZE / 2 + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2)[0];
-            }
-
             if (plane)
-                mode = (candidate_buffer_ptr->candidate_ptr->intra_chroma_mode == UV_CFL_PRED) ? (PredictionMode) UV_DC_PRED : (PredictionMode) candidate_buffer_ptr->candidate_ptr->intra_chroma_mode;
+                mode = (candidate_buffer_ptr->candidate_ptr->intra_chroma_mode == UV_CFL_PRED) ? (PredictionMode)UV_DC_PRED : (PredictionMode)candidate_buffer_ptr->candidate_ptr->intra_chroma_mode;
             else
                 mode = candidate_buffer_ptr->candidate_ptr->pred_mode;
 
+            assert(mode < INTRA_MODES);
+            int ang = plane ? candidate_buffer_ptr->candidate_ptr->angle_delta[PLANE_TYPE_UV] : candidate_buffer_ptr->candidate_ptr->angle_delta[PLANE_TYPE_Y];
+            if (ang == 0) {
+
+                IntraSize intra_size = intra_unit[mode];
+
+                if (plane == 0) {
+                    if (md_context_ptr->blk_origin_y != 0 && intra_size.top)
+                        svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_array) + md_context_ptr->blk_origin_x, md_context_ptr->blk_geom->bwidth * intra_size.top * sizeof(uint16_t));
+
+                    if (md_context_ptr->blk_origin_x != 0 && intra_size.left){
+                        uint16_t multipler = (md_context_ptr->blk_origin_y % sb_size_luma + md_context_ptr->blk_geom->bheight * intra_size.left) > sb_size_luma ? 1 : intra_size.left;
+                            svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->left_array) + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * multipler * sizeof(uint16_t));
+                    }
+
+                    if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
+                        top_neigh_array[0] = left_neigh_array[0] =
+                        ((uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_left_array) + md_context_ptr->luma_recon_neighbor_array16bit->max_pic_h + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y)[0];
+                }
+                else if (plane == 1) {
+                    if (md_context_ptr->round_origin_y != 0 && intra_size.top)
+                        svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->cb_recon_neighbor_array16bit->top_array) + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * intra_size.top * sizeof(uint16_t));
+
+                    if (md_context_ptr->round_origin_x != 0 && intra_size.left){
+                        uint16_t multipler = ((md_context_ptr->round_origin_y / 2) % sb_size_chroma + md_context_ptr->blk_geom->bheight_uv * intra_size.left ) > sb_size_chroma ? 1 : intra_size.left ;
+                        svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->cb_recon_neighbor_array16bit->left_array) + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * multipler * sizeof(uint16_t));
+                    }
+
+                    if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
+                        top_neigh_array[0] = left_neigh_array[0] =
+                        ((uint16_t*)(md_context_ptr->cb_recon_neighbor_array16bit->top_left_array) + md_context_ptr->cb_recon_neighbor_array16bit->max_pic_h + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2)[0];
+
+                }
+                else {
+                    if (md_context_ptr->round_origin_y != 0 && intra_size.top)
+                        svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->cr_recon_neighbor_array16bit->top_array) + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * intra_size.top * sizeof(uint16_t));
+
+                    if (md_context_ptr->round_origin_x != 0 && intra_size.left){
+                        uint16_t multipler = ((md_context_ptr->round_origin_y / 2) % sb_size_chroma + md_context_ptr->blk_geom->bheight_uv * intra_size.left ) > sb_size_chroma ? 1 : intra_size.left ;
+                        svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->cr_recon_neighbor_array16bit->left_array) + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * multipler * sizeof(uint16_t));
+                    }
+
+                    if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
+                        top_neigh_array[0] = left_neigh_array[0] =
+                        ((uint16_t*)(md_context_ptr->cr_recon_neighbor_array16bit->top_left_array) + md_context_ptr->cr_recon_neighbor_array16bit->max_pic_h + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2)[0];
+                }
+            }
+            else {
+
+                if (plane == 0) {
+                    if (md_context_ptr->blk_origin_y != 0)
+                        svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_array) + md_context_ptr->blk_origin_x, md_context_ptr->blk_geom->bwidth * 2 * sizeof(uint16_t));
+
+                    if (md_context_ptr->blk_origin_x != 0){
+                        uint16_t multipler = (md_context_ptr->blk_origin_y % sb_size_luma + md_context_ptr->blk_geom->bheight * 2) > sb_size_luma ? 1 : 2;
+                            svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->left_array) + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * multipler * sizeof(uint16_t));
+                    }
+
+                    if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
+                        top_neigh_array[0] = left_neigh_array[0] =
+                        ((uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_left_array) + md_context_ptr->luma_recon_neighbor_array16bit->max_pic_h + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y)[0];
+                }
+                else if (plane == 1) {
+                    if (md_context_ptr->round_origin_y != 0)
+                        svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->cb_recon_neighbor_array16bit->top_array) + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * 2 * sizeof(uint16_t));
+
+                    if (md_context_ptr->round_origin_x != 0){
+                        uint16_t multipler = ((md_context_ptr->round_origin_y / 2) % sb_size_chroma + md_context_ptr->blk_geom->bheight_uv * 2) > sb_size_chroma ? 1 : 2;
+                            svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->cb_recon_neighbor_array16bit->left_array) + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * multipler * sizeof(uint16_t));
+                    }
+
+                    if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
+                        top_neigh_array[0] = left_neigh_array[0] =
+                        ((uint16_t*)(md_context_ptr->cb_recon_neighbor_array16bit->top_left_array) + md_context_ptr->cb_recon_neighbor_array16bit->max_pic_h + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2)[0];
+
+                }
+                else {
+                    if (md_context_ptr->round_origin_y != 0)
+                        svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->cr_recon_neighbor_array16bit->top_array) + md_context_ptr->round_origin_x / 2, md_context_ptr->blk_geom->bwidth_uv * 2 * sizeof(uint16_t));
+
+                    if (md_context_ptr->round_origin_x != 0){
+                        uint16_t multipler = ((md_context_ptr->round_origin_y / 2) % sb_size_chroma + md_context_ptr->blk_geom->bheight_uv * 2) > sb_size_chroma ? 1 : 2;
+                            svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->cr_recon_neighbor_array16bit->left_array) + md_context_ptr->round_origin_y / 2, md_context_ptr->blk_geom->bheight_uv * multipler * sizeof(uint16_t));
+                    }
+
+                    if (md_context_ptr->round_origin_y != 0 && md_context_ptr->round_origin_x != 0)
+                        top_neigh_array[0] = left_neigh_array[0] =
+                        ((uint16_t*)(md_context_ptr->cr_recon_neighbor_array16bit->top_left_array) + md_context_ptr->cr_recon_neighbor_array16bit->max_pic_h + md_context_ptr->round_origin_x / 2 - md_context_ptr->round_origin_y / 2)[0];
+
+                }
+
+            }
+
             svt_av1_predict_intra_block_16bit(
                     EB_10BIT,
                     !ED_STAGE,
@@ -872,7 +1010,7 @@ EbErrorType svt_av1_intra_prediction_cl(
                     mode,                                                                           //PredictionMode mode,
                     plane ? candidate_buffer_ptr->candidate_ptr->angle_delta[PLANE_TYPE_UV] : candidate_buffer_ptr->candidate_ptr->angle_delta[PLANE_TYPE_Y],
                     plane==0 ? (candidate_buffer_ptr->candidate_ptr->palette_info ?
-                                    candidate_buffer_ptr->candidate_ptr->palette_info->pmi.palette_size[0]>0 : 0) : 0,
+                                    candidate_buffer_ptr->candidate_ptr->palette_size[0]>0 : 0) : 0,
                     plane==0 ? candidate_buffer_ptr->candidate_ptr->palette_info : NULL,    //MD
                     plane ? FILTER_INTRA_MODES : candidate_buffer_ptr->candidate_ptr->filter_intra_mode,
                     top_neigh_array + 1,
@@ -905,29 +1043,19 @@ EbErrorType  intra_luma_prediction_for_interintra(
     EbErrorType return_error = EB_ErrorNone;
     uint8_t is_inter = 0; // set to 0 b/c this is an intra path
 
-    uint32_t mode_type_left_neighbor_index = get_neighbor_array_unit_left_index(
-            md_context_ptr->mode_type_neighbor_array,
-            md_context_ptr->blk_origin_y);
-    uint32_t mode_type_top_neighbor_index = get_neighbor_array_unit_top_index(
-            md_context_ptr->mode_type_neighbor_array,
-            md_context_ptr->blk_origin_x);
-    uint32_t intra_luma_mode_left_neighbor_index = get_neighbor_array_unit_left_index(
-            md_context_ptr->intra_luma_mode_neighbor_array,
-            md_context_ptr->blk_origin_y);
-    uint32_t intra_luma_mode_top_neighbor_index = get_neighbor_array_unit_top_index(
-            md_context_ptr->intra_luma_mode_neighbor_array,
-            md_context_ptr->blk_origin_x);
-
-    md_context_ptr->intra_luma_left_mode = (uint32_t)(
-            (md_context_ptr->mode_type_neighbor_array->left_array[mode_type_left_neighbor_index] != INTRA_MODE) ? DC_PRED:
-            (uint32_t)md_context_ptr->intra_luma_mode_neighbor_array->left_array[intra_luma_mode_left_neighbor_index]);
-
-    md_context_ptr->intra_luma_top_mode = (uint32_t)(
-            (md_context_ptr->mode_type_neighbor_array->top_array[mode_type_top_neighbor_index] != INTRA_MODE) ? DC_PRED:
-            (uint32_t)md_context_ptr->intra_luma_mode_neighbor_array->top_array[intra_luma_mode_top_neighbor_index]);       //   use DC. This seems like we could use a SB-width
+    if (!md_context_ptr->shut_fast_rate) {
 
+        MacroBlockD *xd = md_context_ptr->blk_ptr->av1xd;
+        md_context_ptr->intra_luma_left_mode = DC_PRED;
+        md_context_ptr->intra_luma_top_mode = DC_PRED;
+        if (xd->left_available)
+            md_context_ptr->intra_luma_left_mode = xd->mi[-1]->mbmi.block_mi.mode >= NEARESTMV ? DC_PRED : xd->mi[-1]->mbmi.block_mi.mode;
+        if (xd->up_available)
+            md_context_ptr->intra_luma_top_mode = xd->mi[-xd->mi_stride]->mbmi.block_mi.mode >= NEARESTMV ? DC_PRED : xd->mi[-xd->mi_stride]->mbmi.block_mi.mode;
+    }
     TxSize  tx_size = md_context_ptr->blk_geom->txsize[0][0];  //CHKN  TOcheck
     PredictionMode mode = interintra_to_intra_mode[interintra_mode];
+    uint32_t        sb_size_luma   = pcs_ptr->parent_pcs_ptr->scs_ptr->sb_size_pix;
 
     if (!md_context_ptr->hbd_mode_decision) {
         uint8_t    top_neigh_array[64 * 2 + 1];
@@ -935,11 +1063,14 @@ EbErrorType  intra_luma_prediction_for_interintra(
 
         if (md_context_ptr->blk_origin_y != 0)
             svt_memcpy(top_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->top_array + md_context_ptr->blk_origin_x, md_context_ptr->blk_geom->bwidth * 2);
-        if (md_context_ptr->blk_origin_x != 0)
-            svt_memcpy(left_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->left_array + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * 2);
-        if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
-            top_neigh_array[0] = left_neigh_array[0] = md_context_ptr->luma_recon_neighbor_array->top_left_array[MAX_PICTURE_HEIGHT_SIZE + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y];
+        if (md_context_ptr->blk_origin_x != 0){
+            uint16_t multipler = (md_context_ptr->blk_origin_y % sb_size_luma + md_context_ptr->blk_geom->bheight * 2) > sb_size_luma ? 1 : 2;
+            svt_memcpy(left_neigh_array + 1, md_context_ptr->luma_recon_neighbor_array->left_array + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * multipler);
+        }
 
+        if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
+            top_neigh_array[0] = left_neigh_array[0] =
+            md_context_ptr->luma_recon_neighbor_array->top_left_array[md_context_ptr->luma_recon_neighbor_array->max_pic_h + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y];
         svt_av1_predict_intra_block(
                 !ED_STAGE,
                 md_context_ptr->blk_geom,
@@ -973,10 +1104,14 @@ EbErrorType  intra_luma_prediction_for_interintra(
 
         if (md_context_ptr->blk_origin_y != 0)
             svt_memcpy(top_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_array) + md_context_ptr->blk_origin_x, md_context_ptr->blk_geom->bwidth * 2 * sizeof(uint16_t));
-        if (md_context_ptr->blk_origin_x != 0)
-            svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->left_array) + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * 2 * sizeof(uint16_t));
+        if (md_context_ptr->blk_origin_x != 0){
+            uint16_t multipler = (md_context_ptr->blk_origin_y % sb_size_luma + md_context_ptr->blk_geom->bheight * 2) > sb_size_luma ? 1 : 2;
+            svt_memcpy(left_neigh_array + 1, (uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->left_array) + md_context_ptr->blk_origin_y, md_context_ptr->blk_geom->bheight * multipler * sizeof(uint16_t));
+        }
+
         if (md_context_ptr->blk_origin_y != 0 && md_context_ptr->blk_origin_x != 0)
-            top_neigh_array[0] = left_neigh_array[0] = ((uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_left_array) + MAX_PICTURE_HEIGHT_SIZE + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y)[0];
+            top_neigh_array[0] = left_neigh_array[0] =
+            ((uint16_t*)(md_context_ptr->luma_recon_neighbor_array16bit->top_left_array) + md_context_ptr->luma_recon_neighbor_array16bit->max_pic_h + md_context_ptr->blk_origin_x - md_context_ptr->blk_origin_y)[0];
 
         svt_av1_predict_intra_block_16bit(
                 EB_10BIT,
