@@ -189,6 +189,11 @@ extern "C" {
     extern void av1_estimate_syntax_rate(
         MdRateEstimationContext      *md_rate_estimation_array,
         EbBool                          is_i_slice,
+        uint8_t pic_filter_intra_level,
+        uint8_t allow_screen_content_tools,
+        uint8_t enable_restoration,
+        uint8_t allow_intrabc,
+        uint8_t partition_contexts,
         FRAME_CONTEXT                  *fc);
     /**************************************************************************
     * Estimate the rate of the quantised coefficient
