@@ -22,7 +22,7 @@ extern "C" {
      **************************************/
 EbErrorType packetization_context_ctor(EbThreadContext *  thread_context_ptr,
                                        const EbEncHandle *enc_handle_ptr, int rate_control_index,
-                                       int demux_index);
+                                       int demux_index, int me_port_index);
 
 extern void *packetization_kernel(void *input_ptr);
 #ifdef __cplusplus
