@@ -18,7 +18,8 @@
  * Extern Function Declarations
  **************************************/
 extern EbErrorType rest_context_ctor(EbThreadContext *  thread_context_ptr,
-                                     const EbEncHandle *enc_handle_ptr, int index, int demux_index);
+                                     const EbEncHandle *enc_handle_ptr, EbPtr object_init_data_ptr,
+                                     int index, int demux_index);
 
 extern void *rest_kernel(void *input_ptr);
 
