@@ -607,6 +607,7 @@ static SgrprojInfo search_selfguided_restoration(
     int8_t end_ep   = sg_ref_frame_ep[0] < 0 && sg_ref_frame_ep[1] < 0
           ? SGRPROJ_PARAMS
           : AOMMIN(SGRPROJ_PARAMS, mid_ep + step);
+    UNUSED(sg_frame_ep_cnt);
 
     for (ep = start_ep; ep < end_ep; ep++) {
         int32_t exq[2];
@@ -664,7 +665,6 @@ static SgrprojInfo search_selfguided_restoration(
             bestxqd[1] = exqd[1];
         }
     }
-    sg_frame_ep_cnt[bestep]++;
 
     SgrprojInfo ret;
     ret.ep     = bestep;
@@ -696,10 +696,10 @@ static int32_t count_sgrproj_bits(SgrprojInfo *sgrproj_info, SgrprojInfo *ref_sg
 int8_t get_sg_step(int8_t sg_filter_mode) {
     int8_t step;
     switch (sg_filter_mode) {
-    case 1: step = 0; break;
-    case 2: step = 1; break;
-    case 3: step = 4; break;
-    case 4: step = 16; break;
+    case 1: step = 16; break;
+    case 2: step = 4; break;
+    case 3: step = 1; break;
+    case 4: step = 0; break;
     default: step = 16; break;
     }
     return step;
@@ -1092,109 +1092,115 @@ static int32_t count_wiener_bits(int32_t wiener_win, WienerInfo *wiener_info,
 }
 
 #define USE_WIENER_REFINEMENT_SEARCH 1
+/* Perform refinement search around inital wiener filter coeffs passed in rui->wiener_info;
+   compute and return the SSE of the best filter parameters.
+*/
 static int64_t finer_tile_search_wiener_seg(const RestSearchCtxt *       rsc,
                                             const RestorationTileLimits *limits,
                                             const Av1PixelRect *tile, RestorationUnitInfo *rui,
                                             int32_t wiener_win) {
+    const Av1Common *const cm = rsc->cm;
     const int32_t plane_off = (WIENER_WIN - wiener_win) >> 1;
     int64_t       err       = try_restoration_unit_seg(rsc, limits, tile, rui);
 #if USE_WIENER_REFINEMENT_SEARCH
-    int64_t err2;
-    int32_t tap_min[] = {WIENER_FILT_TAP0_MINV, WIENER_FILT_TAP1_MINV, WIENER_FILT_TAP2_MINV};
-    int32_t tap_max[] = {WIENER_FILT_TAP0_MAXV, WIENER_FILT_TAP1_MAXV, WIENER_FILT_TAP2_MAXV};
-
     WienerInfo *plane_wiener = &rui->wiener_info;
 
     // SVT_LOG("err  pre = %"PRId64"\n", err);
-    const int32_t start_step = 4;
-    for (int32_t s = start_step; s >= 1; s >>= 1) {
-        for (int32_t p = plane_off; p < WIENER_HALFWIN; ++p) {
-            int32_t skip = 0;
-            do {
-                if (plane_wiener->hfilter[p] - s >= tap_min[p]) {
-                    plane_wiener->hfilter[p] -= (int16_t)s;
-                    plane_wiener->hfilter[WIENER_WIN - p - 1] -= (int16_t)s;
-                    plane_wiener->hfilter[WIENER_HALFWIN] += 2 * (int16_t)s;
-                    err2 = try_restoration_unit_seg(rsc, limits, tile, rui);
-                    if (err2 > err) {
-                        plane_wiener->hfilter[p] += (int16_t)s;
-                        plane_wiener->hfilter[WIENER_WIN - p - 1] += (int16_t)s;
-                        plane_wiener->hfilter[WIENER_HALFWIN] -= 2 * (int16_t)s;
-                    } else {
-                        err  = err2;
-                        skip = 1;
-                        // At the highest step size continue moving in the same direction
-                        if (s == start_step)
-                            continue;
-                    }
-                }
-                break;
-            } while (1);
-            if (skip)
-                break;
-            do {
-                if (plane_wiener->hfilter[p] + s <= tap_max[p]) {
-                    plane_wiener->hfilter[p] += (int16_t)s;
-                    plane_wiener->hfilter[WIENER_WIN - p - 1] += (int16_t)s;
-                    plane_wiener->hfilter[WIENER_HALFWIN] -= 2 * (int16_t)s;
-                    err2 = try_restoration_unit_seg(rsc, limits, tile, rui);
-                    if (err2 > err) {
+    if (cm->wn_filter_ctrls.use_refinement) {
+        const int32_t start_step = 4;
+        const int32_t end_step   = cm->wn_filter_ctrls.max_one_refinement_step ? 4 : 1;
+        int64_t       err2;
+        int32_t tap_min[] = {WIENER_FILT_TAP0_MINV, WIENER_FILT_TAP1_MINV, WIENER_FILT_TAP2_MINV};
+        int32_t tap_max[] = {WIENER_FILT_TAP0_MAXV, WIENER_FILT_TAP1_MAXV, WIENER_FILT_TAP2_MAXV};
+        for (int32_t s = start_step; s >= end_step; s >>= 1) {
+            for (int32_t p = plane_off; p < WIENER_HALFWIN; ++p) {
+                int32_t skip = 0;
+                do {
+                    if (plane_wiener->hfilter[p] - s >= tap_min[p]) {
                         plane_wiener->hfilter[p] -= (int16_t)s;
                         plane_wiener->hfilter[WIENER_WIN - p - 1] -= (int16_t)s;
                         plane_wiener->hfilter[WIENER_HALFWIN] += 2 * (int16_t)s;
-                    } else {
-                        err = err2;
-                        // At the highest step size continue moving in the same direction
-                        if (s == start_step)
-                            continue;
+                        err2 = try_restoration_unit_seg(rsc, limits, tile, rui);
+                        if (err2 > err) {
+                            plane_wiener->hfilter[p] += (int16_t)s;
+                            plane_wiener->hfilter[WIENER_WIN - p - 1] += (int16_t)s;
+                            plane_wiener->hfilter[WIENER_HALFWIN] -= 2 * (int16_t)s;
+                        } else {
+                            err  = err2;
+                            skip = 1;
+                            // At the highest step size continue moving in the same direction
+                            if (s == start_step && !cm->wn_filter_ctrls.max_one_refinement_step)
+                                continue;
+                        }
                     }
-                }
-                break;
-            } while (1);
-        }
-        for (int32_t p = plane_off; p < WIENER_HALFWIN; ++p) {
-            int32_t skip = 0;
-            do {
-                if (plane_wiener->vfilter[p] - s >= tap_min[p]) {
-                    plane_wiener->vfilter[p] -= (int16_t)s;
-                    plane_wiener->vfilter[WIENER_WIN - p - 1] -= (int16_t)s;
-                    plane_wiener->vfilter[WIENER_HALFWIN] += 2 * (int16_t)s;
-                    err2 = try_restoration_unit_seg(rsc, limits, tile, rui);
-                    if (err2 > err) {
-                        plane_wiener->vfilter[p] += (int16_t)s;
-                        plane_wiener->vfilter[WIENER_WIN - p - 1] += (int16_t)s;
-                        plane_wiener->vfilter[WIENER_HALFWIN] -= 2 * (int16_t)s;
-                    } else {
-                        err  = err2;
-                        skip = 1;
-                        // At the highest step size continue moving in the same direction
-                        if (s == start_step)
-                            continue;
+                    break;
+                } while (1);
+                if (skip)
+                    break;
+                do {
+                    if (plane_wiener->hfilter[p] + s <= tap_max[p]) {
+                        plane_wiener->hfilter[p] += (int16_t)s;
+                        plane_wiener->hfilter[WIENER_WIN - p - 1] += (int16_t)s;
+                        plane_wiener->hfilter[WIENER_HALFWIN] -= 2 * (int16_t)s;
+                        err2 = try_restoration_unit_seg(rsc, limits, tile, rui);
+                        if (err2 > err) {
+                            plane_wiener->hfilter[p] -= (int16_t)s;
+                            plane_wiener->hfilter[WIENER_WIN - p - 1] -= (int16_t)s;
+                            plane_wiener->hfilter[WIENER_HALFWIN] += 2 * (int16_t)s;
+                        } else {
+                            err = err2;
+                            // At the highest step size continue moving in the same direction
+                            if (s == start_step && !cm->wn_filter_ctrls.max_one_refinement_step)
+                                continue;
+                        }
                     }
-                }
-                break;
-            } while (1);
-            if (skip)
-                break;
-            do {
-                if (plane_wiener->vfilter[p] + s <= tap_max[p]) {
-                    plane_wiener->vfilter[p] += (int16_t)s;
-                    plane_wiener->vfilter[WIENER_WIN - p - 1] += (int16_t)s;
-                    plane_wiener->vfilter[WIENER_HALFWIN] -= 2 * (int16_t)s;
-                    err2 = try_restoration_unit_seg(rsc, limits, tile, rui);
-                    if (err2 > err) {
+                    break;
+                } while (1);
+            }
+            for (int32_t p = plane_off; p < WIENER_HALFWIN; ++p) {
+                int32_t skip = 0;
+                do {
+                    if (plane_wiener->vfilter[p] - s >= tap_min[p]) {
                         plane_wiener->vfilter[p] -= (int16_t)s;
                         plane_wiener->vfilter[WIENER_WIN - p - 1] -= (int16_t)s;
                         plane_wiener->vfilter[WIENER_HALFWIN] += 2 * (int16_t)s;
-                    } else {
-                        err = err2;
-                        // At the highest step size continue moving in the same direction
-                        if (s == start_step)
-                            continue;
+                        err2 = try_restoration_unit_seg(rsc, limits, tile, rui);
+                        if (err2 > err) {
+                            plane_wiener->vfilter[p] += (int16_t)s;
+                            plane_wiener->vfilter[WIENER_WIN - p - 1] += (int16_t)s;
+                            plane_wiener->vfilter[WIENER_HALFWIN] -= 2 * (int16_t)s;
+                        } else {
+                            err  = err2;
+                            skip = 1;
+                            // At the highest step size continue moving in the same direction
+                            if (s == start_step && !cm->wn_filter_ctrls.max_one_refinement_step)
+                                continue;
+                        }
                     }
-                }
-                break;
-            } while (1);
+                    break;
+                } while (1);
+                if (skip)
+                    break;
+                do {
+                    if (plane_wiener->vfilter[p] + s <= tap_max[p]) {
+                        plane_wiener->vfilter[p] += (int16_t)s;
+                        plane_wiener->vfilter[WIENER_WIN - p - 1] += (int16_t)s;
+                        plane_wiener->vfilter[WIENER_HALFWIN] -= 2 * (int16_t)s;
+                        err2 = try_restoration_unit_seg(rsc, limits, tile, rui);
+                        if (err2 > err) {
+                            plane_wiener->vfilter[p] -= (int16_t)s;
+                            plane_wiener->vfilter[WIENER_WIN - p - 1] -= (int16_t)s;
+                            plane_wiener->vfilter[WIENER_HALFWIN] += 2 * (int16_t)s;
+                        } else {
+                            err = err2;
+                            // At the highest step size continue moving in the same direction
+                            if (s == start_step && !cm->wn_filter_ctrls.max_one_refinement_step)
+                                continue;
+                        }
+                    }
+                    break;
+                } while (1);
+            }
         }
     }
     // SVT_LOG("err post = %"PRId64"\n", err);
@@ -1277,6 +1283,7 @@ static int32_t rest_tiles_in_plane(const Av1Common *cm, int32_t plane) {
 void *svt_aom_memalign(size_t align, size_t size);
 void  svt_aom_free(void *memblk);
 
+/* Perform search for the best self-guided filter parameters and compute the SSE. */
 static void search_sgrproj_seg(const RestorationTileLimits *limits, const Av1PixelRect *tile,
                                int32_t rest_unit_idx, void *priv) {
     RestSearchCtxt *    rsc  = (RestSearchCtxt *)priv;
@@ -1311,6 +1318,9 @@ static void search_sgrproj_seg(const RestorationTileLimits *limits, const Av1Pix
                                                   cm->sg_ref_frame_ep,
                                                   cm->sg_frame_ep_cnt,
                                                   step);
+    svt_block_on_mutex(cm->child_pcs->rest_search_mutex);
+    cm->sg_frame_ep_cnt[rusi->sgrproj.ep]++;
+    svt_release_mutex(cm->child_pcs->rest_search_mutex);
 
     RestorationUnitInfo rui;
     rui.restoration_type = RESTORE_SGRPROJ;
@@ -1319,6 +1329,7 @@ static void search_sgrproj_seg(const RestorationTileLimits *limits, const Av1Pix
     rusi->sse[RESTORE_SGRPROJ] = try_restoration_unit_seg(rsc, limits, tile, &rui);
 }
 
+/* Get the cost/SSE/rate of using self-guided filtering for a given restoration unit. */
 static void search_sgrproj_finish(const RestorationTileLimits *limits, const Av1PixelRect *tile,
                                   int32_t rest_unit_idx, void *priv) {
     (void)limits;
@@ -1347,72 +1358,83 @@ static void search_sgrproj_finish(const RestorationTileLimits *limits, const Av1
         rsc->sgrproj = rusi->sgrproj;
 }
 
+/*Get the best Wiender filter parameters and SSE.*/
 static void search_wiener_seg(const RestorationTileLimits *limits, const Av1PixelRect *tile_rect,
                               int32_t rest_unit_idx, void *priv) {
-    RestSearchCtxt *       rsc        = (RestSearchCtxt *)priv;
-    RestUnitSearchInfo *   rusi       = &rsc->rusi[rest_unit_idx];
-    const Av1Common *const cm         = rsc->cm;
-    int32_t                wn_luma    = cm->wn_filter_mode == 1 ? WIENER_WIN_3TAP
-                          : cm->wn_filter_mode == 2             ? WIENER_WIN_CHROMA
-                                                                : WIENER_WIN;
-    const int32_t          wiener_win = cm->wn_filter_mode == 1 ? WIENER_WIN_3TAP
-                 : (rsc->plane == AOM_PLANE_Y)                  ? wn_luma
-                                                                : WIENER_WIN_CHROMA;
-    EB_ALIGN(32) int64_t   M[WIENER_WIN2];
-    EB_ALIGN(32) int64_t   H[WIENER_WIN2 * WIENER_WIN2];
-    int32_t                vfilterd[WIENER_WIN], hfilterd[WIENER_WIN];
+    RestSearchCtxt *       rsc  = (RestSearchCtxt *)priv;
+    RestUnitSearchInfo *   rusi = &rsc->rusi[rest_unit_idx];
+    const Av1Common *const cm   = rsc->cm;
+    int32_t wn_luma = cm->wn_filter_ctrls.filter_tap_lvl == 1 ? WIENER_WIN
+        : cm->wn_filter_ctrls.filter_tap_lvl == 2             ? WIENER_WIN_CHROMA
+                                                              : WIENER_WIN_3TAP;
 
-    if (cm->use_highbitdepth)
-        svt_av1_compute_stats_highbd(wiener_win,
-                                     rsc->dgd_buffer,
-                                     rsc->src_buffer,
-                                     limits->h_start,
-                                     limits->h_end,
-                                     limits->v_start,
-                                     limits->v_end,
-                                     rsc->dgd_stride,
-                                     rsc->src_stride,
-                                     M,
-                                     H,
-                                     (AomBitDepth)cm->bit_depth);
-    else
-        svt_av1_compute_stats(wiener_win,
-                              rsc->dgd_buffer,
-                              rsc->src_buffer,
-                              limits->h_start,
-                              limits->h_end,
-                              limits->v_start,
-                              limits->v_end,
-                              rsc->dgd_stride,
-                              rsc->src_stride,
-                              M,
-                              H);
-
-    if (!wiener_decompose_sep_sym(wiener_win, M, H, vfilterd, hfilterd)) {
-        SVT_LOG("CHKN never get here\n");
-        rusi->best_rtype[RESTORE_WIENER - 1] = RESTORE_NONE;
-        rusi->sse[RESTORE_WIENER]            = INT64_MAX;
-        return;
-    }
+    const int32_t wiener_win = (rsc->plane == AOM_PLANE_Y) ? wn_luma
+                                                           : MIN(wn_luma, WIENER_WIN_CHROMA);
 
     RestorationUnitInfo rui;
     memset(&rui, 0, sizeof(rui));
     rui.restoration_type = RESTORE_WIENER;
-    finalize_sym_filter(wiener_win, vfilterd, rui.wiener_info.vfilter);
-    finalize_sym_filter(wiener_win, hfilterd, rui.wiener_info.hfilter);
+    // Check whether you can use the filter coeffs from previous frames; if not, must generate new coeffs
+    if (cm->wn_filter_ctrls.use_prev_frame_coeffs &&
+        (cm->current_frame.frame_type != KEY_FRAME &&
+         cm->current_frame.frame_type != INTRA_ONLY_FRAME) &&
+        cm->child_pcs->rst_info[rsc->plane].unit_info[rest_unit_idx].restoration_type ==
+            RESTORE_WIENER) {
+        // Copy filter info, stored from previous frame(s)
+        rui.wiener_info = cm->child_pcs->rst_info[rsc->plane].unit_info[rest_unit_idx].wiener_info;
+    } else {
+        EB_ALIGN(32) int64_t M[WIENER_WIN2];
+        EB_ALIGN(32) int64_t H[WIENER_WIN2 * WIENER_WIN2];
+        int32_t              vfilterd[WIENER_WIN], hfilterd[WIENER_WIN];
 
-    // Filter score computes the value of the function x'*A*x - x'*b for the
-    // learned filter and compares it against identity filer. If there is no
-    // reduction in the function, the filter is reverted back to identity
-    if (compute_score(wiener_win, M, H, rui.wiener_info.vfilter, rui.wiener_info.hfilter) > 0) {
-        rusi->sse[RESTORE_WIENER] = INT64_MAX;
-        return;
-    }
+        if (cm->use_highbitdepth)
+            svt_av1_compute_stats_highbd(wiener_win,
+                                         rsc->dgd_buffer,
+                                         rsc->src_buffer,
+                                         limits->h_start,
+                                         limits->h_end,
+                                         limits->v_start,
+                                         limits->v_end,
+                                         rsc->dgd_stride,
+                                         rsc->src_stride,
+                                         M,
+                                         H,
+                                         (AomBitDepth)cm->bit_depth);
+        else
+            svt_av1_compute_stats(wiener_win,
+                                  rsc->dgd_buffer,
+                                  rsc->src_buffer,
+                                  limits->h_start,
+                                  limits->h_end,
+                                  limits->v_start,
+                                  limits->v_end,
+                                  rsc->dgd_stride,
+                                  rsc->src_stride,
+                                  M,
+                                  H);
+
+        if (!wiener_decompose_sep_sym(wiener_win, M, H, vfilterd, hfilterd)) {
+            SVT_LOG("CHKN never get here\n");
+            rusi->best_rtype[RESTORE_WIENER - 1] = RESTORE_NONE;
+            rusi->sse[RESTORE_WIENER]            = INT64_MAX;
+            return;
+        }
+        finalize_sym_filter(wiener_win, vfilterd, rui.wiener_info.vfilter);
+        finalize_sym_filter(wiener_win, hfilterd, rui.wiener_info.hfilter);
+
+        // Filter score computes the value of the function x'*A*x - x'*b for the
+        // learned filter and compares it against identity filer. If there is no
+        // reduction in the function, the filter is reverted back to identity
+        if (compute_score(wiener_win, M, H, rui.wiener_info.vfilter, rui.wiener_info.hfilter) > 0) {
+            rusi->sse[RESTORE_WIENER] = INT64_MAX;
+            return;
+        }
 
 #ifdef ARCH_X86_64
-    aom_clear_system_state();
+        aom_clear_system_state();
 #endif
-
+    }
+    // Perform refinement search for filter coeffs and compute SSE
     rusi->sse[RESTORE_WIENER] = finer_tile_search_wiener_seg(
         rsc, limits, tile_rect, &rui, wiener_win);
     rusi->wiener = rui.wiener_info;
@@ -1422,20 +1444,19 @@ static void search_wiener_seg(const RestorationTileLimits *limits, const Av1Pixe
         assert(rui.wiener_info.hfilter[0] == 0 && rui.wiener_info.hfilter[WIENER_WIN - 1] == 0);
     }
 }
+/* Get the cost/SSE/rate of using wiener filtering for a given restoration unit. */
 static void search_wiener_finish(const RestorationTileLimits *limits, const Av1PixelRect *tile_rect,
                                  int32_t rest_unit_idx, void *priv) {
     (void)limits;
     (void)tile_rect;
-    RestSearchCtxt *       rsc        = (RestSearchCtxt *)priv;
-    RestUnitSearchInfo *   rusi       = &rsc->rusi[rest_unit_idx];
-    const Av1Common *const cm         = rsc->cm;
-    int32_t                wn_luma    = cm->wn_filter_mode == 1 ? WIENER_WIN_3TAP
-                          : cm->wn_filter_mode == 2             ? WIENER_WIN_CHROMA
-                                                                : WIENER_WIN;
-    const int32_t          wiener_win = cm->wn_filter_mode == 1 ? WIENER_WIN_3TAP
-                 : (rsc->plane == AOM_PLANE_Y)                  ? wn_luma
-                                                                : WIENER_WIN_CHROMA;
-
+    RestSearchCtxt *       rsc  = (RestSearchCtxt *)priv;
+    RestUnitSearchInfo *   rusi = &rsc->rusi[rest_unit_idx];
+    const Av1Common *const cm   = rsc->cm;
+    int32_t       wn_luma    = cm->wn_filter_ctrls.filter_tap_lvl == 1 ? WIENER_WIN
+                 : cm->wn_filter_ctrls.filter_tap_lvl == 2             ? WIENER_WIN_CHROMA
+                                                                       : WIENER_WIN_3TAP;
+    const int32_t wiener_win = (rsc->plane == AOM_PLANE_Y) ? wn_luma
+                                                           : MIN(wn_luma, WIENER_WIN_CHROMA);
     const Macroblock *const x         = rsc->x;
     const int64_t           bits_none = x->wiener_restore_cost[0];
 
@@ -1487,6 +1508,7 @@ static void search_norestore_seg(const RestorationTileLimits *limits, const Av1P
     rusi->sse[RESTORE_NONE] = sse_restoration_unit(
         limits, rsc->src, rsc->cm->frame_to_show, rsc->plane, highbd);
 }
+// Get the SSE for a resotration unit with no filtering applied
 static void search_norestore_finish(const RestorationTileLimits *limits,
                                     const Av1PixelRect *tile_rect, int32_t rest_unit_idx,
                                     void *priv) {
@@ -1500,6 +1522,7 @@ static void search_norestore_finish(const RestorationTileLimits *limits,
 
     rsc->sse += rusi->sse[RESTORE_NONE];
 }
+/* Get the cost/SSE/rate for the entire frame associated with using the passed restoration filter type. */
 static double search_rest_type_finish(RestSearchCtxt *rsc, RestorationType rtype) {
     static const RestUnitVisitor funs[RESTORE_TYPES] = {
         search_norestore_finish, search_wiener_finish, search_sgrproj_finish, search_switchable};
@@ -1510,7 +1533,11 @@ static double search_rest_type_finish(RestSearchCtxt *rsc, RestorationType rtype
 
     return RDCOST_DBL(rsc->x->rdmult, rsc->bits >> 4, rsc->sse);
 }
+/* Search the available type of restoration filters: OFF, Wiener, and self-guided.
 
+   The search will return the best parameters for each type of filter and their associated SSEs,
+   to be used in the final decision for filtering the frame.
+*/
 void restoration_seg_search(int32_t *rst_tmpbuf, Yv12BufferConfig *org_fts,
                             const Yv12BufferConfig *src, Yv12BufferConfig *trial_frame_rst,
                             PictureControlSet *pcs_ptr, uint32_t segment_index) {
@@ -1536,13 +1563,18 @@ void restoration_seg_search(int32_t *rst_tmpbuf, Yv12BufferConfig *org_fts,
         rsc_p->tmpbuf = rst_tmpbuf;
 
         const int32_t highbd = rsc.cm->use_highbitdepth;
-        svt_extend_frame(rsc.dgd_buffer,
-                         rsc.plane_width,
-                         rsc.plane_height,
-                         rsc.dgd_stride,
-                         RESTORATION_BORDER,
-                         RESTORATION_BORDER,
-                         highbd);
+        svt_block_on_mutex(pcs_ptr->rest_search_mutex);
+        if (!pcs_ptr->rest_extend_flag[plane]) {
+            svt_extend_frame(rsc.dgd_buffer,
+                             rsc.plane_width,
+                             rsc.plane_height,
+                             rsc.dgd_stride,
+                             RESTORATION_BORDER,
+                             RESTORATION_BORDER,
+                             highbd);
+            pcs_ptr->rest_extend_flag[plane] = EB_TRUE;
+        }
+        svt_release_mutex(pcs_ptr->rest_search_mutex);
 
         av1_foreach_rest_unit_in_frame_seg(rsc_p->cm,
                                            rsc_p->plane,
@@ -1552,7 +1584,7 @@ void restoration_seg_search(int32_t *rst_tmpbuf, Yv12BufferConfig *org_fts,
                                            pcs_ptr->rest_segments_column_count,
                                            pcs_ptr->rest_segments_row_count,
                                            segment_index);
-        if (cm->wn_filter_mode)
+        if (cm->wn_filter_ctrls.enabled)
             av1_foreach_rest_unit_in_frame_seg(rsc_p->cm,
                                                rsc_p->plane,
                                                rsc_on_tile,
@@ -1561,23 +1593,28 @@ void restoration_seg_search(int32_t *rst_tmpbuf, Yv12BufferConfig *org_fts,
                                                pcs_ptr->rest_segments_column_count,
                                                pcs_ptr->rest_segments_row_count,
                                                segment_index);
-        av1_foreach_rest_unit_in_frame_seg(rsc_p->cm,
-                                           rsc_p->plane,
-                                           rsc_on_tile,
-                                           search_sgrproj_seg,
-                                           rsc_p,
-                                           pcs_ptr->rest_segments_column_count,
-                                           pcs_ptr->rest_segments_row_count,
-                                           segment_index);
+        if (cm->sg_filter_mode)
+            av1_foreach_rest_unit_in_frame_seg(rsc_p->cm,
+                                               rsc_p->plane,
+                                               rsc_on_tile,
+                                               search_sgrproj_seg,
+                                               rsc_p,
+                                               pcs_ptr->rest_segments_column_count,
+                                               pcs_ptr->rest_segments_row_count,
+                                               segment_index);
     }
 }
-
-void rest_finish_search(PictureControlSet *pcs_ptr){
-    Macroblock *x = pcs_ptr->parent_pcs_ptr->av1x;
-    Av1Common *const cm = pcs_ptr->parent_pcs_ptr->av1_cm;
+/* Given the best parameters for each type of filter and their associated SSEs,
+   decide which filter should be used for each filter block.
+*/
+void rest_finish_search(PictureControlSet *pcs_ptr) {
+    Macroblock *             x         = pcs_ptr->parent_pcs_ptr->av1x;
+    Av1Common *const         cm        = pcs_ptr->parent_pcs_ptr->av1_cm;
     PictureParentControlSet *p_pcs_ptr = pcs_ptr->parent_pcs_ptr;
-    RestorationType force_restore_type_d = (cm->wn_filter_mode) ? RESTORE_TYPES : RESTORE_SGRPROJ;
-    int32_t         ntiles[2];
+    RestorationType force_restore_type_d = (cm->wn_filter_ctrls.enabled)
+        ? ((cm->sg_filter_mode) ? RESTORE_TYPES : RESTORE_WIENER)
+        : ((cm->sg_filter_mode) ? RESTORE_SGRPROJ : RESTORE_NONE);
+    int32_t ntiles[2];
     for (int32_t is_uv = 0; is_uv < 2; ++is_uv) ntiles[is_uv] = rest_tiles_in_plane(cm, is_uv);
 
     assert(ntiles[1] <= ntiles[0]);
