@@ -50,10 +50,9 @@ EbErrorType svt_sequence_control_set_ctor(SequenceControlSet *scs_ptr, EbPtr obj
         object_init_data_ptr;
     uint32_t segment_index;
     scs_ptr->mvrate_set = 0;
-    scs_ptr->dctor = svt_sequence_control_set_dctor;
+    scs_ptr->dctor      = svt_sequence_control_set_dctor;
 
-    scs_ptr->static_config.sb_sz           = 64;
-    scs_ptr->static_config.partition_depth = 4;
+    scs_ptr->sb_sz           = 64;
 
     // Segments
     for (segment_index = 0; segment_index < MAX_TEMPORAL_LAYERS; ++segment_index) {
@@ -87,7 +86,8 @@ EbErrorType svt_sequence_control_set_ctor(SequenceControlSet *scs_ptr, EbPtr obj
     scs_ptr->max_ref_count = 1;
 
     // SB
-    scs_ptr->sb_sz        = 64;
+    scs_ptr->sb_sz = 64;
+    scs_ptr->super_block_size        = 128;
     scs_ptr->max_sb_depth = 3;
 
     // CU
@@ -95,14 +95,38 @@ EbErrorType svt_sequence_control_set_ctor(SequenceControlSet *scs_ptr, EbPtr obj
     scs_ptr->min_blk_size   = 8;
     scs_ptr->max_intra_size = 32;
     scs_ptr->min_intra_size = 8;
-    // Rate Control
-    scs_ptr->target_bitrate = 0x1000;
     // Quantization
     scs_ptr->static_config.qp = 20;
+    // AV1 features
+    scs_ptr->enable_warped_motion = DEFAULT;
+    scs_ptr->enable_global_motion = 1;
+    scs_ptr->sg_filter_mode = DEFAULT;
+    scs_ptr->wn_filter_mode = DEFAULT;
+    scs_ptr->inter_intra_compound = DEFAULT;
+    scs_ptr->enable_paeth = DEFAULT;
+    scs_ptr->enable_smooth = DEFAULT;
+    scs_ptr->spatial_sse_full_loop_level = DEFAULT;
+    scs_ptr->over_bndry_blk = DEFAULT;
+    scs_ptr->new_nearest_comb_inject = DEFAULT;
+    scs_ptr->nsq_table = DEFAULT;
+    scs_ptr->frame_end_cdf_update = DEFAULT;
+    scs_ptr->set_chroma_mode = DEFAULT;
+    scs_ptr->disable_cfl_flag = DEFAULT;
+    scs_ptr->obmc_level = DEFAULT;
+    scs_ptr->rdoq_level = DEFAULT;
+    scs_ptr->pred_me = DEFAULT;
+    scs_ptr->bipred_3x3_inject = DEFAULT;
+    scs_ptr->compound_level = DEFAULT;
+    scs_ptr->filter_intra_level = DEFAULT;
+    scs_ptr->enable_intra_edge_filter = DEFAULT;
+    scs_ptr->pic_based_rate_est = DEFAULT;
+    scs_ptr->ext_block_flag = EB_FALSE;
+    scs_ptr->ten_bit_format = EB_FALSE;
+
+
     // Initialize SB params
-    EB_MALLOC_ARRAY(scs_ptr->sb_params_array,
-                    ((MAX_PICTURE_WIDTH_SIZE + scs_ptr->sb_sz - 1) / scs_ptr->sb_sz) *
-                        ((MAX_PICTURE_HEIGHT_SIZE + scs_ptr->sb_sz - 1) / scs_ptr->sb_sz));
+    //allocation will happen in ress-corrd
+    scs_ptr->sb_params_array = 0;
 
     scs_ptr->seq_header.frame_width_bits              = 16;
     scs_ptr->seq_header.frame_height_bits             = 16;
@@ -114,14 +138,11 @@ EbErrorType svt_sequence_control_set_ctor(SequenceControlSet *scs_ptr, EbPtr obj
     if (scs_init_data && scs_init_data->sb_size == 128) {
         scs_ptr->seq_header.sb_size = BLOCK_128X128;
         scs_ptr->sb_size_pix        = 128;
-        scs_ptr->max_block_cnt      = 4421;
-
         scs_ptr->seq_header.sb_mi_size   = 32; // Size of the superblock in units of MI blocks
         scs_ptr->seq_header.sb_size_log2 = 5;
     } else {
         scs_ptr->seq_header.sb_size = BLOCK_64X64;
         scs_ptr->sb_size_pix        = 64;
-        scs_ptr->max_block_cnt      = 1101;
 
         scs_ptr->seq_header.sb_mi_size   = 16; // Size of the superblock in units of MI blocks
         scs_ptr->seq_header.sb_size_log2 = 4;
@@ -146,18 +167,6 @@ EbErrorType svt_sequence_control_set_ctor(SequenceControlSet *scs_ptr, EbPtr obj
     // 2 - adaptive
 
     scs_ptr->seq_header.order_hint_info.enable_ref_frame_mvs = 1;
-#if NO_ENCDEC
-    if (scs_ptr->static_config.cdef_mode == DEFAULT)
-        scs_ptr->seq_header.enable_cdef = 0;
-    else
-        scs_ptr->seq_header.enable_cdef = (uint8_t)(scs_ptr->static_config.cdef_mode > 0);
-
-    if (scs_ptr->static_config.enable_restoration_filtering == DEFAULT)
-        scs_ptr->seq_header.enable_restoration = 0;
-    else
-        scs_ptr->seq_header.enable_restoration =
-            (uint8_t)scs_ptr->static_config.enable_restoration_filtering;
-#else
     if (scs_ptr->static_config.cdef_level == DEFAULT)
         scs_ptr->seq_header.cdef_level = 1;
     else
@@ -168,24 +177,21 @@ EbErrorType svt_sequence_control_set_ctor(SequenceControlSet *scs_ptr, EbPtr obj
     else
         scs_ptr->seq_header.enable_restoration =
             (uint8_t)scs_ptr->static_config.enable_restoration_filtering;
-#endif
 
-    if (scs_ptr->static_config.enable_intra_edge_filter == DEFAULT)
+    if (scs_ptr->enable_intra_edge_filter == DEFAULT)
         scs_ptr->seq_header.enable_intra_edge_filter = 1;
     else
         scs_ptr->seq_header.enable_intra_edge_filter =
-            (uint8_t)scs_ptr->static_config.enable_intra_edge_filter;
+            (uint8_t)scs_ptr->enable_intra_edge_filter;
 
-    if (scs_ptr->static_config.pic_based_rate_est == DEFAULT)
+    if (scs_ptr->pic_based_rate_est == DEFAULT)
         scs_ptr->seq_header.pic_based_rate_est = 0;
     else
-        scs_ptr->seq_header.pic_based_rate_est = (uint8_t)scs_ptr->static_config.pic_based_rate_est;
-
-    if (scs_ptr->static_config.enable_warped_motion == DEFAULT)
+        scs_ptr->seq_header.pic_based_rate_est = (uint8_t)scs_ptr->pic_based_rate_est;
+    if (scs_ptr->enable_warped_motion == DEFAULT)
         scs_ptr->seq_header.enable_warped_motion = 1;
     else
-        scs_ptr->seq_header.enable_warped_motion = (uint8_t)
-                                                       scs_ptr->static_config.enable_warped_motion;
+        scs_ptr->seq_header.enable_warped_motion = (uint8_t)scs_ptr->enable_warped_motion;
 
     scs_ptr->film_grain_random_seed = 7391;
     scs_ptr->reference_count        = 4;
@@ -199,6 +205,23 @@ EbErrorType svt_sequence_control_set_ctor(SequenceControlSet *scs_ptr, EbPtr obj
     scs_ptr->picture_analysis_number_of_regions_per_height =
         HIGHER_THAN_CLASS_1_REGION_SPLIT_PER_HEIGHT;
 
+    scs_ptr->palette_level = DEFAULT;
+
+    // intra angle delta
+    scs_ptr->intra_angle_delta = DEFAULT;
+
+    scs_ptr->is_16bit_pipeline = EB_FALSE;
+
+    scs_ptr->intrabc_mode = DEFAULT;
+
+    scs_ptr->enable_qp_scaling_flag = 1;
+
+    scs_ptr->enable_adaptive_mini_gop = 0;
+
+    scs_ptr->max_heirachical_level = 5;
+
+    scs_ptr->speed_control_flag = 0;
+
     return EB_ErrorNone;
 }
 
@@ -239,23 +262,20 @@ EbErrorType copy_sequence_control_set(SequenceControlSet *dst, SequenceControlSe
     dst->subsampling_x                             = src->subsampling_x;
     dst->subsampling_y                             = src->subsampling_y;
     dst->pred_struct_ptr                           = src->pred_struct_ptr;
-    dst->intra_period_length                       = src->intra_period_length;
-    dst->intra_refresh_type                        = src->intra_refresh_type;
     dst->max_ref_count                             = src->max_ref_count;
+    dst->super_block_size                          = src->super_block_size;
     dst->sb_sz                                     = src->sb_sz;
     dst->max_sb_depth                              = src->max_sb_depth;
     dst->max_blk_size                              = src->max_blk_size;
     dst->min_blk_size                              = src->min_blk_size;
     dst->max_intra_size                            = src->max_intra_size;
     dst->min_intra_size                            = src->min_intra_size;
-    dst->target_bitrate                            = src->target_bitrate;
     dst->static_config.qp                          = src->static_config.qp;
-    dst->film_grain_denoise_strength               = src->film_grain_denoise_strength;
     dst->seq_header.film_grain_params_present      = src->seq_header.film_grain_params_present;
     dst->picture_control_set_pool_init_count       = src->picture_control_set_pool_init_count;
     dst->me_pool_init_count                        = src->me_pool_init_count;
     dst->picture_control_set_pool_init_count_child = src->picture_control_set_pool_init_count_child;
-    dst->enc_dec_pool_init_count                 = src->enc_dec_pool_init_count;
+    dst->enc_dec_pool_init_count                   = src->enc_dec_pool_init_count;
     dst->pa_reference_picture_buffer_init_count    = src->pa_reference_picture_buffer_init_count;
     dst->reference_picture_buffer_init_count       = src->reference_picture_buffer_init_count;
     dst->input_buffer_fifo_init_count              = src->input_buffer_fifo_init_count;
@@ -296,8 +316,9 @@ EbErrorType copy_sequence_control_set(SequenceControlSet *dst, SequenceControlSe
         dst->tile_group_col_count_array[i]      = src->tile_group_col_count_array[i];
         dst->tile_group_row_count_array[i]      = src->tile_group_row_count_array[i];
     }
-    dst->tpl_segment_col_count_array  = src->tpl_segment_col_count_array;
-    dst->tpl_segment_row_count_array  = src->tpl_segment_row_count_array;
+
+    dst->tpl_segment_col_count_array = src->tpl_segment_col_count_array;
+    dst->tpl_segment_row_count_array = src->tpl_segment_row_count_array;
 
     dst->cdef_segment_column_count = src->cdef_segment_column_count;
     dst->cdef_segment_row_count    = src->cdef_segment_row_count;
@@ -316,10 +337,60 @@ EbErrorType copy_sequence_control_set(SequenceControlSet *dst, SequenceControlSe
     dst->enable_dec_order               = src->enable_dec_order;
     dst->lap_enabled                    = src->lap_enabled;
 
-    dst->lad_mg                         = src->lad_mg;
-    dst->use_boundaries_in_rest_search  = src->use_boundaries_in_rest_search;
-    dst->mrp_init_level = src->mrp_init_level;
-    dst->enc_mode_2ndpass = src->enc_mode_2ndpass;
+    dst->lad_mg = src->lad_mg;
+    dst->tpl_lad_mg = src->tpl_lad_mg;
+    dst->use_boundaries_in_rest_search = src->use_boundaries_in_rest_search;
+    dst->geom_idx      = src->geom_idx;
+    dst->max_block_cnt = src->max_block_cnt;
+    dst->rc_stat_gen_pass_mode = src->rc_stat_gen_pass_mode;
+    dst->cqp_base_q_tf = src->cqp_base_q_tf;
+    dst->cqp_base_q    = src->cqp_base_q;
+    dst->is_short_clip = src->is_short_clip;
+    dst->input_resolution = src->input_resolution;
+    dst->tf_params_per_type[0] = src->tf_params_per_type[0];
+    dst->tf_params_per_type[1] = src->tf_params_per_type[1];
+    dst->tf_params_per_type[2] = src->tf_params_per_type[2];
+    dst->mrp_ctrls = src->mrp_ctrls;
+    dst->passes            = src->passes;
+    dst->ipp_pass_ctrls    = src->ipp_pass_ctrls;
+    dst->mid_pass_ctrls    = src->mid_pass_ctrls;
+    dst->ipp_was_ds        = src->ipp_was_ds;
+    dst->final_pass_preset = src->final_pass_preset;
+    dst->palette_level                 = src->palette_level;
+    dst->intra_angle_delta             = src->intra_angle_delta;
+    dst->is_16bit_pipeline = src->is_16bit_pipeline;
+    dst->enable_warped_motion = src->enable_warped_motion;
+    dst->enable_global_motion = src->enable_global_motion;
+    dst->sg_filter_mode = src->sg_filter_mode;
+    dst->wn_filter_mode = src->wn_filter_mode;
+    dst->inter_intra_compound = src->inter_intra_compound;
+    dst->enable_paeth = src->enable_paeth;
+    dst->enable_smooth = src->enable_smooth;
+    dst->spatial_sse_full_loop_level = src->spatial_sse_full_loop_level;
+    dst->over_bndry_blk = src->spatial_sse_full_loop_level;
+    dst->new_nearest_comb_inject = src->new_nearest_comb_inject;
+    dst->nsq_table = src->nsq_table;
+    dst->frame_end_cdf_update = src->frame_end_cdf_update;
+    dst->set_chroma_mode = src->set_chroma_mode;
+    dst->disable_cfl_flag = src->disable_cfl_flag;
+    dst->obmc_level = src->obmc_level;
+    dst->rdoq_level = src->rdoq_level;
+    dst->pred_me = src->pred_me;
+    dst->bipred_3x3_inject = src->bipred_3x3_inject;
+    dst->compound_level = src->compound_level;
+    dst->filter_intra_level = src->filter_intra_level;
+    dst->enable_intra_edge_filter = src->enable_intra_edge_filter;
+    dst->pic_based_rate_est = src->pic_based_rate_est;
+    dst->ext_block_flag = src->ext_block_flag;
+    dst->intrabc_mode = src->intrabc_mode;
+    dst->enable_hbd_mode_decision      = src->enable_hbd_mode_decision;
+    dst->enable_qp_scaling_flag        = src->enable_qp_scaling_flag;
+    dst->ten_bit_format                = src->ten_bit_format;
+    dst->enable_adaptive_mini_gop      = src->enable_adaptive_mini_gop;
+    dst->max_heirachical_level         = src->max_heirachical_level;
+    dst->speed_control_flag            = src->speed_control_flag;
+    dst->tpl_level = src->tpl_level;
+
     return EB_ErrorNone;
 }
 
@@ -371,10 +442,11 @@ extern EbErrorType sb_params_init(SequenceControlSet *scs_ptr) {
     EbErrorType return_error = EB_ErrorNone;
     uint16_t    sb_index;
     uint16_t    raster_scan_blk_index;
-    uint8_t     picture_sb_width = (uint8_t)(
-        (scs_ptr->seq_header.max_frame_width + scs_ptr->sb_sz - 1) / scs_ptr->sb_sz);
-    uint8_t picture_sb_height = (uint8_t)(
-        (scs_ptr->seq_header.max_frame_height + scs_ptr->sb_sz - 1) / scs_ptr->sb_sz);
+
+    uint16_t picture_sb_width = (scs_ptr->seq_header.max_frame_width + scs_ptr->sb_sz - 1) /
+        scs_ptr->sb_sz;
+    uint16_t picture_sb_height = (scs_ptr->seq_header.max_frame_height + scs_ptr->sb_sz - 1) /
+        scs_ptr->sb_sz;
     //free old one;
     EB_FREE_ARRAY(scs_ptr->sb_params_array);
 
@@ -441,6 +513,10 @@ extern EbErrorType sb_params_init(SequenceControlSet *scs_ptr) {
     return return_error;
 }
 
+EbErrorType rtime_alloc_sb_geom(SequenceControlSet *scs_ptr, uint32_t size) {
+    EB_MALLOC_ARRAY(scs_ptr->sb_geom, size);
+    return EB_ErrorNone;
+}
 EbErrorType sb_geom_init(SequenceControlSet *scs_ptr) {
     uint16_t sb_index;
     uint16_t md_scan_block_index;
@@ -450,7 +526,7 @@ EbErrorType sb_geom_init(SequenceControlSet *scs_ptr) {
         scs_ptr->sb_size_pix;
 
     EB_FREE_ARRAY(scs_ptr->sb_geom);
-    EB_MALLOC_ARRAY(scs_ptr->sb_geom, picture_sb_width * picture_sb_height);
+    rtime_alloc_sb_geom(scs_ptr, picture_sb_width * picture_sb_height);
 
     for (sb_index = 0; sb_index < picture_sb_width * picture_sb_height; ++sb_index) {
         scs_ptr->sb_geom[sb_index].horizontal_index = sb_index % picture_sb_width;
