@@ -18,11 +18,12 @@
 /***************************************
  * Extern Function Declaration
  ***************************************/
-EbErrorType tpl_disp_context_ctor(EbThreadContext *thread_context_ptr,
-                             const EbEncHandle *enc_handle_ptr, int index, int tasks_index);
+EbErrorType  tpl_disp_context_ctor(EbThreadContext *  thread_context_ptr,
+                                   const EbEncHandle *enc_handle_ptr, int index, int tasks_index);
 extern void *tpl_disp_kernel(void *input_ptr);
-EbErrorType source_based_operations_context_ctor(EbThreadContext *  thread_context_ptr,
-                                                 const EbEncHandle *enc_handle_ptr, int index);
+EbErrorType  source_based_operations_context_ctor(EbThreadContext *  thread_context_ptr,
+                                                  const EbEncHandle *enc_handle_ptr, int tpl_index,
+                                                  int index);
 
 extern void *source_based_operations_kernel(void *input_ptr);
 
