@@ -58,6 +58,11 @@ static INLINE int svt_mv_err_cost(const MV *mv, const MV *ref_mv, const int *mvj
     case MV_COST_L1_LOWRES: return (SSE_LAMBDA_LOWRES * (abs_diff.row + abs_diff.col)) >> 3;
     case MV_COST_L1_MIDRES: return (SSE_LAMBDA_MIDRES * (abs_diff.row + abs_diff.col)) >> 3;
     case MV_COST_L1_HDRES: return (SSE_LAMBDA_HDRES * (abs_diff.row + abs_diff.col)) >> 3;
+    case MV_COST_OPT: {
+        return (int)ROUND_POWER_OF_TWO_64(
+            (int64_t)((abs_diff.row + abs_diff.col) << 8) * error_per_bit,
+            RDDIV_BITS + AV1_PROB_COST_SHIFT - RD_EPB_SHIFT + PIXEL_TRANSFORM_ERROR_SCALE);
+    }
     case MV_COST_NONE: return 0;
     default: assert(0 && "Invalid rd_cost_type"); return 0;
     }
@@ -144,16 +149,16 @@ static int svt_upsampled_pref_error(MacroBlockD *xd, const struct AV1Common *con
 
 // Estimates the variance of prediction residue using bilinear filter for fast
 // search.
-static INLINE int svt_estimated_pref_error(
-    const MV *this_mv, const SUBPEL_SEARCH_VAR_PARAMS *var_params,
-    unsigned int *sse) {
+static INLINE int svt_estimated_pref_error(const MV *                      this_mv,
+                                           const SUBPEL_SEARCH_VAR_PARAMS *var_params,
+                                           unsigned int *                  sse) {
     const AomVarianceFnPtr *vfp = var_params->vfp;
 
     const MSBuffers *ms_buffers = &var_params->ms_buffers;
-    const uint8_t *src = ms_buffers->src->buf;
-    const uint8_t *ref = svt_get_buf_from_mv(ms_buffers->ref, *this_mv);
-    const int src_stride = ms_buffers->src->stride;
-    const int ref_stride = ms_buffers->ref->stride;
+    const uint8_t *  src        = ms_buffers->src->buf;
+    const uint8_t *  ref        = svt_get_buf_from_mv(ms_buffers->ref, *this_mv);
+    const int        src_stride = ms_buffers->src->stride;
+    const int        ref_stride = ms_buffers->ref->stride;
     //const uint8_t *second_pred = ms_buffers->second_pred;
     //const uint8_t *mask = ms_buffers->mask;
     //const int mask_stride = ms_buffers->mask_stride;
@@ -176,38 +181,40 @@ static INLINE int svt_estimated_pref_error(
     //}
 }
 
-
 // Estimates whether this_mv is better than best_mv. This function incorporates
 // both prediction error and residue into account. It is suffixed "fast" because
 // it uses bilinear filter to estimate the prediction.
 static INLINE unsigned int svt_check_better_fast(
     MacroBlockD *xd, const struct AV1Common *const cm, const MV *this_mv, MV *best_mv,
     const SubpelMvLimits *mv_limits, const SUBPEL_SEARCH_VAR_PARAMS *var_params,
-    const MV_COST_PARAMS *mv_cost_params, unsigned int *besterr,
-    unsigned int *sse1, int *distortion, int *has_better_mv, int is_scaled) {
+    const MV_COST_PARAMS *mv_cost_params, unsigned int *besterr, unsigned int *sse1,
+    int *distortion, int *has_better_mv, int is_scaled) {
     unsigned int cost;
     if (svt_av1_is_subpelmv_in_range(mv_limits, *this_mv)) {
         unsigned int sse;
-        int thismse;
+        int          thismse;
+        cost = svt_mv_err_cost_(this_mv, mv_cost_params);
+        if (mv_cost_params->mv_cost_type == MV_COST_OPT) {
+            int64_t bestcost = *distortion + cost;
+            if (bestcost > (((int64_t)*besterr * (int64_t)mv_cost_params->early_exit_th) / 1000))
+                return (uint32_t)bestcost;
+        }
         // TODO: add estimated func
         if (is_scaled) {
             thismse = svt_upsampled_pref_error(xd, cm, this_mv, var_params, &sse);
-        }
-        else {
+        } else {
             thismse = svt_estimated_pref_error(this_mv, var_params, &sse);
         }
-        cost = svt_mv_err_cost_(this_mv, mv_cost_params);
         cost += thismse;
 
         if (cost < *besterr) {
-            *besterr = cost;
-            *best_mv = *this_mv;
+            *besterr    = cost;
+            *best_mv    = *this_mv;
             *distortion = thismse;
-            *sse1 = sse;
+            *sse1       = sse;
             *has_better_mv |= 1;
         }
-    }
-    else {
+    } else {
         cost = INT_MAX;
     }
     return cost;
@@ -391,39 +398,21 @@ static AOM_FORCE_INLINE void svt_second_level_check_v2(
 }
 
 // Gets the error at the beginning when the mv has fullpel precision
-static unsigned int svt_upsampled_setup_center_error(MacroBlockD *                   xd,
-                                                     const struct AV1Common *const   cm,
-                                                     const MV *                      bestmv,
+static unsigned int svt_upsampled_setup_center_error(const MV *                      bestmv,
                                                      const SUBPEL_SEARCH_VAR_PARAMS *var_params,
                                                      const MV_COST_PARAMS *          mv_cost_params,
-                                                     unsigned int *sse1, int *distortion) {
-    unsigned int besterr = svt_upsampled_pref_error(xd, cm, bestmv, var_params, sse1);
-    *distortion          = besterr;
-    besterr += svt_mv_err_cost_(bestmv, mv_cost_params);
-    return besterr;
+                                                     unsigned int *                  distortion) {
+    const MSBuffers *ms_buffers = &var_params->ms_buffers;
+    const uint8_t *  ref        = svt_get_buf_from_mv(ms_buffers->ref, *bestmv);
+    *distortion                 = var_params->vfp->vf(
+        ref, ms_buffers->ref->stride, ms_buffers->src->buf, ms_buffers->src->stride, distortion);
+    return *distortion + svt_mv_err_cost_(bestmv, mv_cost_params);
 }
 
-// Checks the list of mvs searched in the last iteration and see if we are
-// repeating it. If so, return 1. Otherwise we update the last_mv_search_list
-// with current_mv and return 0.
-static INLINE int svt_check_repeated_mv_and_update(int_mv *last_mv_search_list, const MV current_mv,
-                                                   int iter) {
-    if (last_mv_search_list) {
-        if (CHECK_MV_EQUAL(last_mv_search_list[iter].as_mv, current_mv)) {
-            return 1;
-        }
-
-        last_mv_search_list[iter].as_mv = current_mv;
-    }
-    return 0;
-}
-
-static INLINE MV get_best_diag_step(int step_size, unsigned int left_cost,
-    unsigned int right_cost,
-    unsigned int up_cost,
-    unsigned int down_cost) {
-    const MV diag_step = { up_cost <= down_cost ? -step_size : step_size,
-                           left_cost <= right_cost ? -step_size : step_size };
+static INLINE MV get_best_diag_step(int step_size, unsigned int left_cost, unsigned int right_cost,
+                                    unsigned int up_cost, unsigned int down_cost) {
+    const MV diag_step = {up_cost <= down_cost ? -step_size : step_size,
+                          left_cost <= right_cost ? -step_size : step_size};
 
     return diag_step;
 }
@@ -432,41 +421,85 @@ static INLINE MV get_best_diag_step(int step_size, unsigned int left_cost,
 // search in the best quadrant. This uses bilinear filter to speed up the
 // calculation.
 static AOM_FORCE_INLINE MV first_level_check_fast(
-    MacroBlockD *xd, const struct AV1Common *const cm, const MV this_mv, MV *best_mv,
-    int hstep, const SubpelMvLimits *mv_limits,
-    const SUBPEL_SEARCH_VAR_PARAMS *var_params,
-    const MV_COST_PARAMS *mv_cost_params, unsigned int *besterr,
+    MacroBlockD *xd, const struct AV1Common *const cm, const MV this_mv, MV *best_mv, int hstep,
+    const SubpelMvLimits *mv_limits, const SUBPEL_SEARCH_VAR_PARAMS *var_params,
+    const MV_COST_PARAMS *mv_cost_params, unsigned int *besterr, unsigned int orgerr,
     unsigned int *sse1, int *distortion, int is_scaled) {
     // Check the four cardinal directions
-    const MV left_mv = { this_mv.row, this_mv.col - hstep };
-    int dummy = 0;
-    const unsigned int left = svt_check_better_fast(
-        xd, cm, &left_mv, best_mv, mv_limits, var_params, mv_cost_params, besterr,
-        sse1, distortion, &dummy, is_scaled);
+    const MV           left_mv = {this_mv.row, this_mv.col - hstep};
+    int                dummy   = 0;
+    const unsigned int left    = svt_check_better_fast(xd,
+                                                    cm,
+                                                    &left_mv,
+                                                    best_mv,
+                                                    mv_limits,
+                                                    var_params,
+                                                    mv_cost_params,
+                                                    besterr,
+                                                    sse1,
+                                                    distortion,
+                                                    &dummy,
+                                                    is_scaled);
 
-    const MV right_mv = { this_mv.row, this_mv.col + hstep };
-    const unsigned int right = svt_check_better_fast(
-        xd, cm, &right_mv, best_mv, mv_limits, var_params, mv_cost_params,
-        besterr, sse1, distortion, &dummy, is_scaled);
+    const MV           right_mv = {this_mv.row, this_mv.col + hstep};
+    const unsigned int right    = svt_check_better_fast(xd,
+                                                     cm,
+                                                     &right_mv,
+                                                     best_mv,
+                                                     mv_limits,
+                                                     var_params,
+                                                     mv_cost_params,
+                                                     besterr,
+                                                     sse1,
+                                                     distortion,
+                                                     &dummy,
+                                                     is_scaled);
 
-    const MV top_mv = { this_mv.row - hstep, this_mv.col };
-    const unsigned int up = svt_check_better_fast(
-        xd, cm, &top_mv, best_mv, mv_limits, var_params, mv_cost_params, besterr,
-        sse1, distortion, &dummy, is_scaled);
+    const MV           top_mv = {this_mv.row - hstep, this_mv.col};
+    const unsigned int up     = svt_check_better_fast(xd,
+                                                  cm,
+                                                  &top_mv,
+                                                  best_mv,
+                                                  mv_limits,
+                                                  var_params,
+                                                  mv_cost_params,
+                                                  besterr,
+                                                  sse1,
+                                                  distortion,
+                                                  &dummy,
+                                                  is_scaled);
 
-    const MV bottom_mv = { this_mv.row + hstep, this_mv.col };
-    const unsigned int down = svt_check_better_fast(
-        xd, cm, &bottom_mv, best_mv, mv_limits, var_params, mv_cost_params,
-        besterr, sse1, distortion, &dummy, is_scaled);
+    const MV           bottom_mv = {this_mv.row + hstep, this_mv.col};
+    const unsigned int down      = svt_check_better_fast(xd,
+                                                    cm,
+                                                    &bottom_mv,
+                                                    best_mv,
+                                                    mv_limits,
+                                                    var_params,
+                                                    mv_cost_params,
+                                                    besterr,
+                                                    sse1,
+                                                    distortion,
+                                                    &dummy,
+                                                    is_scaled);
 
     const MV diag_step = get_best_diag_step(hstep, left, right, up, down);
-    const MV diag_mv = { this_mv.row + diag_step.row,
-                         this_mv.col + diag_step.col };
-
+    const MV diag_mv   = {this_mv.row + diag_step.row, this_mv.col + diag_step.col};
+    if (*besterr >= orgerr)
+        return diag_step;
     // Check the diagonal direction with the best mv
-    svt_check_better_fast(xd, cm, &diag_mv, best_mv, mv_limits, var_params,
-        mv_cost_params, besterr, sse1, distortion, &dummy,
-        is_scaled);
+    svt_check_better_fast(xd,
+                          cm,
+                          &diag_mv,
+                          best_mv,
+                          mv_limits,
+                          var_params,
+                          mv_cost_params,
+                          besterr,
+                          sse1,
+                          distortion,
+                          &dummy,
+                          is_scaled);
 
     return diag_step;
 }
@@ -481,59 +514,129 @@ static AOM_FORCE_INLINE void second_level_check_fast(
     unsigned int *sse1, int *distortion, int is_scaled) {
     assert(diag_step.row == hstep || diag_step.row == -hstep);
     assert(diag_step.col == hstep || diag_step.col == -hstep);
-    const int tr = this_mv.row;
-    const int tc = this_mv.col;
-    const int br = best_mv->row;
-    const int bc = best_mv->col;
-    int dummy = 0;
+    const int tr    = this_mv.row;
+    const int tc    = this_mv.col;
+    const int br    = best_mv->row;
+    const int bc    = best_mv->col;
+    int       dummy = 0;
     if (tr != br && tc != bc) {
         assert(diag_step.col == bc - tc);
         assert(diag_step.row == br - tr);
-        const MV chess_mv_1 = { br, bc + diag_step.col };
-        const MV chess_mv_2 = { br + diag_step.row, bc };
-        svt_check_better_fast(xd, cm, &chess_mv_1, best_mv, mv_limits, var_params,
-            mv_cost_params, besterr, sse1, distortion, &dummy,
-            is_scaled);
+        const MV chess_mv_1 = {br, bc + diag_step.col};
+        const MV chess_mv_2 = {br + diag_step.row, bc};
+        svt_check_better_fast(xd,
+                              cm,
+                              &chess_mv_1,
+                              best_mv,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              besterr,
+                              sse1,
+                              distortion,
+                              &dummy,
+                              is_scaled);
 
-        svt_check_better_fast(xd, cm, &chess_mv_2, best_mv, mv_limits, var_params,
-            mv_cost_params, besterr, sse1, distortion, &dummy,
-            is_scaled);
-    }
-    else if (tr == br && tc != bc) {
+        svt_check_better_fast(xd,
+                              cm,
+                              &chess_mv_2,
+                              best_mv,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              besterr,
+                              sse1,
+                              distortion,
+                              &dummy,
+                              is_scaled);
+    } else if (tr == br && tc != bc) {
         assert(diag_step.col == bc - tc);
         // Continue searching in the best direction
-        const MV bottom_long_mv = { br + hstep, bc + diag_step.col };
-        const MV top_long_mv = { br - hstep, bc + diag_step.col };
-        svt_check_better_fast(xd, cm, &bottom_long_mv, best_mv, mv_limits, var_params,
-            mv_cost_params, besterr, sse1, distortion, &dummy,
-            is_scaled);
-        svt_check_better_fast(xd, cm, &top_long_mv, best_mv, mv_limits, var_params,
-            mv_cost_params, besterr, sse1, distortion, &dummy,
-            is_scaled);
+        const MV bottom_long_mv = {br + hstep, bc + diag_step.col};
+        const MV top_long_mv    = {br - hstep, bc + diag_step.col};
+        svt_check_better_fast(xd,
+                              cm,
+                              &bottom_long_mv,
+                              best_mv,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              besterr,
+                              sse1,
+                              distortion,
+                              &dummy,
+                              is_scaled);
+        svt_check_better_fast(xd,
+                              cm,
+                              &top_long_mv,
+                              best_mv,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              besterr,
+                              sse1,
+                              distortion,
+                              &dummy,
+                              is_scaled);
 
         // Search in the direction opposite of the best quadrant
-        const MV rev_mv = { br - diag_step.row, bc };
-        svt_check_better_fast(xd, cm, &rev_mv, best_mv, mv_limits, var_params,
-            mv_cost_params, besterr, sse1, distortion, &dummy,
-            is_scaled);
-    }
-    else if (tr != br && tc == bc) {
+        const MV rev_mv = {br - diag_step.row, bc};
+        svt_check_better_fast(xd,
+                              cm,
+                              &rev_mv,
+                              best_mv,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              besterr,
+                              sse1,
+                              distortion,
+                              &dummy,
+                              is_scaled);
+    } else if (tr != br && tc == bc) {
         assert(diag_step.row == br - tr);
         // Continue searching in the best direction
-        const MV right_long_mv = { br + diag_step.row, bc + hstep };
-        const MV left_long_mv = { br + diag_step.row, bc - hstep };
-        svt_check_better_fast(xd, cm, &right_long_mv, best_mv, mv_limits, var_params,
-            mv_cost_params, besterr, sse1, distortion, &dummy,
-            is_scaled);
-        svt_check_better_fast(xd, cm, &left_long_mv, best_mv, mv_limits, var_params,
-            mv_cost_params, besterr, sse1, distortion, &dummy,
-            is_scaled);
+        const MV right_long_mv = {br + diag_step.row, bc + hstep};
+        const MV left_long_mv  = {br + diag_step.row, bc - hstep};
+        svt_check_better_fast(xd,
+                              cm,
+                              &right_long_mv,
+                              best_mv,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              besterr,
+                              sse1,
+                              distortion,
+                              &dummy,
+                              is_scaled);
+        svt_check_better_fast(xd,
+                              cm,
+                              &left_long_mv,
+                              best_mv,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              besterr,
+                              sse1,
+                              distortion,
+                              &dummy,
+                              is_scaled);
 
         // Search in the direction opposite of the best quadrant
-        const MV rev_mv = { br, bc - diag_step.col };
-        svt_check_better_fast(xd, cm, &rev_mv, best_mv, mv_limits, var_params,
-            mv_cost_params, besterr, sse1, distortion, &dummy,
-            is_scaled);
+        const MV rev_mv = {br, bc - diag_step.col};
+        svt_check_better_fast(xd,
+                              cm,
+                              &rev_mv,
+                              best_mv,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              besterr,
+                              sse1,
+                              distortion,
+                              &dummy,
+                              is_scaled);
     }
 }
 
@@ -541,161 +644,131 @@ static AOM_FORCE_INLINE void second_level_check_fast(
 // searches the four cardinal directions, and perform several
 // diagonal/chess-pattern searches in the best quadrant.
 static AOM_FORCE_INLINE void two_level_checks_fast(
-    MacroBlockD *xd, const struct AV1Common *const cm, const MV this_mv, MV *best_mv,
-    int hstep, const SubpelMvLimits *mv_limits,
-    const SUBPEL_SEARCH_VAR_PARAMS *var_params,
-    const MV_COST_PARAMS *mv_cost_params, unsigned int *besterr,
+    MacroBlockD *xd, const struct AV1Common *const cm, const MV this_mv, MV *best_mv, int hstep,
+    const SubpelMvLimits *mv_limits, const SUBPEL_SEARCH_VAR_PARAMS *var_params,
+    const MV_COST_PARAMS *mv_cost_params, unsigned int *besterr, unsigned int orgerr,
     unsigned int *sse1, int *distortion, int iters, int is_scaled) {
-    const MV diag_step = first_level_check_fast(
-        xd, cm, this_mv, best_mv, hstep, mv_limits, var_params, mv_cost_params,
-        besterr, sse1, distortion, is_scaled);
-    if (iters > 1) {
-        second_level_check_fast(xd, cm, this_mv, diag_step, best_mv, hstep,
-            mv_limits, var_params, mv_cost_params, besterr,
-            sse1, distortion, is_scaled);
-    }
+    const MV diag_step = first_level_check_fast(xd,
+                                                cm,
+                                                this_mv,
+                                                best_mv,
+                                                hstep,
+                                                mv_limits,
+                                                var_params,
+                                                mv_cost_params,
+                                                besterr,
+                                                orgerr,
+                                                sse1,
+                                                distortion,
+                                                is_scaled);
+    if (*besterr < orgerr)
+        if (iters > 1) {
+            second_level_check_fast(xd,
+                                    cm,
+                                    this_mv,
+                                    diag_step,
+                                    best_mv,
+                                    hstep,
+                                    mv_limits,
+                                    var_params,
+                                    mv_cost_params,
+                                    besterr,
+                                    sse1,
+                                    distortion,
+                                    is_scaled);
+        }
 }
-
-int svt_av1_find_best_sub_pixel_tree_pruned(
-    MacroBlockD *xd, const struct AV1Common *const cm,
-    const SUBPEL_MOTION_SEARCH_PARAMS *ms_params, MV start_mv, MV *bestmv,
-    int *distortion, unsigned int *sse1, int_mv *last_mv_search_list) {
+static const uint8_t eb_av1_var_offs[MAX_SB_SIZE] = {
+    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
+    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
+    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
+    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
+    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
+    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
+    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128};
+int svt_av1_find_best_sub_pixel_tree_pruned(MacroBlockD *xd, const struct AV1Common *const cm,
+                                            const SUBPEL_MOTION_SEARCH_PARAMS *ms_params,
+                                            MV start_mv, MV *bestmv,
+                                            int *distortion, unsigned int *sse1, int qp,
+                                            BlockSize bsize, uint8_t early_neigh_check_exit) {
     (void)cm;
-    const int allow_hp = ms_params->allow_hp;
-    const int forced_stop = ms_params->forced_stop;
+    const int allow_hp       = ms_params->allow_hp;
+    const int forced_stop    = ms_params->forced_stop;
     const int iters_per_step = ms_params->iters_per_step;
-    const int *cost_list = ms_params->cost_list;
-    const SubpelMvLimits *mv_limits = &ms_params->mv_limits;
-    const MV_COST_PARAMS *mv_cost_params = &ms_params->mv_cost_params;
-    const SUBPEL_SEARCH_VAR_PARAMS *var_params = &ms_params->var_params;
-
-    // The iteration we are current searching for. Iter 0 corresponds to fullpel
-    // mv, iter 1 to half pel, and so on
-    int iter = 0;
-    int hstep = INIT_SUBPEL_STEP_SIZE;  // Step size, initialized to 4/8=1/2 pel
+    const SubpelMvLimits *          mv_limits      = &ms_params->mv_limits;
+    const MV_COST_PARAMS *          mv_cost_params = &ms_params->mv_cost_params;
+    const SUBPEL_SEARCH_VAR_PARAMS *var_params     = &ms_params->var_params;
+    int          hstep = INIT_SUBPEL_STEP_SIZE; // Step size, initialized to 4/8=1/2 pel
     unsigned int besterr;
+    unsigned int org_error;
     *bestmv = start_mv;
 
     const int is_scaled = 0;
-    besterr = svt_upsampled_setup_center_error(xd, cm, bestmv, var_params,
-        mv_cost_params, sse1, distortion);
-    //besterr = setup_center_error_facade(
-    //    xd, cm, bestmv, var_params, mv_cost_params, sse1, distortion, is_scaled);
+    besterr = svt_upsampled_setup_center_error(
+        bestmv, var_params, mv_cost_params, (unsigned int *)distortion);
+    if (early_neigh_check_exit)
+        return besterr;
+    uint64_t th_normalizer = (((var_params->w * var_params->h) >> 3) * ms_params->abs_th_mult *
+                              (qp >> 1));
+    if (besterr < th_normalizer)
+        return besterr;
+    // How many steps to take. A round of 0 means fullpel search only, 1 means
+    // half-pel, and so on.
+    const int round = AOMMIN(FULL_PEL - forced_stop, 3 - !allow_hp);
 
     // If forced_stop is FULL_PEL, return.
-    if (forced_stop == FULL_PEL) return besterr;
+    if (!round)
+        return besterr;
+    // Exit subpel search if the variance of the full-pel predicted samples is low (i.e. where likely interpolation will not modify the integer samples)
+    const MSBuffers *  ms_buffers = &var_params->ms_buffers;
+    const uint8_t *    ref        = svt_get_buf_from_mv(ms_buffers->ref, *bestmv);
+    unsigned int       sse;
+    const unsigned int var = var_params->vfp->vf(
+        ref, ms_buffers->ref->stride, eb_av1_var_offs, 0, &sse);
+    int block_var = ROUND_POWER_OF_TWO(var, num_pels_log2_lookup[bsize]);
 
-    if (svt_check_repeated_mv_and_update(last_mv_search_list, *bestmv, iter)) {
-        return INT_MAX;
+    if (block_var < ms_params->pred_variance_th)
+        return besterr;
+    if (ms_params->skip_diag_refinement >= 4) {
+        org_error = 0;
+    } else {
+        unsigned int demo = ms_params->skip_diag_refinement >= 2
+            ? ((var_params->w >= 64 || var_params->h >= 64) ? 2 : 1)
+            : 1;
+        org_error         = ms_params->skip_diag_refinement ? besterr / demo : INT_MAX;
     }
-    iter++;
-
-    if (cost_list && cost_list[0] != INT_MAX && cost_list[1] != INT_MAX &&
-        cost_list[2] != INT_MAX && cost_list[3] != INT_MAX &&
-        cost_list[4] != INT_MAX) {
-        const unsigned int whichdir = (cost_list[1] < cost_list[3] ? 0 : 1) +
-            (cost_list[2] < cost_list[4] ? 0 : 2);
-
-        const MV left_mv = { start_mv.row, start_mv.col - hstep };
-        const MV right_mv = { start_mv.row, start_mv.col + hstep };
-        const MV bottom_mv = { start_mv.row + hstep, start_mv.col };
-        const MV top_mv = { start_mv.row - hstep, start_mv.col };
-
-        const MV bottom_left_mv = { start_mv.row + hstep, start_mv.col - hstep };
-        const MV bottom_right_mv = { start_mv.row + hstep, start_mv.col + hstep };
-        const MV top_left_mv = { start_mv.row - hstep, start_mv.col - hstep };
-        const MV top_right_mv = { start_mv.row - hstep, start_mv.col + hstep };
-
-        int dummy = 0;
-
-        switch (whichdir) {
-        case 0:  // bottom left quadrant
-            svt_check_better_fast(xd, cm, &left_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            svt_check_better_fast(xd, cm, &bottom_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            svt_check_better_fast(xd, cm, &bottom_left_mv, bestmv, mv_limits,
-                var_params, mv_cost_params, &besterr, sse1,
-                distortion, &dummy, is_scaled);
-            break;
-        case 1:  // bottom right quadrant
-            svt_check_better_fast(xd, cm, &right_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            svt_check_better_fast(xd, cm, &bottom_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            svt_check_better_fast(xd, cm, &bottom_right_mv, bestmv, mv_limits,
-                var_params, mv_cost_params, &besterr, sse1,
-                distortion, &dummy, is_scaled);
-            break;
-        case 2:  // top left quadrant
-            svt_check_better_fast(xd, cm, &left_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            svt_check_better_fast(xd, cm, &top_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            svt_check_better_fast(xd, cm, &top_left_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            break;
-        case 3:  // top right quadrant
-            svt_check_better_fast(xd, cm, &right_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            svt_check_better_fast(xd, cm, &top_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            svt_check_better_fast(xd, cm, &top_right_mv, bestmv, mv_limits, var_params,
-                mv_cost_params, &besterr, sse1, distortion, &dummy,
-                is_scaled);
-            break;
-        }
-    }
-    else {
-        two_level_checks_fast(xd, cm, start_mv, bestmv, hstep, mv_limits,
-            var_params, mv_cost_params, &besterr, sse1,
-            distortion, iters_per_step, is_scaled);
-    }
-
-    // Each subsequent iteration checks at least one point in common with
-    // the last iteration could be 2 ( if diag selected) 1/4 pel
-    if (forced_stop < HALF_PEL) {
-        if (svt_check_repeated_mv_and_update(last_mv_search_list, *bestmv, iter)) {
-            return INT_MAX;
-        }
-        iter++;
-
+    for (int iter = 0; iter < round; ++iter) {
+        unsigned int prev_besterr = besterr;
+        two_level_checks_fast(xd,
+                              cm,
+                              start_mv,
+                              bestmv,
+                              hstep,
+                              mv_limits,
+                              var_params,
+                              mv_cost_params,
+                              &besterr,
+                              org_error,
+                              sse1,
+                              distortion,
+                              iters_per_step,
+                              is_scaled);
         hstep >>= 1;
         start_mv = *bestmv;
-        two_level_checks_fast(xd, cm, start_mv, bestmv, hstep, mv_limits,
-            var_params, mv_cost_params, &besterr, sse1,
-            distortion, iters_per_step, is_scaled);
+        if (ms_params->skip_diag_refinement && iter < QUARTER_PEL)
+            org_error = MIN(org_error, besterr);
+        int32_t deviation = (((int64_t)MAX(besterr, 1) - (int64_t)MAX(prev_besterr, 1)) * 100) /
+            (int64_t)MAX(prev_besterr, 1);
+        if (deviation >= ms_params->round_dev_th)
+            return besterr;
     }
-
-    if (allow_hp && forced_stop == EIGHTH_PEL) {
-        if (svt_check_repeated_mv_and_update(last_mv_search_list, *bestmv, iter)) {
-            return INT_MAX;
-        }
-        iter++;
-
-        hstep >>= 1;
-        start_mv = *bestmv;
-        two_level_checks_fast(xd, cm, start_mv, bestmv, hstep, mv_limits,
-            var_params, mv_cost_params, &besterr, sse1,
-            distortion, iters_per_step, is_scaled);
-    }
-
     return besterr;
 }
 
 int svt_av1_find_best_sub_pixel_tree(MacroBlockD *xd, const struct AV1Common *const cm,
                                      const SUBPEL_MOTION_SEARCH_PARAMS *ms_params, MV start_mv,
-                                     MV *bestmv, int *distortion, unsigned int *sse1,
-                                     int_mv *last_mv_search_list) {
+                                     MV *bestmv, int *distortion, unsigned int *sse1, int qp,
+                                     BlockSize bsize, uint8_t early_neigh_check_exit) {
     const int allow_hp       = ms_params->allow_hp;
     const int forced_stop    = ms_params->forced_stop;
     const int iters_per_step = ms_params->iters_per_step;
@@ -713,19 +786,30 @@ int svt_av1_find_best_sub_pixel_tree(MacroBlockD *xd, const struct AV1Common *co
 
     *bestmv             = start_mv;
     const int is_scaled = 0;
-    besterr             = svt_upsampled_setup_center_error(
-        xd, cm, bestmv, var_params, mv_cost_params, sse1, distortion);
+    besterr = svt_upsampled_setup_center_error(
+        bestmv, var_params, mv_cost_params, (unsigned int *)distortion);
+    if (early_neigh_check_exit)
+        return besterr;
+    // Exit subpel search if the variance of the full-pel predicted samples is low (i.e. where likely interpolation will not modify the integer samples)
+    const MSBuffers *  ms_buffers = &var_params->ms_buffers;
+    const uint8_t *    ref        = svt_get_buf_from_mv(ms_buffers->ref, *bestmv);
+    unsigned int       sse;
+    const unsigned int var = var_params->vfp->vf(
+        ref, ms_buffers->ref->stride, eb_av1_var_offs, 0, &sse);
+    int block_var = ROUND_POWER_OF_TWO(var, num_pels_log2_lookup[bsize]);
 
+    if (block_var < ms_params->pred_variance_th)
+        return besterr;
+    uint64_t th_normalizer = (((var_params->w * var_params->h) >> 2) * ms_params->abs_th_mult *
+                              (qp >> 1));
+    if (besterr < th_normalizer)
+        return besterr;
     // If forced_stop is FULL_PEL, return.
     if (!round)
         return besterr;
 
     for (int iter = 0; iter < round; ++iter) {
         MV iter_center_mv = *bestmv;
-        if (svt_check_repeated_mv_and_update(last_mv_search_list, iter_center_mv, iter)) {
-            return INT_MAX;
-        }
-
         MV diag_step;
         diag_step = svt_first_level_check(xd,
                                           cm,
