@@ -17,11 +17,13 @@
 #include "mathutils.h"
 #include "random.h"
 #include "common_dsp_rtcd.h"
+#include "EbUtility.h"
+
 #define MAX_MINPTS 4
 #define MAX_DEGENERATE_ITER 10
 #define MINPTS_MULTIPLIER 5
 
-#define INLIER_THRESHOLD 1.25
+#define INLIER_THRESHOLD_POW2 1.5625 /*(1.25 * 1.25)*/
 #define MIN_TRIALS 20
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -87,16 +89,16 @@ static void normalize_homography(double *pts, int n, double *T) {
         msqe += sqrt(p[0] * p[0] + p[1] * p[1]);
     }
     msqe /= n;
-    scale = (msqe == 0 ? 1.0 : sqrt(2) / msqe);
-    T[0]  = scale;
-    T[1]  = 0;
-    T[2]  = -scale * mean[0];
-    T[3]  = 0;
-    T[4]  = scale;
-    T[5]  = -scale * mean[1];
-    T[6]  = 0;
-    T[7]  = 0;
-    T[8]  = 1;
+    scale = (msqe == 0 ? 1.0 : CONST_SQRT2 / msqe);
+    T[0] = scale;
+    T[1] = 0;
+    T[2] = -scale * mean[0];
+    T[3] = 0;
+    T[4] = scale;
+    T[5] = -scale * mean[1];
+    T[6] = 0;
+    T[7] = 0;
+    T[8] = 1;
     for (p = pts, i = 0; i < n; ++i, p += 2) {
         p[0] *= scale;
         p[1] *= scale;
@@ -454,14 +456,14 @@ static int ransac(const int *matched_points, int npoints, int *num_inliers_by_mo
         projectpoints(params_this_motion, corners1, image1_coord, npoints, 2, 2);
 
         for (int i = 0; i < npoints; ++i) {
-            double dx       = image1_coord[i * 2] - corners2[i * 2];
-            double dy       = image1_coord[i * 2 + 1] - corners2[i * 2 + 1];
-            double distance = sqrt(dx * dx + dy * dy);
+            double dx = image1_coord[i * 2] - corners2[i * 2];
+            double dy = image1_coord[i * 2 + 1] - corners2[i * 2 + 1];
+            double distance_pow2 = dx * dx + dy * dy;
 
-            if (distance < INLIER_THRESHOLD) {
+            if (distance_pow2 < INLIER_THRESHOLD_POW2) {
                 current_motion.inlier_indices[current_motion.num_inliers++] = i;
-                sum_distance += distance;
-                sum_distance_squared += distance * distance;
+                sum_distance += sqrt(distance_pow2);
+                sum_distance_squared += distance_pow2;
             }
         }
 
@@ -541,190 +543,6 @@ finish_ransac:
     return ret_val;
 }
 
-static int ransac_double_prec(const double *matched_points, int npoints, int *num_inliers_by_motion,
-                              MotionModel *params_by_motion, int num_desired_motions, int minpts,
-                              IsDegenerateFunc        is_degenerate,
-                              FindTransformationFunc  find_transformation,
-                              ProjectPointsDoubleFunc projectpoints) {
-    int trial_count = 0;
-    int ret_val     = 0;
-
-    unsigned int seed = (unsigned int)npoints;
-
-    int indices[MAX_MINPTS] = {0};
-
-    double *points1, *points2;
-    double *corners1, *corners2;
-    double *image1_coord;
-
-    // Store information for the num_desired_motions best transformations found
-    // and the worst motion among them, as well as the motion currently under
-    // consideration.
-    RANSAC_MOTION *motions, *worst_kept_motion = NULL;
-    RANSAC_MOTION  current_motion;
-
-    // Store the parameters and the indices of the inlier points for the motion
-    // currently under consideration.
-    double params_this_motion[MAX_PARAMDIM];
-
-    double *cnp1, *cnp2;
-
-    for (int i = 0; i < num_desired_motions; ++i) num_inliers_by_motion[i] = 0;
-
-    if (npoints < minpts * MINPTS_MULTIPLIER || npoints == 0)
-        return 1;
-
-    points1      = (double *)malloc(sizeof(*points1) * npoints * 2);
-    points2      = (double *)malloc(sizeof(*points2) * npoints * 2);
-    corners1     = (double *)malloc(sizeof(*corners1) * npoints * 2);
-    corners2     = (double *)malloc(sizeof(*corners2) * npoints * 2);
-    image1_coord = (double *)malloc(sizeof(*image1_coord) * npoints * 2);
-
-    motions = (RANSAC_MOTION *)malloc(sizeof(RANSAC_MOTION) * num_desired_motions);
-    assert(motions != NULL);
-    for (int i = 0; i < num_desired_motions; ++i) {
-        motions[i].inlier_indices = (int *)malloc(sizeof(*motions->inlier_indices) * npoints);
-        clear_motion(motions + i, npoints);
-    }
-    current_motion.inlier_indices = (int *)malloc(sizeof(*current_motion.inlier_indices) * npoints);
-    clear_motion(&current_motion, npoints);
-
-    worst_kept_motion = motions;
-
-    if (!(points1 && points2 && corners1 && corners2 && image1_coord && motions &&
-          current_motion.inlier_indices)) {
-        ret_val = 1;
-        goto finish_ransac;
-    }
-
-    cnp1 = corners1;
-    cnp2 = corners2;
-    for (int i = 0; i < npoints; ++i) {
-        *(cnp1++) = *(matched_points++);
-        *(cnp1++) = *(matched_points++);
-        *(cnp2++) = *(matched_points++);
-        *(cnp2++) = *(matched_points++);
-    }
-
-    while (MIN_TRIALS > trial_count) {
-        double sum_distance         = 0.0;
-        double sum_distance_squared = 0.0;
-
-        clear_motion(&current_motion, npoints);
-
-        int degenerate          = 1;
-        int num_degenerate_iter = 0;
-
-        while (degenerate) {
-            num_degenerate_iter++;
-            if (!get_rand_indices(npoints, minpts, indices, &seed)) {
-                ret_val = 1;
-                goto finish_ransac;
-            }
-
-            copy_points_at_indices(points1, corners1, indices, minpts);
-            copy_points_at_indices(points2, corners2, indices, minpts);
-
-            degenerate = is_degenerate(points1);
-            if (num_degenerate_iter > MAX_DEGENERATE_ITER) {
-                ret_val = 1;
-                goto finish_ransac;
-            }
-        }
-
-        if (find_transformation(minpts, points1, points2, params_this_motion)) {
-            trial_count++;
-            continue;
-        }
-
-        projectpoints(params_this_motion, corners1, image1_coord, npoints, 2, 2);
-
-        for (int i = 0; i < npoints; ++i) {
-            double dx       = image1_coord[i * 2] - corners2[i * 2];
-            double dy       = image1_coord[i * 2 + 1] - corners2[i * 2 + 1];
-            double distance = sqrt(dx * dx + dy * dy);
-
-            if (distance < INLIER_THRESHOLD) {
-                current_motion.inlier_indices[current_motion.num_inliers++] = i;
-                sum_distance += distance;
-                sum_distance_squared += distance * distance;
-            }
-        }
-
-        if (current_motion.num_inliers >= worst_kept_motion->num_inliers &&
-            current_motion.num_inliers > 1) {
-            double mean_distance;
-            mean_distance           = sum_distance / ((double)current_motion.num_inliers);
-            current_motion.variance = sum_distance_squared /
-                    ((double)current_motion.num_inliers - 1.0) -
-                mean_distance * mean_distance * ((double)current_motion.num_inliers) /
-                    ((double)current_motion.num_inliers - 1.0);
-            if (is_better_motion(&current_motion, worst_kept_motion)) {
-                // This motion is better than the worst currently kept motion. Remember
-                // the inlier points and variance. The parameters for each kept motion
-                // will be recomputed later using only the inliers.
-                worst_kept_motion->num_inliers = current_motion.num_inliers;
-                worst_kept_motion->variance    = current_motion.variance;
-                if (svt_memcpy != NULL)
-                    svt_memcpy(worst_kept_motion->inlier_indices,
-                               current_motion.inlier_indices,
-                               sizeof(*current_motion.inlier_indices) * npoints);
-                else
-                    svt_memcpy_c(worst_kept_motion->inlier_indices,
-                                 current_motion.inlier_indices,
-                                 sizeof(*current_motion.inlier_indices) * npoints);
-                assert(npoints > 0);
-                // Determine the new worst kept motion and its num_inliers and variance.
-                for (int i = 0; i < num_desired_motions; ++i) {
-                    if (is_better_motion(worst_kept_motion, &motions[i])) {
-                        worst_kept_motion = &motions[i];
-                    }
-                }
-            }
-        }
-        trial_count++;
-    }
-
-    // Sort the motions, best first.
-    qsort(motions, num_desired_motions, sizeof(RANSAC_MOTION), compare_motions);
-
-    // Recompute the motions using only the inliers.
-    for (int i = 0; i < num_desired_motions; ++i) {
-        if (motions[i].num_inliers >= minpts) {
-            copy_points_at_indices(
-                points1, corners1, motions[i].inlier_indices, motions[i].num_inliers);
-            copy_points_at_indices(
-                points2, corners2, motions[i].inlier_indices, motions[i].num_inliers);
-
-            find_transformation(
-                motions[i].num_inliers, points1, points2, params_by_motion[i].params);
-            if (svt_memcpy != NULL)
-                svt_memcpy(params_by_motion[i].inliers,
-                           motions[i].inlier_indices,
-                           sizeof(*motions[i].inlier_indices) * npoints);
-            else
-                svt_memcpy_c(params_by_motion[i].inliers,
-                             motions[i].inlier_indices,
-                             sizeof(*motions[i].inlier_indices) * npoints);
-        }
-        num_inliers_by_motion[i] = motions[i].num_inliers;
-    }
-
-finish_ransac:
-    free(points1);
-    free(points2);
-    free(corners1);
-    free(corners2);
-    free(image1_coord);
-    free(current_motion.inlier_indices);
-    if (motions) {
-        for (int i = 0; i < num_desired_motions; ++i) free(motions[i].inlier_indices);
-        free(motions);
-    }
-
-    return ret_val;
-}
-
 static int is_collinear3(double *p1, double *p2, double *p3) {
     static const double collinear_eps = 1e-3;
     const double        v = (p2[0] - p1[0]) * (p3[1] - p1[1]) - (p2[1] - p1[1]) * (p3[0] - p1[0]);
@@ -784,54 +602,3 @@ RansacFunc svt_av1_get_ransac_type(TransformationType type) {
     default: assert(0); return NULL;
     }
 }
-
-static int ransac_translation_double_prec(double *matched_points, int npoints,
-                                          int *num_inliers_by_motion, MotionModel *params_by_motion,
-                                          int num_desired_motions) {
-    return ransac_double_prec(matched_points,
-                              npoints,
-                              num_inliers_by_motion,
-                              params_by_motion,
-                              num_desired_motions,
-                              3,
-                              is_degenerate_translation,
-                              find_translation,
-                              project_points_double_translation);
-}
-
-static int ransac_rotzoom_double_prec(double *matched_points, int npoints,
-                                      int *num_inliers_by_motion, MotionModel *params_by_motion,
-                                      int num_desired_motions) {
-    return ransac_double_prec(matched_points,
-                              npoints,
-                              num_inliers_by_motion,
-                              params_by_motion,
-                              num_desired_motions,
-                              3,
-                              is_degenerate_affine,
-                              find_rotzoom,
-                              project_points_double_rotzoom);
-}
-
-static int ransac_affine_double_prec(double *matched_points, int npoints,
-                                     int *num_inliers_by_motion, MotionModel *params_by_motion,
-                                     int num_desired_motions) {
-    return ransac_double_prec(matched_points,
-                              npoints,
-                              num_inliers_by_motion,
-                              params_by_motion,
-                              num_desired_motions,
-                              3,
-                              is_degenerate_affine,
-                              find_affine,
-                              project_points_double_affine);
-}
-
-RansacFuncDouble svt_av1_get_ransac_double_prec_type(TransformationType type) {
-    switch (type) {
-    case AFFINE: return ransac_affine_double_prec;
-    case ROTZOOM: return ransac_rotzoom_double_prec;
-    case TRANSLATION: return ransac_translation_double_prec;
-    default: assert(0); return NULL;
-    }
-}
