@@ -1,13 +1,14 @@
 /*
  * Copyright(c) 2019 Netflix, Inc.
-*
-* This source code is subject to the terms of the BSD 2 Clause License and
-* the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
-* was not distributed with this source code in the LICENSE file, you can
-* obtain it at https://www.aomedia.org/license/software-license. If the Alliance for Open
-* Media Patent License 1.0 was not distributed with this source code in the
-* PATENTS file, you can obtain it at https://www.aomedia.org/license/patent-license.
-*/
+ *
+ * This source code is subject to the terms of the BSD 2 Clause License and
+ * the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
+ * was not distributed with this source code in the LICENSE file, you can
+ * obtain it at https://www.aomedia.org/license/software-license. If the
+ * Alliance for Open Media Patent License 1.0 was not distributed with this
+ * source code in the PATENTS file, you can obtain it at
+ * https://www.aomedia.org/license/patent-license.
+ */
 
 /******************************************************************************
  * @file CdefTest.cc
@@ -41,28 +42,25 @@
 #include "EbUnitTestUtility.h"
 #include "EbUtility.h"
 
-using ::testing::make_tuple;
 using svt_av1_test_tool::SVTRandom;
+using ::testing::make_tuple;
 namespace {
-
-typedef void (*svt_cdef_filter_block_8x8_16_func)(
+typedef void (*svt_cdef_filter_block_8xn_16_func)(
     const uint16_t *const in, const int32_t pri_strength,
     const int32_t sec_strength, const int32_t dir, int32_t pri_damping,
     int32_t sec_damping, const int32_t coeff_shift, uint16_t *const dst,
-    const int32_t dstride);
+    const int32_t dstride, uint8_t height, uint8_t subsampling_factor);
 
-static const svt_cdef_filter_block_8x8_16_func
-    svt_cdef_filter_block_8x8_16_func_table[] = {
-        svt_cdef_filter_block_8x8_16_avx2,
-#if EN_AVX512_SUPPORT
-        svt_cdef_filter_block_8x8_16_avx512
+static const svt_cdef_filter_block_8xn_16_func
+    svt_cdef_filter_block_8xn_16_func_table[] = {
+        svt_cdef_filter_block_8xn_16_avx2,
+#ifndef EN_AVX512_SUPPORT
+        svt_cdef_filter_block_8xn_16_avx512
 #endif
 };
-
 using cdef_dir_param_t =
-    ::testing::tuple<CdefFilterBlockFunc, CdefFilterBlockFunc, BlockSize,
-                     int, int, svt_cdef_filter_block_8x8_16_func>;
-
+    ::testing::tuple<CdefFilterBlockFunc, CdefFilterBlockFunc, BlockSize, int,
+                     int, svt_cdef_filter_block_8xn_16_func>;
 /**
  * @brief Unit test for svt_cdef_filter_block_avx2
  *
@@ -99,7 +97,7 @@ class CDEFBlockTest : public ::testing::TestWithParam<cdef_dir_param_t> {
         bsize_ = TEST_GET_PARAM(2);
         boundary_ = TEST_GET_PARAM(3);
         bd_ = TEST_GET_PARAM(4);
-        svt_cdef_filter_block_8x8_16 = TEST_GET_PARAM(5);
+        svt_cdef_filter_block_8xn_16 = TEST_GET_PARAM(5);
 
         memset(dst_ref_, 0, sizeof(dst_ref_));
         memset(dst_tst_, 0, sizeof(dst_tst_));
@@ -138,7 +136,8 @@ class CDEFBlockTest : public ::testing::TestWithParam<cdef_dir_param_t> {
         }
     }
 
-    void run_test(int pri_damping, int sec_damping) {
+    void run_test(int pri_damping, int sec_damping,
+                  uint8_t subsampling_factor) {
         int pri_strength, sec_strength;
         int dir;
         unsigned int pos = 0;
@@ -167,7 +166,8 @@ class CDEFBlockTest : public ::testing::TestWithParam<cdef_dir_param_t> {
                               pri_damping,
                               sec_damping,
                               bsize_,
-                              bd_ - 8);
+                              bd_ - 8,
+                              subsampling_factor);
                     cdef_tst_(bd_ == 8 ? (uint8_t *)dst_tst_ : 0,
                               dst_tst_,
                               size_,
@@ -178,8 +178,8 @@ class CDEFBlockTest : public ::testing::TestWithParam<cdef_dir_param_t> {
                               pri_damping,
                               sec_damping,
                               bsize_,
-                              bd_ - 8);
-
+                              bd_ - 8,
+                              subsampling_factor);
                     for (pos = 0; pos < max_pos; pos++) {
                         ASSERT_EQ(dst_ref_[pos], dst_tst_[pos])
                             << "Error: CDEFBlockTest, SIMD and C mismatch."
@@ -218,7 +218,19 @@ class CDEFBlockTest : public ::testing::TestWithParam<cdef_dir_param_t> {
                          level += (2 + 6 * scale) << (bd_ - 8)) {
                         for (bits = 1; bits <= bd_; bits += 1 + 3 * scale) {
                             prepare_data(level, bits);
-                            run_test(pri_damping, sec_damping);
+                            // Allowable subsampling values are: 1, 2
+                            run_test(pri_damping, sec_damping, 1);
+
+                            /*
+                            The 2x subsampled avx2 kernel for 4x4 blocks is the
+                            same as the non-subsampled kernel because all values
+                            can be filtered at once, so subsampling would not
+                            give a speed gain. Therefore, the C/AVX2 kernels for
+                            4x4 2x subsampled blocks will produce different
+                            outputs (expected), and are not tested.
+                            */
+                            if (bsize_ > BLOCK_4X4)
+                                run_test(pri_damping, sec_damping, 2);
                         }
                     }
                 }
@@ -267,7 +279,9 @@ class CDEFBlockTest : public ::testing::TestWithParam<cdef_dir_param_t> {
                             pri_damping,
                             sec_damping,
                             bsize_,
-                            bd_ - 8);
+                            bd_ - 8,
+                            1);  // subsampling - test only non-subsampled for
+                                 // the speed
                     }
                 }
             }
@@ -300,7 +314,9 @@ class CDEFBlockTest : public ::testing::TestWithParam<cdef_dir_param_t> {
                             pri_damping,
                             sec_damping,
                             bsize_,
-                            bd_ - 8);
+                            bd_ - 8,
+                            1);  // subsampling - test only non-subsampled for
+                                 // the speed
                     }
                 }
             }
@@ -351,7 +367,6 @@ TEST_P(CDEFBlockTest, DISABLED_SpeedTest) {
 // structs as arguments, which makes the v256 type of the intrinsics
 // hard to support, so optimizations for this target are disabled.
 #if defined(_WIN64) || !defined(_MSC_VER) || defined(__clang__)
-
 INSTANTIATE_TEST_CASE_P(
     Cdef, CDEFBlockTest,
     ::testing::Combine(
@@ -359,12 +374,20 @@ INSTANTIATE_TEST_CASE_P(
         ::testing::Values(&svt_cdef_filter_block_c),
         ::testing::Values(BLOCK_4X4, BLOCK_4X8, BLOCK_8X4, BLOCK_8X8),
         ::testing::Range(0, 16), ::testing::Range(8, 13, 2),
-        ::testing::ValuesIn(svt_cdef_filter_block_8x8_16_func_table)));
+        ::testing::ValuesIn(svt_cdef_filter_block_8xn_16_func_table)));
+
+INSTANTIATE_TEST_CASE_P(
+    Cdef_sse4_1, CDEFBlockTest,
+    ::testing::Combine(
+        ::testing::Values(&svt_cdef_filter_block_avx2),
+        ::testing::Values(&svt_av1_cdef_filter_block_sse4_1),
+        ::testing::Values(BLOCK_4X4, BLOCK_4X8, BLOCK_8X4, BLOCK_8X8),
+        ::testing::Range(0, 16), ::testing::Range(8, 13, 2),
+        ::testing::ValuesIn(svt_cdef_filter_block_8xn_16_func_table)));
 
 #endif  // defined(_WIN64) || !defined(_MSC_VER)
-
-using FindDirFunc = int (*)(const uint16_t *img, int stride, int32_t *var,
-                            int coeff_shift);
+using FindDirFunc = uint8_t (*)(const uint16_t *img, int stride, int32_t *var,
+                                int coeff_shift);
 using TestFindDirParam = ::testing::tuple<FindDirFunc, FindDirFunc>;
 
 /**
@@ -410,7 +433,7 @@ class CDEFFindDirTest : public ::testing::TestWithParam<TestFindDirParam> {
 
     void test_finddir() {
         int depth, bits, level, count;
-        int res_ref = 0, res_tst = 0;
+        uint8_t res_ref = 0, res_tst = 0;
         int32_t var_ref = 0, var_tst = 0;
 
         for (depth = 8; depth <= 12; depth += 2) {
@@ -569,26 +592,33 @@ TEST(CdefToolTest, ComputeCdefDistMatchTest) {
                 BLOCK_4X4, BLOCK_4X8, BLOCK_8X4, BLOCK_8X8};
             for (int i = 0; i < 4; ++i) {
                 for (int plane = 0; plane < 3; ++plane) {
-                    const uint64_t c_mse = compute_cdef_dist_c(dst_data_,
-                                                               stride,
-                                                               src_data_,
-                                                               dlist,
-                                                               cdef_count,
-                                                               test_bs[i],
-                                                               coeff_shift,
-                                                               plane);
-                    const uint64_t avx_mse = compute_cdef_dist_16bit_avx2(dst_data_,
-                                                                          stride,
-                                                                          src_data_,
-                                                                          dlist,
-                                                                          cdef_count,
-                                                                          test_bs[i],
-                                                                          coeff_shift,
-                                                                          plane);
-                    ASSERT_EQ(c_mse, avx_mse)
-                        << "compute_cdef_dist_16bit_avx2 failed "
-                        << "bitdepth: " << bd << " plane: " << plane
-                        << " BlockSize " << test_bs[i] << " loop: " << k;
+                    // Allowable subsampling values are: 1, 2
+                    for (uint8_t subsampling = 1; subsampling <= 2;
+                         subsampling <<= 1) {
+                        const uint64_t c_mse = compute_cdef_dist_c(dst_data_,
+                                                                   stride,
+                                                                   src_data_,
+                                                                   dlist,
+                                                                   cdef_count,
+                                                                   test_bs[i],
+                                                                   coeff_shift,
+                                                                   plane,
+                                                                   subsampling);
+                        const uint64_t avx_mse =
+                            compute_cdef_dist_16bit_avx2(dst_data_,
+                                                         stride,
+                                                         src_data_,
+                                                         dlist,
+                                                         cdef_count,
+                                                         test_bs[i],
+                                                         coeff_shift,
+                                                         plane,
+                                                         subsampling);
+                        ASSERT_EQ(c_mse, avx_mse)
+                            << "compute_cdef_dist_16bit_avx2 failed "
+                            << "bitdepth: " << bd << " plane: " << plane
+                            << " BlockSize " << test_bs[i] << " loop: " << k;
+                    }
                 }
             }
         }
@@ -632,27 +662,34 @@ TEST(CdefToolTest, ComputeCdefDist8bitMatchTest) {
                 BLOCK_4X4, BLOCK_4X8, BLOCK_8X4, BLOCK_8X8};
             for (int i = 0; i < 4; ++i) {
                 for (int plane = 0; plane < 3; ++plane) {
-                    const uint64_t c_mse = compute_cdef_dist_8bit_c(dst_data_,
-                                                                    stride,
-                                                                    src_data_,
-                                                                    dlist,
-                                                                    cdef_count,
-                                                                    test_bs[i],
-                                                                    coeff_shift,
-                                                                    plane);
-                    const uint64_t avx_mse =
-                        compute_cdef_dist_8bit_avx2(dst_data_,
-                                                    stride,
-                                                    src_data_,
-                                                    dlist,
-                                                    cdef_count,
-                                                    test_bs[i],
-                                                    coeff_shift,
-                                                    plane);
-                    ASSERT_EQ(c_mse, avx_mse)
-                        << "compute_cdef_dist_8bit_avx2 failed "
-                        << "bitdepth: " << bd << " plane: " << plane
-                        << " BlockSize " << test_bs[i] << " loop: " << k;
+                    // Allowable subsampling values are: 1, 2
+                    for (uint8_t subsampling = 1; subsampling <= 2;
+                         subsampling <<= 1) {
+                        const uint64_t c_mse =
+                            compute_cdef_dist_8bit_c(dst_data_,
+                                                     stride,
+                                                     src_data_,
+                                                     dlist,
+                                                     cdef_count,
+                                                     test_bs[i],
+                                                     coeff_shift,
+                                                     plane,
+                                                     subsampling);
+                        const uint64_t avx_mse =
+                            compute_cdef_dist_8bit_avx2(dst_data_,
+                                                        stride,
+                                                        src_data_,
+                                                        dlist,
+                                                        cdef_count,
+                                                        test_bs[i],
+                                                        coeff_shift,
+                                                        plane,
+                                                        subsampling);
+                        ASSERT_EQ(c_mse, avx_mse)
+                            << "compute_cdef_dist_8bit_avx2 failed "
+                            << "bitdepth: " << bd << " plane: " << plane
+                            << " BlockSize " << test_bs[i] << " loop: " << k;
+                    }
                 }
             }
         }
@@ -677,10 +714,10 @@ TEST(CdefToolTest, ComputeCdefDist8bitMatchTest) {
  * end_gi: TOTAL_STRENGTHS
  *
  */
-
 typedef uint64_t (*svt_search_one_dual_func)(int *lev0, int *lev1,
-    int nb_strengths, uint64_t (**mse)[64],
-    int sb_count, int start_gi, int end_gi);
+                                             int nb_strengths,
+                                             uint64_t **mse[2], int sb_count,
+                                             int start_gi, int end_gi);
 
 static const svt_search_one_dual_func search_one_dual_func_table[] = {
     svt_search_one_dual_avx2,
@@ -696,9 +733,13 @@ TEST(CdefToolTest, SearchOneDualMatchTest) {
     const int end_gi = TOTAL_STRENGTHS;
     int lvl_luma_ref[CDEF_MAX_STRENGTHS], lvl_chroma_ref[CDEF_MAX_STRENGTHS];
     int lvl_luma_tst[CDEF_MAX_STRENGTHS], lvl_chroma_tst[CDEF_MAX_STRENGTHS];
-    uint64_t(*mse[2])[TOTAL_STRENGTHS];
-    mse[0] = (uint64_t(*)[64])svt_aom_memalign(32, sizeof(**mse) * sb_count);
-    mse[1] = (uint64_t(*)[64])svt_aom_memalign(32, sizeof(**mse) * sb_count);
+    uint64_t **mse[2];
+    mse[0] = (uint64_t **)svt_aom_memalign(32, sizeof(*mse[0]) * sb_count);
+    mse[1] = (uint64_t **)svt_aom_memalign(32, sizeof(*mse[1]) * sb_count);
+    for (int i = 0; i < sb_count; i++) {
+        mse[0][i] = (uint64_t *)svt_aom_memalign(32, sizeof(**mse[0]) * 64);
+        mse[1][i] = (uint64_t *)svt_aom_memalign(32, sizeof(**mse[1]) * 64);
+    }
 
     SVTRandom rnd_(10, false);
     for (int k = 0; k < 100; ++k) {
@@ -724,8 +765,8 @@ TEST(CdefToolTest, SearchOneDualMatchTest) {
                                                               sb_count,
                                                               start_gi,
                                                               end_gi);
-                for (int l = 0; l < (int) (sizeof(search_one_dual_func_table) /
-                                        sizeof(*search_one_dual_func_table));
+                for (int l = 0; l < (int)(sizeof(search_one_dual_func_table) /
+                                          sizeof(*search_one_dual_func_table));
                      ++l) {
                     uint64_t best_mse_tst =
                         search_one_dual_func_table[l](lvl_luma_tst,
@@ -753,7 +794,10 @@ TEST(CdefToolTest, SearchOneDualMatchTest) {
             }
         }
     }
-
+    for (int i = 0; i < sb_count; i++) {
+        svt_aom_free(mse[0][i]);
+        svt_aom_free(mse[1][i]);
+    }
     svt_aom_free(mse[0]);
     svt_aom_free(mse[1]);
 }
@@ -766,10 +810,13 @@ TEST(CdefToolTest, DISABLED_SearchOneDualSpeedTest) {
     const int nb_strengths = 8;
     int lvl_luma_ref[CDEF_MAX_STRENGTHS], lvl_chroma_ref[CDEF_MAX_STRENGTHS];
     int lvl_luma_tst[CDEF_MAX_STRENGTHS], lvl_chroma_tst[CDEF_MAX_STRENGTHS];
-    uint64_t(*mse[2])[TOTAL_STRENGTHS];
-    mse[0] = (uint64_t(*)[64])svt_aom_memalign(32, sizeof(**mse) * sb_count);
-    mse[1] = (uint64_t(*)[64])svt_aom_memalign(32, sizeof(**mse) * sb_count);
-
+    uint64_t **mse[2];
+    mse[0] = (uint64_t **)svt_aom_memalign(32, sizeof(*mse[0]) * sb_count);
+    mse[1] = (uint64_t **)svt_aom_memalign(32, sizeof(*mse[1]) * sb_count);
+    for (int i = 0; i < sb_count; i++) {
+        mse[0][i] = (uint64_t *)svt_aom_memalign(32, sizeof(**mse[0]) * 64);
+        mse[1][i] = (uint64_t *)svt_aom_memalign(32, sizeof(**mse[1]) * 64);
+    }
     SVTRandom rnd_(10, false);
 
     // generate mse randomly
@@ -784,8 +831,8 @@ TEST(CdefToolTest, DISABLED_SearchOneDualSpeedTest) {
     memset(lvl_luma_tst, 0, sizeof(lvl_luma_tst));
     memset(lvl_chroma_tst, 0, sizeof(lvl_chroma_tst));
 
-    for (int i = 0; i < (int) (sizeof(search_one_dual_func_table) /
-                            sizeof(*search_one_dual_func_table));
+    for (int i = 0; i < (int)(sizeof(search_one_dual_func_table) /
+                              sizeof(*search_one_dual_func_table));
          ++i) {
         for (int j = 0; j < nb_strengths; ++j) {
             uint64_t best_mse_ref, best_mse_tst;
@@ -856,6 +903,10 @@ TEST(CdefToolTest, DISABLED_SearchOneDualSpeedTest) {
         }
     }
 
+    for (int i = 0; i < sb_count; i++) {
+        svt_aom_free(mse[0][i]);
+        svt_aom_free(mse[1][i]);
+    }
     svt_aom_free(mse[0]);
     svt_aom_free(mse[1]);
 }
