@@ -1,14 +1,15 @@
 /*
-* Copyright(c) 2019 Netflix, Inc.
-* Copyright (c) 2016, Alliance for Open Media. All rights reserved
-*
-* This source code is subject to the terms of the BSD 2 Clause License and
-* the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
-* was not distributed with this source code in the LICENSE file, you can
-* obtain it at https://www.aomedia.org/license/software-license. If the Alliance for Open
-* Media Patent License 1.0 was not distributed with this source code in the
-* PATENTS file, you can obtain it at https://www.aomedia.org/license/patent-license.
-*/
+ * Copyright(c) 2019 Netflix, Inc.
+ * Copyright (c) 2016, Alliance for Open Media. All rights reserved
+ *
+ * This source code is subject to the terms of the BSD 2 Clause License and
+ * the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
+ * was not distributed with this source code in the LICENSE file, you can
+ * obtain it at https://www.aomedia.org/license/software-license. If the
+ * Alliance for Open Media Patent License 1.0 was not distributed with this
+ * source code in the PATENTS file, you can obtain it at
+ * https://www.aomedia.org/license/patent-license.
+ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -115,9 +116,9 @@ class BlendA64MaskTest : public ACMRandom {
         for (int r = 0; r < h_; ++r) {
             for (int c = 0; c < w_; ++c) {
                 ASSERT_EQ(dst_ref_[dst_offset_ + r * dst_stride_ + c],
-                    dst_tst_[dst_offset_ + r * dst_stride_ + c])
-                    << w_ << "x" << h_ << " subx " << subx_ << " suby "
-                    << suby_ << " r: " << r << " c: " << c;
+                          dst_tst_[dst_offset_ + r * dst_stride_ + c])
+                    << w_ << "x" << h_ << " subx " << subx_ << " suby " << suby_
+                    << " r: " << r << " c: " << c;
             }
         }
     }
@@ -209,14 +210,32 @@ class BlendA64MaskTest8B : public BlendA64MaskTest<F8B, uint8_t, uint8_t> {
     void Execute(const uint8_t *p_src0, const uint8_t *p_src1,
                  int run_times) override {
         for (int i = 0; i < run_times; ++i) {
-            ref_func_(dst_ref_ + dst_offset_, dst_stride_, p_src0 + src0_offset_,
-                      src0_stride_, p_src1 + src1_offset_, src1_stride_,
-                      mask_, kMaxMaskWidth, w_, h_, subx_, suby_);
+            ref_func_(dst_ref_ + dst_offset_,
+                      dst_stride_,
+                      p_src0 + src0_offset_,
+                      src0_stride_,
+                      p_src1 + src1_offset_,
+                      src1_stride_,
+                      mask_,
+                      kMaxMaskWidth,
+                      w_,
+                      h_,
+                      subx_,
+                      suby_);
         }
         for (int i = 0; i < run_times; ++i) {
-            tst_func_(dst_tst_ + dst_offset_, dst_stride_, p_src0 + src0_offset_,
-                      src0_stride_, p_src1 + src1_offset_, src1_stride_,
-                      mask_, kMaxMaskWidth, w_, h_, subx_, suby_);
+            tst_func_(dst_tst_ + dst_offset_,
+                      dst_stride_,
+                      p_src0 + src0_offset_,
+                      src0_stride_,
+                      p_src1 + src1_offset_,
+                      src1_stride_,
+                      mask_,
+                      kMaxMaskWidth,
+                      w_,
+                      h_,
+                      subx_,
+                      suby_);
         }
     }
 };
@@ -228,8 +247,8 @@ class BlendA64MaskTest8B : public BlendA64MaskTest<F8B, uint8_t, uint8_t> {
         delete test;                                \
     }
 
-TEST_CLASS(BlendA64MaskTest8B, svt_aom_blend_a64_mask_c, svt_aom_blend_a64_mask_sse4_1,
-           Mask_Blend_SSE4_1)
+TEST_CLASS(BlendA64MaskTest8B, svt_aom_blend_a64_mask_c,
+           svt_aom_blend_a64_mask_sse4_1, Mask_Blend_SSE4_1)
 TEST_CLASS(BlendA64MaskTest8B, svt_aom_blend_a64_mask_sse4_1,
            svt_aom_blend_a64_mask_avx2, Mask_Blend_AVX2)
 
@@ -245,58 +264,74 @@ typedef void (*F8B_D16)(uint8_t *dst, uint32_t dst_stride, const uint16_t *src0,
 
 class BlendA64MaskTest8B_d16
     : public BlendA64MaskTest<F8B_D16, uint16_t, uint8_t> {
-  // max number of bits used by the source
-  static const int kSrcMaxBitsMask = 0x3fff;
+    // max number of bits used by the source
+    static const int kSrcMaxBitsMask = 0x3fff;
 
-public:
-  BlendA64MaskTest8B_d16(F8B_D16 ref, F8B_D16 tst) {
-      ref_func_ = ref;
-      tst_func_ = tst;
-  }
-
-  void prepare_data(int type) override {
-      UNUSED(type);
-      if (!type) {
-          for (int i = 0; i < kBufSize; ++i) {
-              dst_ref_[i] = this->Rand8();
-              dst_tst_[i] = this->Rand8();
-              src0_[i] = this->Rand16() & kSrcMaxBitsMask;
-              src1_[i] = this->Rand16() & kSrcMaxBitsMask;
-          }
-          for (int i = 0; i < kMaxMaskSize; ++i)
-              mask_[i] = this->PseudoUniform(AOM_BLEND_A64_MAX_ALPHA + 1);
-      }
-      else {
-          for (int i = 0; i < kBufSize; ++i) {
-              dst_ref_[i] = (uint16_t)255;
-              dst_tst_[i] = (uint16_t)255;
-              src0_[i] = (uint16_t)kSrcMaxBitsMask;
-              src1_[i] = (uint16_t)kSrcMaxBitsMask;
-          }
-          for (int i = 0; i < kMaxMaskSize; ++i)
-              mask_[i] = AOM_BLEND_A64_MAX_ALPHA - 1;
-      }
-  }
-
-  void Execute(const uint16_t *p_src0, const uint16_t *p_src1,
-      int run_times) override
-  {
-    ConvolveParams conv_params;
-    conv_params.round_0 = ROUND0_BITS;
-    conv_params.round_1 = COMPOUND_ROUND1_BITS;
-    for (int i = 0; i < run_times; ++i) {
-      ref_func_(dst_ref_ + dst_offset_, dst_stride_,
-                       p_src0 + src0_offset_, src0_stride_,
-                       p_src1 + src1_offset_, src1_stride_, mask_,
-                       kMaxMaskWidth, w_, h_, subx_, suby_, &conv_params);
+  public:
+    BlendA64MaskTest8B_d16(F8B_D16 ref, F8B_D16 tst) {
+        ref_func_ = ref;
+        tst_func_ = tst;
     }
-    for (int i = 0; i < run_times; ++i) {
-      tst_func_(dst_tst_ + dst_offset_, dst_stride_,
-                       p_src0 + src0_offset_, src0_stride_,
-                       p_src1 + src1_offset_, src1_stride_, mask_,
-                       kMaxMaskWidth, w_, h_, subx_, suby_, &conv_params);
+
+    void prepare_data(int type) override {
+        UNUSED(type);
+        if (!type) {
+            for (int i = 0; i < kBufSize; ++i) {
+                dst_ref_[i] = this->Rand8();
+                dst_tst_[i] = this->Rand8();
+                src0_[i] = this->Rand16() & kSrcMaxBitsMask;
+                src1_[i] = this->Rand16() & kSrcMaxBitsMask;
+            }
+            for (int i = 0; i < kMaxMaskSize; ++i)
+                mask_[i] = this->PseudoUniform(AOM_BLEND_A64_MAX_ALPHA + 1);
+        } else {
+            for (int i = 0; i < kBufSize; ++i) {
+                dst_ref_[i] = (uint16_t)255;
+                dst_tst_[i] = (uint16_t)255;
+                src0_[i] = (uint16_t)kSrcMaxBitsMask;
+                src1_[i] = (uint16_t)kSrcMaxBitsMask;
+            }
+            for (int i = 0; i < kMaxMaskSize; ++i)
+                mask_[i] = AOM_BLEND_A64_MAX_ALPHA - 1;
+        }
+    }
+
+    void Execute(const uint16_t *p_src0, const uint16_t *p_src1,
+                 int run_times) override {
+        ConvolveParams conv_params;
+        conv_params.round_0 = ROUND0_BITS;
+        conv_params.round_1 = COMPOUND_ROUND1_BITS;
+        for (int i = 0; i < run_times; ++i) {
+            ref_func_(dst_ref_ + dst_offset_,
+                      dst_stride_,
+                      p_src0 + src0_offset_,
+                      src0_stride_,
+                      p_src1 + src1_offset_,
+                      src1_stride_,
+                      mask_,
+                      kMaxMaskWidth,
+                      w_,
+                      h_,
+                      subx_,
+                      suby_,
+                      &conv_params);
+        }
+        for (int i = 0; i < run_times; ++i) {
+            tst_func_(dst_tst_ + dst_offset_,
+                      dst_stride_,
+                      p_src0 + src0_offset_,
+                      src0_stride_,
+                      p_src1 + src1_offset_,
+                      src1_stride_,
+                      mask_,
+                      kMaxMaskWidth,
+                      w_,
+                      h_,
+                      subx_,
+                      suby_,
+                      &conv_params);
+        }
     }
-  }
 };
 
 TEST_CLASS(BlendA64MaskTest8B_d16, svt_aom_lowbd_blend_a64_d16_mask_c,
@@ -358,16 +393,34 @@ class BlendA64MaskTestHBD : public BlendA64MaskTest<FHBD, uint16_t, uint16_t> {
     void Execute(const uint16_t *p_src0, const uint16_t *p_src1,
                  int run_times) override {
         for (int i = 0; i < run_times; ++i) {
-            ref_func_((uint8_t *)(dst_ref_ + dst_offset_), dst_stride_,
-                      (uint8_t *)(p_src0 + src0_offset_), src0_stride_,
-                      (uint8_t *)(p_src1 + src1_offset_), src1_stride_,
-                      mask_, kMaxMaskWidth, w_,  h_, subx_, suby_, bit_depth_);
+            ref_func_((uint8_t *)(dst_ref_ + dst_offset_),
+                      dst_stride_,
+                      (uint8_t *)(p_src0 + src0_offset_),
+                      src0_stride_,
+                      (uint8_t *)(p_src1 + src1_offset_),
+                      src1_stride_,
+                      mask_,
+                      kMaxMaskWidth,
+                      w_,
+                      h_,
+                      subx_,
+                      suby_,
+                      bit_depth_);
         }
         for (int i = 0; i < run_times; ++i) {
-            tst_func_((uint8_t *)(dst_tst_ + dst_offset_), dst_stride_,
-                      (uint8_t *)(p_src0 + src0_offset_), src0_stride_,
-                      (uint8_t *)(p_src1 + src1_offset_), src1_stride_,
-                      mask_, kMaxMaskWidth, w_, h_, subx_, suby_, bit_depth_);
+            tst_func_((uint8_t *)(dst_tst_ + dst_offset_),
+                      dst_stride_,
+                      (uint8_t *)(p_src0 + src0_offset_),
+                      src0_stride_,
+                      (uint8_t *)(p_src1 + src1_offset_),
+                      src1_stride_,
+                      mask_,
+                      kMaxMaskWidth,
+                      w_,
+                      h_,
+                      subx_,
+                      suby_,
+                      bit_depth_);
         }
     }
 
@@ -390,81 +443,100 @@ typedef void (*FHBD_D16)(uint8_t *dst, uint32_t dst_stride,
 
 class BlendA64MaskTestHBD_d16
     : public BlendA64MaskTest<FHBD_D16, uint16_t, uint16_t> {
- public:
-  // max number of bits used by the source
-  static const int kSrcMaxBitsMask = (1 << 14) - 1;
-  static const int kSrcMaxBitsMaskHBD = (1 << 16) - 1;
-  BlendA64MaskTestHBD_d16(FHBD_D16 ref, FHBD_D16 tst) {
-      ref_func_ = ref;
-      tst_func_ = tst;
-  }
-  void prepare_data(int type) override {
-      if (!type) {
-          switch (this->PseudoUniform(3)) {
-          case 0: bit_depth_ = 8; break;
-          case 1: bit_depth_ = 10; break;
-          default: bit_depth_ = 12; break;
-          }
-          src_max_bits_mask_ =
-              (bit_depth_ == 8) ? kSrcMaxBitsMask : kSrcMaxBitsMaskHBD;
-
-          for (int i = 0; i < kBufSize; ++i) {
-              dst_ref_[i] = this->Rand8();
-              dst_tst_[i] = this->Rand8();
-              src0_[i] = this->Rand16() & src_max_bits_mask_;
-              src1_[i] = this->Rand16() & src_max_bits_mask_;
-          }
-          for (int i = 0; i < kMaxMaskSize; ++i)
-              mask_[i] = this->PseudoUniform(AOM_BLEND_A64_MAX_ALPHA + 1);
-      } else {
-          switch (this->PseudoUniform(3)) {
-          case 0: bit_depth_ = 8; break;
-          case 1: bit_depth_ = 10; break;
-          default: bit_depth_ = 12; break;
-          }
-
-          const int hi = 1 << bit_depth_;
-          const int lo = hi - 2;
-
-          for (int i = 0; i < kBufSize; ++i) {
-              dst_ref_[i] = this->PseudoUniform(hi - lo) + lo;
-              dst_tst_[i] = this->PseudoUniform(hi - lo) + lo;
-              src0_[i] = this->PseudoUniform(hi - lo) + lo;
-              src1_[i] = this->PseudoUniform(hi - lo) + lo;
-          }
-
-          for (int i = 0; i < kMaxMaskSize; ++i)
-              mask_[i] = this->PseudoUniform(2) + AOM_BLEND_A64_MAX_ALPHA - 1;
-      }
-  }
-  void Execute(const uint16_t *p_src0, const uint16_t *p_src1, int run_times) override
-  {
-    ASSERT_GT(run_times, 0) << "Cannot run 0 iterations of the test.";
-    ConvolveParams conv_params;
-    conv_params.round_0 = (bit_depth_ == 12) ? ROUND0_BITS + 2 : ROUND0_BITS;
-    conv_params.round_1 = COMPOUND_ROUND1_BITS;
-    for (int i = 0; i < run_times; ++i) {
-      ref_func_((uint8_t *)(dst_ref_ + dst_offset_), dst_stride_,
-                p_src0 + src0_offset_, src0_stride_,
-                p_src1 + src1_offset_, src1_stride_, mask_,
-                kMaxMaskWidth, w_, h_, subx_, suby_, &conv_params,
-                bit_depth_);
+  public:
+    // max number of bits used by the source
+    static const int kSrcMaxBitsMask = (1 << 14) - 1;
+    static const int kSrcMaxBitsMaskHBD = (1 << 16) - 1;
+    BlendA64MaskTestHBD_d16(FHBD_D16 ref, FHBD_D16 tst) {
+        ref_func_ = ref;
+        tst_func_ = tst;
     }
-    if (tst_func_) {
-      for (int i = 0; i < run_times; ++i) {
-        tst_func_((uint8_t *)(dst_tst_ + dst_offset_),
-                  dst_stride_, p_src0 + src0_offset_, src0_stride_,
-                  p_src1 + src1_offset_, src1_stride_, mask_,
-                  kMaxMaskWidth, w_, h_, subx_, suby_, &conv_params,
-                  bit_depth_);
-      }
-    }
-  }
+    void prepare_data(int type) override {
+        if (!type) {
+            switch (this->PseudoUniform(3)) {
+            case 0: bit_depth_ = 8; break;
+            case 1: bit_depth_ = 10; break;
+            default: bit_depth_ = 12; break;
+            }
+            src_max_bits_mask_ =
+                (bit_depth_ == 8) ? kSrcMaxBitsMask : kSrcMaxBitsMaskHBD;
 
-  int bit_depth_;
-  int src_max_bits_mask_;
+            for (int i = 0; i < kBufSize; ++i) {
+                dst_ref_[i] = this->Rand8();
+                dst_tst_[i] = this->Rand8();
+                src0_[i] = this->Rand16() & src_max_bits_mask_;
+                src1_[i] = this->Rand16() & src_max_bits_mask_;
+            }
+            for (int i = 0; i < kMaxMaskSize; ++i)
+                mask_[i] = this->PseudoUniform(AOM_BLEND_A64_MAX_ALPHA + 1);
+        } else {
+            switch (this->PseudoUniform(3)) {
+            case 0: bit_depth_ = 8; break;
+            case 1: bit_depth_ = 10; break;
+            default: bit_depth_ = 12; break;
+            }
+
+            const int hi = 1 << bit_depth_;
+            const int lo = hi - 2;
+
+            for (int i = 0; i < kBufSize; ++i) {
+                dst_ref_[i] = this->PseudoUniform(hi - lo) + lo;
+                dst_tst_[i] = this->PseudoUniform(hi - lo) + lo;
+                src0_[i] = this->PseudoUniform(hi - lo) + lo;
+                src1_[i] = this->PseudoUniform(hi - lo) + lo;
+            }
+
+            for (int i = 0; i < kMaxMaskSize; ++i)
+                mask_[i] = this->PseudoUniform(2) + AOM_BLEND_A64_MAX_ALPHA - 1;
+        }
+    }
+    void Execute(const uint16_t *p_src0, const uint16_t *p_src1,
+                 int run_times) override {
+        ASSERT_GT(run_times, 0) << "Cannot run 0 iterations of the test.";
+        ConvolveParams conv_params;
+        conv_params.round_0 =
+            (bit_depth_ == 12) ? ROUND0_BITS + 2 : ROUND0_BITS;
+        conv_params.round_1 = COMPOUND_ROUND1_BITS;
+        for (int i = 0; i < run_times; ++i) {
+            ref_func_((uint8_t *)(dst_ref_ + dst_offset_),
+                      dst_stride_,
+                      p_src0 + src0_offset_,
+                      src0_stride_,
+                      p_src1 + src1_offset_,
+                      src1_stride_,
+                      mask_,
+                      kMaxMaskWidth,
+                      w_,
+                      h_,
+                      subx_,
+                      suby_,
+                      &conv_params,
+                      bit_depth_);
+        }
+        if (tst_func_) {
+            for (int i = 0; i < run_times; ++i) {
+                tst_func_((uint8_t *)(dst_tst_ + dst_offset_),
+                          dst_stride_,
+                          p_src0 + src0_offset_,
+                          src0_stride_,
+                          p_src1 + src1_offset_,
+                          src1_stride_,
+                          mask_,
+                          kMaxMaskWidth,
+                          w_,
+                          h_,
+                          subx_,
+                          suby_,
+                          &conv_params,
+                          bit_depth_);
+            }
+        }
+    }
+
+    int bit_depth_;
+    int src_max_bits_mask_;
 };
 
 TEST_CLASS(BlendA64MaskTestHBD_d16, svt_aom_highbd_blend_a64_d16_mask_c,
            svt_aom_highbd_blend_a64_d16_mask_avx2, _Mask_Blend_Hbd_d16_AVX2)
-}; // namespace
+};  // namespace
