@@ -1,13 +1,14 @@
 /*
-* Copyright(c) 2019 Netflix, Inc.
-*
-* This source code is subject to the terms of the BSD 2 Clause License and
-* the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
-* was not distributed with this source code in the LICENSE file, you can
-* obtain it at https://www.aomedia.org/license/software-license. If the Alliance for Open
-* Media Patent License 1.0 was not distributed with this source code in the
-* PATENTS file, you can obtain it at https://www.aomedia.org/license/patent-license.
-*/
+ * Copyright(c) 2019 Netflix, Inc.
+ *
+ * This source code is subject to the terms of the BSD 2 Clause License and
+ * the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
+ * was not distributed with this source code in the LICENSE file, you can
+ * obtain it at https://www.aomedia.org/license/software-license. If the
+ * Alliance for Open Media Patent License 1.0 was not distributed with this
+ * source code in the PATENTS file, you can obtain it at
+ * https://www.aomedia.org/license/patent-license.
+ */
 
 /******************************************************************************
  * @file FwdTxfm2dAsmTest.c
@@ -49,7 +50,8 @@ using svt_av1_test_tool::SVTRandom;
 
 namespace {
 using FwdTxfm2dAsmParam = std::tuple<int, int>;
-static const FwdTxfm2dFunc fwd_txfm_2d_asm_func[TX_SIZES_ALL] = {
+
+static const FwdTxfm2dFunc fwd_txfm_2d_avx2_func[TX_SIZES_ALL] = {
     svt_av1_fwd_txfm2d_4x4_sse4_1, svt_av1_fwd_txfm2d_8x8_avx2,
     svt_av1_fwd_txfm2d_16x16_avx2, svt_av1_fwd_txfm2d_32x32_avx2,
     svt_av1_fwd_txfm2d_64x64_avx2, svt_av1_fwd_txfm2d_4x8_avx2,
@@ -62,8 +64,21 @@ static const FwdTxfm2dFunc fwd_txfm_2d_asm_func[TX_SIZES_ALL] = {
     svt_av1_fwd_txfm2d_64x16_avx2,
 };
 
+static const FwdTxfm2dFunc fwd_txfm_2d_sse4_1_func[TX_SIZES_ALL] = {
+    svt_av1_fwd_txfm2d_4x4_sse4_1,   svt_av1_fwd_txfm2d_8x8_sse4_1,
+    svt_av1_fwd_txfm2d_16x16_sse4_1, svt_av1_fwd_txfm2d_32x32_sse4_1,
+    svt_av1_fwd_txfm2d_64x64_sse4_1, svt_av1_fwd_txfm2d_4x8_sse4_1,
+    svt_av1_fwd_txfm2d_8x4_sse4_1,   svt_av1_fwd_txfm2d_8x16_sse4_1,
+    svt_av1_fwd_txfm2d_16x8_sse4_1,  svt_av1_fwd_txfm2d_16x32_sse4_1,
+    svt_av1_fwd_txfm2d_32x16_sse4_1, svt_av1_fwd_txfm2d_32x64_sse4_1,
+    svt_av1_fwd_txfm2d_64x32_sse4_1, svt_av1_fwd_txfm2d_4x16_sse4_1,
+    svt_av1_fwd_txfm2d_16x4_sse4_1,  svt_av1_fwd_txfm2d_8x32_sse4_1,
+    svt_av1_fwd_txfm2d_32x8_sse4_1,  svt_av1_fwd_txfm2d_16x64_sse4_1,
+    svt_av1_fwd_txfm2d_64x16_sse4_1,
+};
+
 static const FwdTxfm2dFunc fwd_txfm_2d_N2_asm_func[TX_SIZES_ALL] = {
-   svt_av1_fwd_txfm2d_4x4_N2_sse4_1, svt_av1_fwd_txfm2d_8x8_N2_avx2,
+    svt_av1_fwd_txfm2d_4x4_N2_sse4_1, svt_av1_fwd_txfm2d_8x8_N2_avx2,
     svt_av1_fwd_txfm2d_16x16_N2_avx2, svt_av1_fwd_txfm2d_32x32_N2_avx2,
     svt_av1_fwd_txfm2d_64x64_N2_avx2, svt_av1_fwd_txfm2d_4x8_N2_avx2,
     svt_av1_fwd_txfm2d_8x4_N2_avx2,   svt_av1_fwd_txfm2d_8x16_N2_avx2,
@@ -92,12 +107,12 @@ static const FwdTxfm2dFunc fwd_txfm_2d_N2_c_func[TX_SIZES_ALL] = {
     av1_transform_two_d_4x4_N2_c,   av1_transform_two_d_8x8_N2_c,
     av1_transform_two_d_16x16_N2_c, av1_transform_two_d_32x32_N2_c,
     av1_transform_two_d_64x64_N2_c, svt_av1_fwd_txfm2d_4x8_N2_c,
-    svt_av1_fwd_txfm2d_8x4_N2_c,     svt_av1_fwd_txfm2d_8x16_N2_c,
-    svt_av1_fwd_txfm2d_16x8_N2_c,    svt_av1_fwd_txfm2d_16x32_N2_c,
-    svt_av1_fwd_txfm2d_32x16_N2_c,   svt_av1_fwd_txfm2d_32x64_N2_c,
-    svt_av1_fwd_txfm2d_64x32_N2_c,   svt_av1_fwd_txfm2d_4x16_N2_c,
-    svt_av1_fwd_txfm2d_16x4_N2_c,    svt_av1_fwd_txfm2d_8x32_N2_c,
-    svt_av1_fwd_txfm2d_32x8_N2_c,    svt_av1_fwd_txfm2d_16x64_N2_c,
+    svt_av1_fwd_txfm2d_8x4_N2_c,    svt_av1_fwd_txfm2d_8x16_N2_c,
+    svt_av1_fwd_txfm2d_16x8_N2_c,   svt_av1_fwd_txfm2d_16x32_N2_c,
+    svt_av1_fwd_txfm2d_32x16_N2_c,  svt_av1_fwd_txfm2d_32x64_N2_c,
+    svt_av1_fwd_txfm2d_64x32_N2_c,  svt_av1_fwd_txfm2d_4x16_N2_c,
+    svt_av1_fwd_txfm2d_16x4_N2_c,   svt_av1_fwd_txfm2d_8x32_N2_c,
+    svt_av1_fwd_txfm2d_32x8_N2_c,   svt_av1_fwd_txfm2d_16x64_N2_c,
     svt_av1_fwd_txfm2d_64x16_N2_c,
 };
 
@@ -105,12 +120,12 @@ static const FwdTxfm2dFunc fwd_txfm_2d_N4_c_func[TX_SIZES_ALL] = {
     av1_transform_two_d_4x4_N4_c,   av1_transform_two_d_8x8_N4_c,
     av1_transform_two_d_16x16_N4_c, av1_transform_two_d_32x32_N4_c,
     av1_transform_two_d_64x64_N4_c, svt_av1_fwd_txfm2d_4x8_N4_c,
-    svt_av1_fwd_txfm2d_8x4_N4_c,     svt_av1_fwd_txfm2d_8x16_N4_c,
-    svt_av1_fwd_txfm2d_16x8_N4_c,    svt_av1_fwd_txfm2d_16x32_N4_c,
-    svt_av1_fwd_txfm2d_32x16_N4_c,   svt_av1_fwd_txfm2d_32x64_N4_c,
-    svt_av1_fwd_txfm2d_64x32_N4_c,   svt_av1_fwd_txfm2d_4x16_N4_c,
-    svt_av1_fwd_txfm2d_16x4_N4_c,    svt_av1_fwd_txfm2d_8x32_N4_c,
-    svt_av1_fwd_txfm2d_32x8_N4_c,    svt_av1_fwd_txfm2d_16x64_N4_c,
+    svt_av1_fwd_txfm2d_8x4_N4_c,    svt_av1_fwd_txfm2d_8x16_N4_c,
+    svt_av1_fwd_txfm2d_16x8_N4_c,   svt_av1_fwd_txfm2d_16x32_N4_c,
+    svt_av1_fwd_txfm2d_32x16_N4_c,  svt_av1_fwd_txfm2d_32x64_N4_c,
+    svt_av1_fwd_txfm2d_64x32_N4_c,  svt_av1_fwd_txfm2d_4x16_N4_c,
+    svt_av1_fwd_txfm2d_16x4_N4_c,   svt_av1_fwd_txfm2d_8x32_N4_c,
+    svt_av1_fwd_txfm2d_32x8_N4_c,   svt_av1_fwd_txfm2d_16x64_N4_c,
     svt_av1_fwd_txfm2d_64x16_N4_c,
 };
 
@@ -191,8 +206,14 @@ class FwdTxfm2dAsmTest : public ::testing::TestWithParam<FwdTxfm2dAsmParam> {
         aom_clear_system_state();
     }
 
-    void run_match_test_default() {
-        FwdTxfm2dFunc test_func = fwd_txfm_2d_asm_func[tx_size_];
+    void run_match_test_avx2() {
+        FwdTxfm2dFunc test_func = fwd_txfm_2d_avx2_func[tx_size_];
+        FwdTxfm2dFunc ref_func = fwd_txfm_2d_c_func[tx_size_];
+        execute_test(test_func, ref_func, DEFAULT_SHAPE);
+    }
+
+    void run_match_test_sse4_1() {
+        FwdTxfm2dFunc test_func = fwd_txfm_2d_sse4_1_func[tx_size_];
         FwdTxfm2dFunc ref_func = fwd_txfm_2d_c_func[tx_size_];
         execute_test(test_func, ref_func, DEFAULT_SHAPE);
     }
@@ -213,8 +234,8 @@ class FwdTxfm2dAsmTest : public ::testing::TestWithParam<FwdTxfm2dAsmParam> {
         execute_test(test_func_c, ref_func, N4_SHAPE);
     }
 
-    void speed_test() {
-        FwdTxfm2dFunc test_func = fwd_txfm_2d_asm_func[tx_size_];
+    void speed_test_avx2() {
+        FwdTxfm2dFunc test_func = fwd_txfm_2d_avx2_func[tx_size_];
         FwdTxfm2dFunc ref_func = fwd_txfm_2d_c_func[tx_size_];
         run_speed_test("C   AND ASM", test_func, ref_func);
         run_speed_test(
@@ -222,6 +243,12 @@ class FwdTxfm2dAsmTest : public ::testing::TestWithParam<FwdTxfm2dAsmParam> {
         run_speed_test(
             "ASM AND N4 ", fwd_txfm_2d_N4_asm_func[tx_size_], test_func);
     }
+
+    void speed_test_sse4_1() {
+        FwdTxfm2dFunc test_func = fwd_txfm_2d_sse4_1_func[tx_size_];
+        FwdTxfm2dFunc ref_func = fwd_txfm_2d_c_func[tx_size_];
+        run_speed_test("C   AND ASM", test_func, ref_func);
+    }
 #if EN_AVX512_SUPPORT
     void run_match_test_N2_512() {
         FwdTxfm2dFunc test_func_asm = fwd_txfm_2d_N2_asm512_func[tx_size_];
@@ -295,7 +322,9 @@ class FwdTxfm2dAsmTest : public ::testing::TestWithParam<FwdTxfm2dAsmParam> {
                 }
                 test_func(input_, output_test_, stride_, type, (uint8_t)bd_);
 
-                if (0 != memcmp(output_test_, output_ref_,
+                if (0 !=
+                    memcmp(output_test_,
+                           output_ref_,
                            MAX_TX_SQUARE * sizeof(int32_t) + TEST_OFFSET)) {
                     for (int i = 0; i < height_; i++)
                         for (int j = 0; j < width_; j++) {
@@ -365,22 +394,25 @@ class FwdTxfm2dAsmTest : public ::testing::TestWithParam<FwdTxfm2dAsmParam> {
             }
             svt_av1_get_time(&finish_time_seconds, &finish_time_useconds);
 
-            time_c = svt_av1_compute_overall_elapsed_time_ms(start_time_seconds,
-                                                             start_time_useconds,
-                                                             middle_time_seconds,
-                                                             middle_time_useconds);
+            time_c =
+                svt_av1_compute_overall_elapsed_time_ms(start_time_seconds,
+                                                        start_time_useconds,
+                                                        middle_time_seconds,
+                                                        middle_time_useconds);
 
-            time_o = svt_av1_compute_overall_elapsed_time_ms(middle_time_seconds,
-                                                             middle_time_useconds,
-                                                             finish_time_seconds,
-                                                             finish_time_useconds);
+            time_o =
+                svt_av1_compute_overall_elapsed_time_ms(middle_time_seconds,
+                                                        middle_time_useconds,
+                                                        finish_time_seconds,
+                                                        finish_time_useconds);
 
-            printf("[%s]; Transform: ;%02ix%02i; %17s; Speed compare: ;%5.2fx\n",
-                    name_cmp,
-                    tx_size_wide[tx_size_],
-                    tx_size_high[tx_size_],
-                    tx_type_name[tx_type],
-                    time_c / time_o);
+            printf(
+                "[%s]; Transform: ;%02ix%02i; %17s; Speed compare: ;%5.2fx\n",
+                name_cmp,
+                tx_size_wide[tx_size_],
+                tx_size_high[tx_size_],
+                tx_type_name[tx_type],
+                time_c / time_o);
         }
     }
     void populate_with_random() {
@@ -410,8 +442,12 @@ class FwdTxfm2dAsmTest : public ::testing::TestWithParam<FwdTxfm2dAsmParam> {
     int32_t *output_ref_;  /**< aligned address for output ref */
 };
 
-TEST_P(FwdTxfm2dAsmTest, match_test) {
-    run_match_test_default();
+TEST_P(FwdTxfm2dAsmTest, match_test_avx2) {
+    run_match_test_avx2();
+}
+
+TEST_P(FwdTxfm2dAsmTest, match_test_sse4_1) {
+    run_match_test_sse4_1();
 }
 
 TEST_P(FwdTxfm2dAsmTest, match_test_N2) {
@@ -422,8 +458,12 @@ TEST_P(FwdTxfm2dAsmTest, match_test_N4) {
     run_match_test_N4();
 }
 
-TEST_P(FwdTxfm2dAsmTest, DISABLED_speed_test) {
-    speed_test();
+TEST_P(FwdTxfm2dAsmTest, DISABLED_speed_test_avx2) {
+    speed_test_avx2();
+}
+
+TEST_P(FwdTxfm2dAsmTest, DISABLED_speed_test_sse4_1) {
+    speed_test_sse4_1();
 }
 
 #if EN_AVX512_SUPPORT
