@@ -1,13 +1,14 @@
 /*
-* Copyright(c) 2019 Netflix, Inc.
-*
-* This source code is subject to the terms of the BSD 2 Clause License and
-* the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
-* was not distributed with this source code in the LICENSE file, you can
-* obtain it at https://www.aomedia.org/license/software-license. If the Alliance for Open
-* Media Patent License 1.0 was not distributed with this source code in the
-* PATENTS file, you can obtain it at https://www.aomedia.org/license/patent-license.
-*/
+ * Copyright(c) 2019 Netflix, Inc.
+ *
+ * This source code is subject to the terms of the BSD 2 Clause License and
+ * the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
+ * was not distributed with this source code in the LICENSE file, you can
+ * obtain it at https://www.aomedia.org/license/software-license. If the
+ * Alliance for Open Media Patent License 1.0 was not distributed with this
+ * source code in the PATENTS file, you can obtain it at
+ * https://www.aomedia.org/license/patent-license.
+ */
 
 /******************************************************************************
  * @file params.h
@@ -111,17 +112,15 @@ static const vector<int32_t> invalid_intra_period_length = {
  * 1 = CRA, open GOP.
  * 2 = IDR, closed GOP.
  *
- * Default is 1. */
-static const vector<uint32_t> default_intra_refresh_type = {
-    1,
+ * Default is 2. */
+static const vector<SvtAv1IntraRefreshType> default_intra_refresh_type = {
+    SVT_AV1_KF_REFRESH,
 };
-static const vector<uint32_t> valid_intra_refresh_type = {
-    1,  // CRA, open GOP
-    2,  // IDR, closed GOP
+static const vector<SvtAv1IntraRefreshType> valid_intra_refresh_type = {
+    SVT_AV1_FWDKF_REFRESH,  // CRA, open GOP
+    SVT_AV1_KF_REFRESH,  // IDR, closed GOP
 };
-static const vector<uint32_t> invalid_intra_refresh_type = {
-    0,
-    3,
+static const vector<SvtAv1IntraRefreshType> invalid_intra_refresh_type = {
 };
 
 /* Number of hierarchical layers used to construct GOP.
@@ -129,11 +128,11 @@ static const vector<uint32_t> invalid_intra_refresh_type = {
  *
  * Default is 3. */
 static const vector<uint32_t> default_hierarchical_levels = {
-    4,
+    3,
 };
-static const vector<uint32_t> valid_hierarchical_levels = {3, 4};
+static const vector<uint32_t> valid_hierarchical_levels = {3, 4, 5};
 static const vector<uint32_t> invalid_hierarchical_levels = {
-    0, 1, 2, 5,  // ...
+    0, 1, 2, 6,  // ...
 };
 
 /* Prediction structure used to construct GOP. There are two main structures
@@ -167,7 +166,6 @@ static const vector<uint8_t> invalid_pred_structure = {
     /* _pred_structure override in code
     EB_PRED_TOTAL_COUNT, EB_PRED_TOTAL_COUNT + 1, EB_PRED_INVALID*/};
 
-
 // Input Info
 /* The width of input source in units of picture luma pixels.
  *
@@ -307,7 +305,7 @@ static const vector<uint32_t> default_compressed_ten_bit_format = {
 };
 static const vector<uint32_t> valid_compressed_ten_bit_format = {
     0,
-    //1, Not supported in this version
+    // 1, Not supported in this version
 };
 static const vector<uint32_t> invalid_compressed_ten_bit_format = {
     2, 10,  // ...
@@ -370,25 +368,6 @@ static const vector<uint32_t> invalid_super_block_size = {
     */
 };
 
-/* The maximum partitioning depth with 0 being the superblock depth
- *
- * Default is 4. */
-static const vector<uint32_t> default_partition_depth = {
-    4,
-};
-static const vector<uint32_t> valid_partition_depth = {
-    0,
-    1,
-    2,
-    3,
-    EB_MAX_SB_DEPTH,
-};
-static const vector<uint32_t> invalid_partition_depth = {
-    /* partition_depth override in code
-    (EB_MAX_SB_DEPTH + 1),
-    */
-};
-
 // Quantization
 /* Initial quantization parameter for the Intra pictures used under constant
  * qp rate control mode.
@@ -427,49 +406,19 @@ static const vector<EbBool> invalid_use_qp_file = {
     // none
 };
 
-/* Enable picture QP scaling between hierarchical levels
- *
- * Default is null.*/
-// TODO: the description of this parameter is incorrect, refer to source code,
-// it should be a EbBool
-static const vector<uint32_t> default_enable_qp_scaling_flag = {
-    EB_FALSE,
-};
-static const vector<uint32_t> valid_enable_qp_scaling_flag = {
-    EB_FALSE,
-    EB_TRUE,
-};
-static const vector<uint32_t> invalid_enable_qp_scaling_flag = {
-    // none
-};
 
 // Deblock Filter
-/* Flag to disable the Deblocking Loop Filtering.
+/* Flag to enable the Deblocking Loop Filtering.
  *
- * Default is 0. */
-static const vector<EbBool> default_disable_dlf_flag = {
-    EB_FALSE,
+ * Default is true. */
+static const vector<EbBool> default_enable_dlf_flag = {
+    EB_TRUE,
 };
-static const vector<EbBool> valid_disable_dlf_flag = {
+static const vector<EbBool> valid_enable_dlf_flag = {
     EB_FALSE,
     EB_TRUE,
 };
-static const vector<EbBool> invalid_disable_dlf_flag = {
-    // none
-};
-
-/* Denoise the input picture when noise levels are too high
- * Flag to enable the denoising
- *
- * Default is 0. */
-static const vector<EbBool> default_enable_denoise_flag = {
-    EB_FALSE,
-};
-static const vector<EbBool> valid_enable_denoise_flag = {
-    EB_FALSE,
-    EB_TRUE,
-};
-static const vector<EbBool> invalid_enable_denoise_flag = {
+static const vector<EbBool> invalid_enable_dlf_flag = {
     // none
 };
 
@@ -560,8 +509,6 @@ static const vector<EbBool> invalid_ext_block_flag = {
     // none
 };
 
-
-
 // ME Parameters
 /* Number of search positions in the horizontal direction.
  *
@@ -601,9 +548,9 @@ static const vector<uint32_t> invalid_search_area_height = {
  * 6:Fastest NIC=4/2/1 + No K means for non base + step for non base for
  * most dominant
  * Default is -1. */
-static const vector<int32_t> default_palette_level = { -1 };
-static const vector<int32_t> valid_palette_level = { -1, 0, 1, 2, 3, 4, 5, 6 };
-static const vector<int32_t> invalid_palette_level = { -2, 7 };
+static const vector<int32_t> default_palette_level = {-1};
+static const vector<int32_t> valid_palette_level = {-1, 0, 1, 2, 3, 4, 5, 6};
+static const vector<int32_t> invalid_palette_level = {-2, 7};
 
 /* Enable the use of Constrained Intra, which yields sending two picture
  * parameter sets in the elementary streams .
@@ -645,7 +592,6 @@ static const vector<uint32_t> invalid_scene_change_detection = {
     2,
 };
 
-
 /* Target bitrate in bits/second, only apllicable when rate control mode is
  * set to 1.
  *
@@ -816,13 +762,14 @@ static const vector<CPU_FLAGS> valid_use_cpu_flags = {
     CPU_FLAGS_AVX512BW,
     CPU_FLAGS_AVX512VL,
 };
-static const vector<CPU_FLAGS> invalid_use_cpu_flags = {
-    CPU_FLAGS_INVALID
-};
+static const vector<CPU_FLAGS> invalid_use_cpu_flags = {CPU_FLAGS_INVALID};
 
 // Application Specific parameters
-/* ID assigned to each channel when multiple instances are running within the
- * same application. */
+/**
+ * @brief API signal for the library to know the channel ID (used for pinning to
+ * cores)
+ *
+ */
 static const vector<uint32_t> default_channel_id = {
     0,
 };
@@ -839,6 +786,11 @@ static const vector<uint32_t> invalid_channel_id = {
     // none
 };
 
+/**
+ * @brief API signal for the library to know the active number of channels being
+ * encoded simultaneously
+ *
+ */
 static const vector<uint32_t> default_active_channel_count = {
     1,
 };
@@ -873,38 +825,10 @@ static const vector<uint32_t> invalid_speed_control_flag = {
     2,
 };
 
-/* Frame Rate used for the injector. Recommended to match the encoder speed.
- *
- * Default is 60. */
-static const vector<int32_t> default_injector_frame_rate = {
-    60<<16,
-};
-static const vector<int32_t> valid_injector_frame_rate = {
-    24,
-    25,
-    30,
-    50,
-    60,
-    120,
-    240,
-    24 << 16,
-    25 << 16,
-    30 << 16,
-    50 << 16,
-    60 << 16,
-    120 << 16,
-    240 << 16,  // ...
-};
-static const vector<int32_t> invalid_injector_frame_rate = {
-    /* injector_frame_rate override in code
-    0, 1, 2, 10, 15, 29, 241,  // ...
-    */
-};
-
 // Threads management
 
 /* The number of logical processor which encoder threads run on. If
- * LogicalProcessorNumber and TargetSocket are not set, threads are managed by
+ * LogicalProcessors and TargetSocket are not set, threads are managed by
  * OS thread scheduler. */
 static const vector<uint32_t> default_logical_processors = {
     0,
@@ -1007,16 +931,17 @@ static const vector<uint32_t> invalid_screen_content_mode = {3};
 
 /* Variables to control the use of ALT-REF (temporally filtered frames)
  */
-static const vector<int8_t> default_tf_level = { DEFAULT };
-static const vector<int8_t> valid_tf_level = { DEFAULT, 0, 1, 2, 3 };
-static const vector<int8_t> invalid_tf_level = { -2, 4 };
+static const vector<EbBool> default_enable_tf = {EB_TRUE};
+static const vector<EbBool> valid_enable_tf = {EB_FALSE, EB_TRUE};
+static const vector<EbBool> invalid_enable_tf = {/*none*/};
 
 static const vector<uint8_t> default_altref_strength = {5};
 static const vector<uint8_t> valid_altref_strength = {0, 1, 2, 3, 4, 5, 6};
 static const vector<uint8_t> invalid_altref_strength = {7};
 
 static const vector<uint8_t> default_altref_nframes = {7};
-static const vector<uint8_t> valid_altref_nframes = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
+static const vector<uint8_t> valid_altref_nframes = {
+    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
 static const vector<uint8_t> invalid_altref_nframes = {11};
 
 static const vector<EbBool> default_enable_overlays = {EB_FALSE};
@@ -1025,16 +950,18 @@ static const vector<EbBool> invalid_enable_overlays = {/*none*/};
 
 /* Variables to control the super-resolution tool
  */
-static const vector<EbBool> default_superres_mode = {0};
-static const vector<EbBool> valid_superres_mode = {0, 1, 2};
-static const vector<EbBool> invalid_superres_mode = {3};
+static const vector<uint8_t> default_superres_mode = {0};
+static const vector<uint8_t> valid_superres_mode = {0, 1, 2};
+static const vector<uint8_t> invalid_superres_mode = {3};
 
 static const vector<uint8_t> default_superres_denom = {8};
-static const vector<uint8_t> valid_superres_denom = {8, 9, 10, 11, 12, 13, 14, 15, 16};
+static const vector<uint8_t> valid_superres_denom = {
+    8, 9, 10, 11, 12, 13, 14, 15, 16};
 static const vector<uint8_t> invalid_superres_denom = {7};
 
 static const vector<uint8_t> default_superres_kf_denom = {8};
-static const vector<uint8_t> valid_superres_kf_denom = {8, 9, 10, 11, 12, 13, 14, 15, 16};
+static const vector<uint8_t> valid_superres_kf_denom = {
+    8, 9, 10, 11, 12, 13, 14, 15, 16};
 static const vector<uint8_t> invalid_superres_kf_denom = {7};
 
 // Color description
@@ -1105,7 +1032,8 @@ static const vector<uint32_t> valid_matrix_coefficients = {
     13,
     14,
 };
-static const vector<uint32_t> invalid_matrix_coefficients = {0}; // not actually invalid, but requires 4:4:4
+static const vector<uint32_t> invalid_matrix_coefficients = {
+    0};  // not actually invalid, but requires 4:4:4
 
 }  // namespace svt_av1_test_params
 
