@@ -1,13 +1,14 @@
 /*
-* Copyright(c) 2019 Netflix, Inc.
-*
-* This source code is subject to the terms of the BSD 2 Clause License and
-* the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
-* was not distributed with this source code in the LICENSE file, you can
-* obtain it at https://www.aomedia.org/license/software-license. If the Alliance for Open
-* Media Patent License 1.0 was not distributed with this source code in the
-* PATENTS file, you can obtain it at https://www.aomedia.org/license/patent-license.
-*/
+ * Copyright(c) 2019 Netflix, Inc.
+ *
+ * This source code is subject to the terms of the BSD 2 Clause License and
+ * the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
+ * was not distributed with this source code in the LICENSE file, you can
+ * obtain it at https://www.aomedia.org/license/software-license. If the
+ * Alliance for Open Media Patent License 1.0 was not distributed with this
+ * source code in the PATENTS file, you can obtain it at
+ * https://www.aomedia.org/license/patent-license.
+ */
 
 /******************************************************************************
  * @file Convolve2dTest.cc
@@ -67,6 +68,7 @@ using lowbd_convolve_func = void (*)(const uint8_t *src, int src_stride,
 
 static const lowbd_convolve_func lowbd_convolve_2d_sr_func_table[] = {
     svt_av1_convolve_2d_sr_avx2,
+    svt_av1_convolve_2d_sr_sse2,
 #if EN_AVX512_SUPPORT
     svt_av1_convolve_2d_sr_avx512
 #endif
@@ -74,6 +76,7 @@ static const lowbd_convolve_func lowbd_convolve_2d_sr_func_table[] = {
 
 static const lowbd_convolve_func lowbd_convolve_x_sr_func_table[] = {
     svt_av1_convolve_x_sr_avx2,
+    svt_av1_convolve_x_sr_sse2,
 #if EN_AVX512_SUPPORT
     svt_av1_convolve_x_sr_avx512
 #endif
@@ -81,6 +84,7 @@ static const lowbd_convolve_func lowbd_convolve_x_sr_func_table[] = {
 
 static const lowbd_convolve_func lowbd_convolve_y_sr_func_table[] = {
     svt_av1_convolve_y_sr_avx2,
+    svt_av1_convolve_y_sr_sse2,
 #if EN_AVX512_SUPPORT
     svt_av1_convolve_y_sr_avx512
 #endif
@@ -88,6 +92,7 @@ static const lowbd_convolve_func lowbd_convolve_y_sr_func_table[] = {
 
 static const lowbd_convolve_func lowbd_convolve_copy_sr_func_table[] = {
     svt_av1_convolve_2d_copy_sr_avx2,
+    svt_av1_convolve_2d_copy_sr_sse2,
 #if EN_AVX512_SUPPORT
     svt_av1_convolve_2d_copy_sr_avx512
 #endif
@@ -95,6 +100,8 @@ static const lowbd_convolve_func lowbd_convolve_copy_sr_func_table[] = {
 
 static const lowbd_convolve_func lowbd_jnt_convolve_2d_func_table[] = {
     svt_av1_jnt_convolve_2d_avx2,
+    svt_av1_jnt_convolve_2d_sse2,
+    svt_av1_jnt_convolve_2d_ssse3,
 #if EN_AVX512_SUPPORT
     svt_av1_jnt_convolve_2d_avx512
 #endif
@@ -102,6 +109,7 @@ static const lowbd_convolve_func lowbd_jnt_convolve_2d_func_table[] = {
 
 static const lowbd_convolve_func lowbd_jnt_convolve_x_func_table[] = {
     svt_av1_jnt_convolve_x_avx2,
+    svt_av1_jnt_convolve_x_sse2,
 #if EN_AVX512_SUPPORT
     svt_av1_jnt_convolve_x_avx512
 #endif
@@ -109,6 +117,7 @@ static const lowbd_convolve_func lowbd_jnt_convolve_x_func_table[] = {
 
 static const lowbd_convolve_func lowbd_jnt_convolve_y_func_table[] = {
     svt_av1_jnt_convolve_y_avx2,
+    svt_av1_jnt_convolve_y_sse2,
 #if EN_AVX512_SUPPORT
     svt_av1_jnt_convolve_y_avx512
 #endif
@@ -116,6 +125,7 @@ static const lowbd_convolve_func lowbd_jnt_convolve_y_func_table[] = {
 
 static const lowbd_convolve_func lowbd_jnt_convolve_copy_func_table[] = {
     svt_av1_jnt_convolve_2d_copy_avx2,
+    svt_av1_jnt_convolve_2d_copy_sse2,
 #if EN_AVX512_SUPPORT
     svt_av1_jnt_convolve_2d_copy_avx512
 #endif
@@ -799,15 +809,25 @@ INSTANTIATE_TEST_CASE_P(ConvolveTestY, AV1LbdJntConvolve2DTest,
 INSTANTIATE_TEST_CASE_P(ConvolveTest2D, AV1LbdJntConvolve2DTest,
                         BuildParams(1, 1, 0, 0));
 
+INSTANTIATE_TEST_CASE_P(ConvolveTest2D_sse2, AV1LbdJntConvolve2DTest,
+                        BuildParams(1, 1, 1, 0));
+INSTANTIATE_TEST_CASE_P(ConvolveTest2D_ssse3, AV1LbdJntConvolve2DTest,
+                        BuildParams(1, 1, 2, 0));
+INSTANTIATE_TEST_CASE_P(ConvolveTestX_sse2, AV1LbdJntConvolve2DTest,
+                        BuildParams(1, 0, 1, 0));
+INSTANTIATE_TEST_CASE_P(ConvolveTestY_sse2, AV1LbdJntConvolve2DTest,
+                        BuildParams(0, 1, 1, 0));
+INSTANTIATE_TEST_CASE_P(ConvolveTestCOPY_sse2, AV1LbdJntConvolve2DTest,
+                        BuildParams(0, 0, 1, 0));
 #if EN_AVX512_SUPPORT
 INSTANTIATE_TEST_CASE_P(ConvolveTestCOPY_AVX512, AV1LbdJntConvolve2DTest,
-                        BuildParams(0, 0, 1, 0));
+                        BuildParams(0, 0, 2, 0));
 INSTANTIATE_TEST_CASE_P(ConvolveTestX_AVX512, AV1LbdJntConvolve2DTest,
-                        BuildParams(1, 0, 1, 0));
+                        BuildParams(1, 0, 2, 0));
 INSTANTIATE_TEST_CASE_P(ConvolveTestY_AVX512, AV1LbdJntConvolve2DTest,
-                        BuildParams(0, 1, 1, 0));
+                        BuildParams(0, 1, 2, 0));
 INSTANTIATE_TEST_CASE_P(ConvolveTest2D_AVX512, AV1LbdJntConvolve2DTest,
-                        BuildParams(1, 1, 1, 0));
+                        BuildParams(1, 1, 3, 0));
 #endif
 
 class AV1LbdSrConvolve2DTest : public AV1LbdConvolve2DTest {
@@ -849,15 +869,23 @@ INSTANTIATE_TEST_CASE_P(ConvolveTestY, AV1LbdSrConvolve2DTest,
 INSTANTIATE_TEST_CASE_P(ConvolveTest2D, AV1LbdSrConvolve2DTest,
                         BuildParams(1, 1, 0, 0));
 
+INSTANTIATE_TEST_CASE_P(ConvolveTestX_SSE2, AV1LbdSrConvolve2DTest,
+                        BuildParams(1, 0, 1, 0));
+INSTANTIATE_TEST_CASE_P(ConvolveTestY_SSE2, AV1LbdSrConvolve2DTest,
+                        BuildParams(0, 1, 1, 0));
+INSTANTIATE_TEST_CASE_P(ConvolveTest2D_SSE2, AV1LbdSrConvolve2DTest,
+                        BuildParams(1, 1, 1, 0));
+INSTANTIATE_TEST_CASE_P(ConvolveTestCopy_SSE2, AV1LbdSrConvolve2DTest,
+                        BuildParams(0, 0, 1, 0));
 #if EN_AVX512_SUPPORT
 INSTANTIATE_TEST_CASE_P(ConvolveTestCopy_AVX512, AV1LbdSrConvolve2DTest,
-                        BuildParams(0, 0, 1, 0));
+                        BuildParams(0, 0, 2, 0));
 INSTANTIATE_TEST_CASE_P(ConvolveTestX_AVX512, AV1LbdSrConvolve2DTest,
-                        BuildParams(1, 0, 1, 0));
+                        BuildParams(1, 0, 2, 0));
 INSTANTIATE_TEST_CASE_P(ConvolveTestY_AVX512, AV1LbdSrConvolve2DTest,
-                        BuildParams(0, 1, 1, 0));
+                        BuildParams(0, 1, 2, 0));
 INSTANTIATE_TEST_CASE_P(ConvolveTest2D_AVX512, AV1LbdSrConvolve2DTest,
-                        BuildParams(1, 1, 1, 0));
+                        BuildParams(1, 1, 2, 0));
 #endif
 
 class AV1HbdConvolve2DTest
@@ -970,14 +998,27 @@ class AV1HbdJntConvolve2DTest : public AV1HbdConvolve2DTest {
         func_ref_ = svt_av1_highbd_jnt_convolve_2d_c;
         const int has_subx = TEST_GET_PARAM(1);
         const int has_suby = TEST_GET_PARAM(2);
-        if (has_subx == 1 && has_suby == 1)
-            func_tst_ = svt_av1_highbd_jnt_convolve_2d_avx2;
-        else if (has_subx == 1)
-            func_tst_ = svt_av1_highbd_jnt_convolve_x_avx2;
-        else if (has_suby == 1)
-            func_tst_ = svt_av1_highbd_jnt_convolve_y_avx2;
-        else
-            func_tst_ = svt_av1_highbd_jnt_convolve_2d_copy_avx2;
+
+        const int fn_idx = TEST_GET_PARAM(3);
+        if (fn_idx == 0) {  // avx2
+            if (has_subx == 1 && has_suby == 1)
+                func_tst_ = svt_av1_highbd_jnt_convolve_2d_avx2;
+            else if (has_subx == 1)
+                func_tst_ = svt_av1_highbd_jnt_convolve_x_avx2;
+            else if (has_suby == 1)
+                func_tst_ = svt_av1_highbd_jnt_convolve_y_avx2;
+            else
+                func_tst_ = svt_av1_highbd_jnt_convolve_2d_copy_avx2;
+        } else {//SSE
+            if (has_subx == 1 && has_suby == 1)
+                func_tst_ = svt_av1_highbd_jnt_convolve_2d_sse4_1;
+            else if (has_subx == 1)
+                func_tst_ = svt_av1_highbd_jnt_convolve_x_sse4_1;
+            else if (has_suby == 1)
+                func_tst_ = svt_av1_highbd_jnt_convolve_y_sse4_1;
+            else
+                func_tst_ = svt_av1_highbd_jnt_convolve_2d_copy_sse4_1;
+        }
 
         bd_ = TEST_GET_PARAM(0);
     }
@@ -993,6 +1034,15 @@ TEST_P(AV1HbdJntConvolve2DTest, DISABLED_SpeedTest) {
     speed_test();
 }
 
+INSTANTIATE_TEST_CASE_P(SSE41_COPY, AV1HbdJntConvolve2DTest,
+                        BuildParams(0, 0, 1, 1));
+INSTANTIATE_TEST_CASE_P(SSE41_ConvolveTest2D, AV1HbdJntConvolve2DTest,
+                        BuildParams(1, 1, 1, 1));
+INSTANTIATE_TEST_CASE_P(SSE41_ConvolveTestX, AV1HbdJntConvolve2DTest,
+                        BuildParams(1, 0, 1, 1));
+INSTANTIATE_TEST_CASE_P(SSE41_ConvolveTestY, AV1HbdJntConvolve2DTest,
+                        BuildParams(0, 1, 1, 1));
+
 INSTANTIATE_TEST_CASE_P(AVX2_COPY, AV1HbdJntConvolve2DTest,
                         BuildParams(0, 0, 0, 1));
 INSTANTIATE_TEST_CASE_P(ConvolveTest2D, AV1HbdJntConvolve2DTest,
@@ -1009,14 +1059,26 @@ class AV1HbdSrConvolve2DTest : public AV1HbdConvolve2DTest {
         func_ref_ = svt_av1_highbd_convolve_2d_sr_c;
         const int has_subx = TEST_GET_PARAM(1);
         const int has_suby = TEST_GET_PARAM(2);
-        if (has_subx == 1 && has_suby == 1)
-            func_tst_ = svt_av1_highbd_convolve_2d_sr_avx2;
-        else if (has_subx == 1)
-            func_tst_ = svt_av1_highbd_convolve_x_sr_avx2;
-        else if (has_suby == 1)
-            func_tst_ = svt_av1_highbd_convolve_y_sr_avx2;
-        else
-            func_tst_ = svt_av1_highbd_convolve_2d_copy_sr_avx2;
+        const int fn_idx = TEST_GET_PARAM(3);
+        if (fn_idx == 0) {  // avx2
+            if (has_subx == 1 && has_suby == 1)
+                func_tst_ = svt_av1_highbd_convolve_2d_sr_avx2;
+            else if (has_subx == 1)
+                func_tst_ = svt_av1_highbd_convolve_x_sr_avx2;
+            else if (has_suby == 1)
+                func_tst_ = svt_av1_highbd_convolve_y_sr_avx2;
+            else
+                func_tst_ = svt_av1_highbd_convolve_2d_copy_sr_avx2;
+        } else {//SSE
+            if (has_subx == 1 && has_suby == 1)
+                func_tst_ = svt_av1_highbd_convolve_2d_sr_ssse3;
+            else if (has_subx == 1)
+                func_tst_ = svt_av1_highbd_convolve_x_sr_ssse3;
+            else if (has_suby == 1)
+                func_tst_ = svt_av1_highbd_convolve_y_sr_ssse3;
+            else
+                func_tst_ = svt_av1_highbd_convolve_2d_copy_sr_ssse3;
+        }
         bd_ = TEST_GET_PARAM(0);
     }
     virtual ~AV1HbdSrConvolve2DTest() {
@@ -1031,6 +1093,15 @@ TEST_P(AV1HbdSrConvolve2DTest, DISABLED_SpeedTest) {
     speed_test();
 }
 
+INSTANTIATE_TEST_CASE_P(SSS3E_ConvolveTestX, AV1HbdSrConvolve2DTest,
+                        BuildParams(1, 0, 1, 1));
+INSTANTIATE_TEST_CASE_P(SSS3E_ConvolveTest2D, AV1HbdSrConvolve2DTest,
+                        BuildParams(1, 1, 1, 1));
+INSTANTIATE_TEST_CASE_P(SSS3E_ConvolveTestY, AV1HbdSrConvolve2DTest,
+                        BuildParams(0, 1, 1, 1));
+INSTANTIATE_TEST_CASE_P(SSS3E_ConvolveTestCopy, AV1HbdSrConvolve2DTest,
+                        BuildParams(0, 0, 1, 1));
+
 INSTANTIATE_TEST_CASE_P(ConvolveTestX, AV1HbdSrConvolve2DTest,
                         BuildParams(1, 0, 0, 1));
 INSTANTIATE_TEST_CASE_P(ConvolveTest2D, AV1HbdSrConvolve2DTest,
