@@ -5,9 +5,10 @@
  * This source code is subject to the terms of the BSD 2 Clause License and
  * the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
  * was not distributed with this source code in the LICENSE file, you can
- * obtain it at https://www.aomedia.org/license/software-license. If the Alliance for Open
- * Media Patent License 1.0 was not distributed with this source code in the
- * PATENTS file, you can obtain it at https://www.aomedia.org/license/patent-license.
+ * obtain it at https://www.aomedia.org/license/software-license. If the
+ * Alliance for Open Media Patent License 1.0 was not distributed with this
+ * source code in the PATENTS file, you can obtain it at
+ * https://www.aomedia.org/license/patent-license.
  */
 
 #include "gtest/gtest.h"
@@ -101,16 +102,6 @@ class QuantizeTest : public ::testing::TestWithParam<ParamType> {
 
     virtual void QuantizeRun(bool is_loop, int q = 0, int test_num = 1) = 0;
 
-    void CompareResults(const TranLow *buf_ref, const TranLow *buf, int size,
-                        const char *text, int q, int number) {
-        int i;
-        for (i = 0; i < size; ++i) {
-            ASSERT_EQ(buf_ref[i], buf[i])
-                << text << " mismatch on test: " << number
-                << " at position: " << i << " Q: " << q;
-        }
-    }
-
     int coeff_num() const {
         return av1_get_max_eob(tx_size_);
     }
@@ -142,11 +133,6 @@ class QuantizeTest : public ::testing::TestWithParam<ParamType> {
         FillCoeff(0);
     }
 
-    void FillCoeffConstant() {
-        TranLow c = GetRandomCoeff();
-        FillCoeff(c);
-    }
-
     void FillDcOnly() {
         FillCoeffZero();
         coeff_[0] = GetRandomCoeff();
@@ -599,6 +585,16 @@ using std::make_tuple;
 
 #if HAS_AVX2
 const QuantizeParam kQParamArrayAvx2[] = {
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_sse4_1,
+               static_cast<TxSize>(TX_16X16), TYPE_FP, AOM_BITS_8),
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_sse4_1,
+               static_cast<TxSize>(TX_4X16), TYPE_FP, AOM_BITS_8),
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_sse4_1,
+               static_cast<TxSize>(TX_16X4), TYPE_FP, AOM_BITS_8),
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_sse4_1,
+               static_cast<TxSize>(TX_32X8), TYPE_FP, AOM_BITS_8),
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_sse4_1,
+               static_cast<TxSize>(TX_8X32), TYPE_FP, AOM_BITS_8),
     make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_avx2,
                static_cast<TxSize>(TX_16X16), TYPE_FP, AOM_BITS_8),
     make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_avx2,
